#/*  #Copyright 2011 Cristina Bosco [bosco] AT di DOT unito DOT it

    #This file is part of TUTtoPENN converter re. 1.0.

    #TUTtoPENN converter is free software: you can redistribute it and/or modify
    #it under the terms of the GNU General Public License as published by
    #the Free Software Foundation, either version 3 of the License, or
    #(at your option) any later version.

    #TUTtoPENN converter is distributed in the hope that it will be useful,
    #but WITHOUT ANY WARRANTY; without even the implied warranty of
    #MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    #GNU General Public License for more details.

    #You should have received a copy of the GNU General Public License
    #along with TUTtoPENN converter. If not, see <http://www.gnu.org/licenses/>.
#*/


#This file is part of the TUTtoPENN converter, which implements a conversion from TUT 
#native format to Penn Treebank format adapted to Italian (called TUT-Penn).
#The TUTtoPENN converter User MANUAL, which can be dowloaded from 
#http://www.di.unito.it/~tutreeb/TUTtoPENN/TUTtoPENNconverter-manual.zip), describes 
#the organization and use of the converter.
#
#This is the THIRD MODULE, i.e. the module that is called to convert the constituency 
#syntactic structures of the CONStituency-TUT into Penn structures with the 
#functional syntactic relations of TUT annotated on constituents, i.e. the 
#AUGmented-Penn. The input of this module is therefore in CONStituency-TUT format, i.e.
#the output of the SECOND module constuttifier and is in the file 'outconstut.out'.
#The call of this module is done by the MAIN module after the preprocessing (FIRST module) 
#and the constuttifier (SECOND module), if the parameter is -a or -p.
#The output of this module are sentences in form of array and in AUGmented-Penn format, 
#and is in the file called 'outaugpenn.out'.

#The main comments are in English, the minor in Italian

package package_augpen;
BEGIN{}


### main_augpen #############################################################
#manages the call of the rest of the module on the single sentences included in the input file

sub main_augpen{
	print "in augpennifier\n";
	$contarighe = 0; 		#contiene il numero righe del file ancora da leggere
	@frase_augpen = ();		#conterra' l'albero augpen
	$frase_augpen = '';
	$file_frasi_augpen = '';
	$titolo = '';
	$contarighe = scalar(@_);	#numero di righe del file di input
	while ($contarighe > 0){        #fino a quando ci sono righe di input
			$riga = shift(@_);
			@frase_augpen = ();
			$contarighe = --$contarighe;
			while ($riga !~ m/^\s/){		#se la riga non inizia con carattere vuoto
					$contarighe = --$contarighe;
					if ($riga =~ m/\*\*\*\*/){  #se la riga contiene gli *
							$titolo = $riga;     			#assegna riga a $titolo
					}
					else{if ($riga =~ m/^\(/){        #se la riga inizia con parentesi aperta
								push(@frase_augpen,$riga);	#la mette nell'array
	     	    		  }
					}
					$riga = shift(@_);
			}	
			$file_frasi_augpen = $file_frasi_augpen."\n".$titolo.aug_pennify(@frase_augpen);
			print $titolo,"\n";
			$frase_augpen = '';
	}
	$file_frasi_augpen =~ s/^\n//;
	return $file_frasi_augpen."\n";
}


### AUG-PENNIFY ###########################################################
#manages the call of the procedures that convert the sentences in AUGmented-Penn format

sub aug_pennify{
	my @frase_input = @_;
	my @frase_augpen = ();
	my $trovato = 0;
	my $root = '';
	while (scalar(@frase_input) > 0){
			$riga = shift(@frase_input);
			push (@frase_augpen,$riga);
	}
	foreach $it(@frase_augpen){	
			if ($it =~ m/ADJP2\-RMOD/){
					@frase_augpen = flat_prdt($it,padre_in_quadre($it),@frase_augpen);
			}
	}
	foreach $it(@frase_augpen){		#elimina le tracce degli intramod e aggiorna i mod stessi
			if ($it =~ m/\-INTRAMD/){
					@frase_augpen = find_update_coindexed($it,@frase_augpen);
			}
	}
	foreach $it(@frase_augpen){		
			if ($it =~ m/EXTRAPOSEDSUBJ/){
					@frase_augpen = find_update_extraposedsubj($it,@frase_augpen);
			}
	}
	foreach $it(@frase_augpen){
			if ((substr $it,0,(index $it,' ')) =~ m/bar/){
					@frase_augpen = remove_Xbar_from($it,@frase_augpen);
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if (((substr $it,0,(index $it,' ')) =~ m/\(ADJP/) or ((substr $it,0,(index $it,' ')) =~ m/\(ADVP/)){
					@frase_augpen = flat_ADP($it,@frase_augpen);				
			}
	}
	foreach $it(@frase_augpen){
			if ((((substr $it,0,(index $it,' ')) =~ m/\(PPSUBJ/) or ((substr $it,0,(index $it,' ')) =~ m/\(PP-OBJ/) or
				((substr $it,0,(index $it,' ')) =~ m/\(PP-INDOBJ/)) and (padre($it,@frase_augpen) !~ m/VP/)){
						@frase_augpen = standardize_NOUN_ARG($it,@frase_augpen);			
			}	
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if ((substr $it,0,(index $it,' ')) =~ m/\(DP/){
					@frase_augpen = del_and_flat_DP($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if ((substr $it,0,(index $it,' ')) =~ m/\(CONJP/){
					@frase_augpen = flat_CONJP($it,@frase_augpen);	
					@frasetemp = @frase_augpen;
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if ((substr $it,0,(index $it,' ')) =~ m/\(NP/){
					@frase_augpen = flat_doubleNP($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if (((substr $it,0,(index $it,' ')) =~ m/\(NOU\~PR/) 
				and (join ("", fratelli($it,@frase_augpen)) !~ m/ART~/)){	#per escludere casi come alb28
							@frase_augpen = flat_NPnames($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if (((substr $it,0,(index $it,' ')) =~ m/\(Srelclbbaarr/) or 
			((substr $it,0,(index $it,' ')) =~ m/\(Srespecialclbbaarr/)){
					@frase_augpen = arrange_relclause($it,@frase_augpen);				
			}
	}
	foreach $word(@frase_augpen){		#gestione del caso di root della frase (=nodo top) di tipo Xbar (es. frase alb6)
			if (($word !~ m/]/) and ($word =~ m/bar\s/)){
						$newword = $word;
						$newword =~ s/bar/P/;
						@frase_augpen = sostituisciinarray($word,$newword,@frase_augpen);
			} 
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){
			$it = shift(@frasetemp);
			if ((substr $it,0,(index $it,' ')) =~ m/\(VP/){
					@frase_augpen = rotate_aux_structure($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){		#gestisce l'appiattimento delle strutture bilanciate con quotes
			$it = shift(@frasetemp);
			if ((substr $it,0,(index $it,' ')) =~ m/\(PUNCT\-OPEN\+QUOTES/){
					@frase_augpen = flat_quoted($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){		#gestisce la rimozione delle LOCUT dalle locuzioni flessibili
			$it = shift(@frasetemp);
			if ($it =~ m/\*LOCUT/){
					@frase_augpen = remove_LOCUT($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){		#gestisce l'appiattimento dei DP residui
			$it = shift(@frasetemp);
			if ($it =~ m/\(DP/){
					@frase_augpen = lastflat_DP($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){		#gestisce la rimozione degli EXTRAPOSED e relative tracce
			$it = shift(@frasetemp);
			if ($it =~ m/EXTRAPOSED/){
					@frase_augpen = remove_EXTRAPOSITION($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){		
			$it = shift(@frasetemp);
			if ($it =~ m/\-LGS/){
					@frase_augpen = remove_tracedLGS($it,@frase_augpen);				
			}
	}
	@frasetemp = @frase_augpen;
	while (scalar(@frasetemp) > 0){		
			$it = shift(@frasetemp);
			if ($it =~ m/\*/){
					@frase_augpen = modify_tracing($it,@frase_augpen);	
			}
	}
	return join("",@frase_augpen);
}



### flat_prdt ###########################################################
#makes flat like in Penn format the structures that include a predeterminer

sub flat_prdt{
		my $prdt = shift(@_);
		my $padre = shift(@_);
		my @frase = @_;
		my @figliprdt = figliconstit($prdt,@frase);
		if (($prdt =~ m/ADJP2\-RMOD/) or ($prdt =~ m/ADJbar2/)){
				if  (scalar(@figliprdt ==1)){		#se ha un solo figlio
						@frase = cancelladaarray($prdt,@frase);
						foreach $fi(@figliprdt){
								@frase = flat_prdt($fi,$padre,@frase);
						}
				}
				else{if ($prdt =~ m/ADJbar/){	#se ha piu' di un figlio ed e' proiezione intermedia ADJbar 
								foreach $fi(@figliprdt){	#aggancio i figli sotto la sua proiezione massimale		
										$newfi = (substr $fi, 0,(rindex $fi, '[')+1).padre_in_quadre($prdt)."]\n";
										@frase = sostituisciinarray($fi,$newfi,@frase);
										@frase = flat_prdt($fi,$padre,@frase);
								}
								@frase = cancelladaarray($prdt,@frase);			#lo cancello
						}
						else{foreach $fi(@figliprdt){ #se ha piu' di un figlio ed e' proiezione massimale ADJP
										$padre = pos_in_ttree($prdt);
										@frase = flat_prdt($fi,$padre,@frase);
								}
					   }
				}
		}
		elsif ($prdt =~ m/PRDT/){				
				$newprdt = (substr $prdt, 0,(rindex $prdt, '[')+1).$padre."]\n";
				@frase = sostituisciinarray($prdt,$newprdt,@frase);
		}
		return @frase;
}



###remove_tracedLGS##########################################################
#deletes the traces for the deep subject annotated for verbs in TUT

sub remove_tracedLGS{
	my $lgs = shift(@_);
	my @tree = @_;
	my @lgs_figli = figliconstit($lgs,@tree); #se un lgs ha solo un figlio che e' traccia oppure non ha figli allora va rimosso
	if ((scalar(@lgs_figli) == 0) or ((scalar(@lgs_figli) == 1) and (@lgs_figli[0] =~ m/\*\[/))){
			@tree = cancelladaarray($lgs,@tree);
	}
	return @tree;
}



#### remove_EXTRAPOSITION ####################################################
#In CONStituency-TUT are considered as extraposed the subject which are on the right of the verb or
#separated from the verb by some modifier; instead in AUGmented-Penn are considered as extraposed 
#only the subjects positioned after the verb. This procedures removes the traces generated in CONStituency-TUT 
#foe all the others

sub remove_EXTRAPOSITION{
	my $extraposed = shift(@_);
	my @sentence = @_;
	my $fatherofextraposed = padre($extraposed,@sentence);
	my $phrase_extraposed = nome_phrase($extraposed);
	my $tracedsubj = '';
	my $tracecoindex = '';
	my $coindex = '';
	my @figli_extraposed = ();
	if (($extraposed !~ m/\*/) and ($extraposed !~ m/\*\[/)){ #se extraposed non e' coindicizzante
			@figli_extraposed = figliconstit($extraposed,@sentence);
			if (join(" ",@figli_extraposed) =~ m/CONJ/){#se il suo subtree contiene un coindicizzante
					foreach $figlio(@figli_extraposed){
							if (($figlio !~ m/CONJ/) and (nome_phrase($figlio) =~ m/$phrase_extraposed/) and 
								($figlio =~ m/\*/)){#cerco il coindicizzante nel subtree di extraposed		
									$figlio_coindexed = $figlio;
							}
					}
			}#sposto il coindice dal nodo coindicizzante ad extraposed
			$coindex = substr $figlio_coindexed,(index $figlio_coindexed,'*')+1;
			$coindex = substr $coindex,0,(index $coindex,' ');
			$newfiglio_coindexed = $figlio_coindexed;
			$newfiglio_coindexed =~ s/\*$coindex//;
			@sentence = sostituisciinarray($figlio_coindexed,$newfiglio_coindexed,@sentence);
			$coindexing_extraposed = (substr $extraposed,0,(index $extraposed,' ')).'*'.$coindex.' '.
												   (substr $extraposed,(index $extraposed,' ')+1);
			@sentence = sostituisciinarray($extraposed,$coindexing_extraposed,@sentence);
			$extraposed = $coindexing_extraposed;
	}
	if ((spina($fatherofextraposed) > spina($extraposed)) and ($fatherofextraposed !~ m/\(VP/)){
			$coindex = substr $extraposed,(index $extraposed,'*')+1;
			$coindex = substr $coindex,0,(index $coindex,' ');
			foreach $item(@sentence){
					if (($item =~ m/\*\[/) and ($item !~ m/\*\[\]/)){	#se e' una traccia coindicizzata
							$tracecoindex = substr $item,(index $item,'*')+2;		#calcola il coindice
							$tracecoindex = substr $tracecoindex,0,(index $tracecoindex,']');
							if ($tracecoindex eq $coindex){	#se il coindice corrisponde con quello di extraposed
									$tracedsubj = $item;
							}
					}
			}
			@sentence = cancelladaarray($tracedsubj,@sentence);
			$newextraposed = $extraposed;
			$newextraposed =~ s/EXTRAPOSED//;
			if (join("",@sentence) !~ m/\*\[$coindex/){
					$newextraposed = (substr $newextraposed,0,(index $newextraposed,'*')).(substr $newextraposed,(index $newextraposed,' '));
			}
			@sentence = sostituisciinarray($extraposed,$newextraposed,@sentence);
	}
	return @sentence;
}



#### lastflat_DP ####################################################
#makes flat the structure of residual DP

sub lastflat_DP{
my $dparg = shift(@_);
my @fra = @_;
my @figlidp = figliconstit($dparg,@fra);
my $det = '';
my $np = '';
my $pp = '';
my $predeterminer = '';
my $npnep = '';
my $ppnep = '';
my $vpnep = '';
my $newpredeterminer = '';
my $conj = '';
foreach $son(@figlidp){
		if (($son =~ m/\(ART\~DE/) or ($son =~ m/\(ART\~IN/) or ($son =~ m/\(ADJ\~DE/) or 
		($son =~ m/\(ADJ\~DI2/) or ($son =~ m/\(ADJ\~IN/) or ($son =~ m/\(ADJ\~IR/) or 
		($son =~ m/\(NUMR2/)){
				$det = $son;
		}
		elsif ($son =~ m/\(NP/){
				$np = $son;
		}
		elsif ($son =~ m/\(PP/){
				$pp = $son;
		}
		elsif ($son =~ m/\(CONJ/){
				$conj = $son;
		}
		elsif ($son =~ m/PRDT/){
				$predeterminer = $son;
		}
}
if (($det ne '') and ($det =~ m/ADJ\~DI2/)){
		$newdparg = $dparg;
		$newdparg =~ s/DP/NP/;
		@fra = sostituisciinarray($dparg,$newdparg,@fra);
}
if (($det ne '') and ($np ne '')){	#se c'e' articolo e np
		@figlinp = figliconstit($np,@fra);
		foreach $ff(@figlinp){		#se np ha un figlio np ed uno pp
				if ($ff =~ m/\(NP/){
						$npnep = $ff;
				}
				elsif ($ff =~ m/\(PP/){
						$ppnep = $ff;
				}
				elsif ($ff =~ m/\(VP/){
						$vpnep = $ff;
				}
		}
		if (($npnep ne '') and (($ppnep ne '') or ($vpnep ne ''))){
				$newnp = substr $np,0,(index $np,'[')+1;
				$newnp = $newnp.padre_in_quadre($dparg).']'."\n";
				$newdet = substr $det,0,(index $det,'[')+1;
				$newdet = $newdet.pos_in_ttree($np).']'."\n";
				@fra = cancelladaarray($dparg,@fra);
				@fra = sostituisciinarray($np,$newnp,@fra);
				@fra = sostituisciinarray($det,$newdet,@fra);	
		}
}
elsif ((($np ne '') and ($pp ne '')) or (($np ne '') and ($conj ne ''))){
		$newdparg = $dparg;
		$newdparg =~ s/\(DP/\(NP/;
		@fra = sostituisciinarray($dparg,$newdparg,@fra);
				if ($predeterminer ne ''){
					$newpredeterminer = substr $predeterminer,0,(index $predeterminer,'[')+1;
					$newpredeterminer = $newpredeterminer.pos_in_ttree($np).']'."\n";
					@fra = sostituisciinarray($predeterminer,$newpredeterminer,@fra);
				@fra = cancelladaarray($dparg,@fra);		
		}
}
foreach $ele(@fra){
		if ($ele =~ m/\(DP/){
				$newele = $ele;
				$newele =~ s/\(DP/\(NP/;
				@fra = sostituisciinarray($ele,$newele,@fra);	
		}
}
return @fra;
}


#### remove_LOCUT ####################################################
#remove the relation *LOCUT that is used in TUT and CONStituency-TUT to mark a particular kind (flexible)
#locutive expressions which are not marked in AUGmented-Penn

sub remove_LOCUT{
	my $arg = shift(@_);
	my @sent = @_;
	my $newarg = $arg;
	$newarg =~ s/\*LOCUT//;
	my $beginarg = substr $newarg,1,(index $newarg,' ');
	my $beginarg = nome_phrase($newarg);
	if (($beginarg =~ m/NP/) or ($beginarg =~ m/VP/) or ($beginarg =~ m/ADJP/) or 
		($beginarg =~ m/ADVP/) or ($beginarg =~ m/PP/)){
				if ($newarg !~ m/\*LOCUT/){		#se necessario devono contenere locut solo le proiezioni massimali
						$newarg = $arg;
				}
				else{
						@sent = remove_LOCUT($newarg,@sent);
				}
	}
	else{
			$newarg = $newarg;
	}
	@sent = sostituisciinarray($arg,$newarg,@sent);
return @sent;
}


###flat_quoted ##########################################
#In CONStituency-TUT the balanced quoted structures are represented as follows:
#(XP (quote-open) (XP ...) (quote-close)) with double XP;
#in Penn instead the structure for such a kind of structures is as follows: 
#(quote-open) (XP ...) (quote-close);
#This procedure deletes the duplicated XP

sub flat_quoted{
	my $openquot = shift(@_);
	my @sent = @_;
	my $closequot = '';
	my $fatherquotes = '';
	my @brothersquotes = ();
	my $brother = '';
	my $phrasebrother = '';
	my $phrasefatherquotes = '';
	my $relazfatherquotes = '';
	$fatherquotes = padre($openquot,@sent);	
	foreach $wo(figliconstit($fatherquotes,@sent)){
			if ($wo =~ m/\(PUNCT\-CLOSE\+QUOTES/){
					$closequot = $wo;
			}
	}
	@figlifatherquotes = figliconstit($fatherquotes,@sent);	#cerco i fratelli di open e close (dovrebbe essercene uno solo)
	@brotherquotes = fratelli($openquot,@sent);
	$relazfatherquotes = substr $fatherquotes,1,(index $fatherquotes,' ');	#cerco la relazione di father
	$phrasefatherquotes = nome_phrase($fatherquotes);
	$relazfatherquotes =~ s/$phrasefatherquotes//;
	if ((scalar(@brotherquotes) == 1) and (nome_phrase(@brotherquotes[0]) eq nome_phrase($fatherquotes))){
			foreach $item(@figlifatherquotes){
					if (nome_phrase($item) eq nome_phrase($fatherquotes)){
							foreach $it(figliconstit($item,@sent)){
									$newit = substr $it,0,(index $it,'[')+1;
									$newit = $newit.padre_in_quadre($fatherquotes).']'."\n";
									@sent = sostituisciinarray($it,$newit,@sent);
							}
					}
					else{	$newitem = substr $item,0,(index $item,'[')+1;
							$newitem = $newitem.padre_in_quadre($fatherquotes).']'."\n";
							@sent = sostituisciinarray($item,$newitem,@sent);
					}
					@sent = cancelladaarray($item,@sent);
			}
	}
	return @sent;
}



### arrange_relclause ##############################################
#In CONStituency-TUT non reduced relative clauses are represented as follows:
#(XP (Xbar (Xbar (...)) (S-RMOD-RELCL (NP-SUBJ ...) (Sbar (VP ...)))),
#while for the reduced relative clauses is used the following pattern:
#(XP*1 (Xbar (Xbar (...)) (S-RMOD-RELCL+REDUC  (NP-SUBJ*[1]) (Sbar (VP ...))))
#In Penn instead non reduced relative clauses are represented as follows:
#(XP (XP (...)) (SBAR (NP*1) (S (NP-SBJ*[1]) (VP ...))))
#while for the reduced relative clauses is used the following pattern:
#(XP (XP (...)) (VP (...) (NP traccia di null) (...)))
#with the exception of the reduced relative clauses with missing VP, which are represented as follows:
#(XP (XP (...)) (RRC (...))) where all the dependents of VP depend on the RRC.
#This procedure apply the conversion of relative clauses.

sub arrange_relclause{
	my $anchorage = shift(@_);
	my @sentence = @_;
	my $padre_anchorage = '';
	my $newpadre_anchorage = '';
	my @figli_padre_anchorage = ();
	my $newanchorage = '';
	my $subjrelativa = '';
	my $newsubjrelativa = '';
	my $newfigliosubjrelativa = '';
	my $vprelativa = '';
	my $newvprelativa = '';
	my @figlisubjrelativa = ();
	my $pronrelativo = '';
	$padre_anchorage = padre($anchorage,@sentence);
	@figli_padre_anchorage = figliconstit($padre_anchorage,@sentence);
	foreach $ff(@figli_padre_anchorage){
			if (($ff =~ m/\(NPSUBJ/) or ()){	#trova NP soggetto  della relativa 
					$subjrelativa = $ff;
			}
			elsif ($ff =~ m/\(VP/){		#trova il VP della relativa
					$vprelativa = $ff;
			}	
	}
###############trattamento relative non ridotte
	if ((($padre_anchorage =~ m/RELCL/) and ($padre_anchorage !~ m/\+REDUC/))
		or ($anchorage =~ m/Srespecialclbbaarr/)){	#trattamento relative non ridotte
				$newpadre_anchorage = $padre_anchorage;	#rinomina S-RELCL in SBAR
				$newpadre_anchorage =~ s/S\-RMOD\+RELCL/SBAR/;		
				@sentence = sostituisciinarray($padre_anchorage,$newpadre_anchorage,@sentence);
				$newanchorage = $anchorage; 			#rinomina anchorage in S
				$newanchorage =~ s/relclbbaarr//;
				if ($anchorage =~ m/Srespecialclbbaarr/){
						$newanchorage =~ s/respecialclbbaarr//;
				}
				@sentence = sostituisciinarray($anchorage,$newanchorage,@sentence);
				$pos = pos_in_ttree($subjrelativa);			#rinomina il soggetto in NPREL
				$pos = substr $pos,0,1;
				$pos = $pos.'333';
				$newsubjrelativa = $subjrelativa;
				$newsubjrelativa =~ s/NPSUBJ/NP\*$pos/;
				@figlisubjrelativa = figliconstit($subjrelativa,@sentence);
				$newpronrelativo = @figlisubjrelativa[0];
				$newpronrelativo =~ s/SUBJ//;
				@sentence = sostituisciinarray(@figlisubjrelativa[0],$newpronrelativo,@sentence);
				if ($subjrelativa !~ m/\*\[/){ #e' il caso di 'che'
						@sentence = sostituisciinarray($subjrelativa,$newsubjrelativa,@sentence);
						#crea una traccia di NPREL ovvero 1) associa un indice a NPREL e 2)crea un nuovo nodo coindicizzato con NPREL
						$trace_subjrelativa = substr $subjrelativa,0,(index $subjrelativa,'[');
						$trace_subjrelativa =~ s/SUBJ/SUBJ\*\[$pos\]/;
						$trace_subjrelativa = $trace_subjrelativa.'['.(pos_in_ttree($newanchorage)).']';
						$trace_subjrelativa = (substr $trace_subjrelativa,0,(index $trace_subjrelativa,'/')).
														'33'.(substr $trace_subjrelativa,(index $trace_subjrelativa,'/'))."\n";
				}
				else{ #e' il caso di 'chi'
						@sentence = cancelladaarray($subjrelativa,@sentence);
						$trace_subjrelativa = substr $subjrelativa,0,(rindex $subjrelativa,'[');
						$trace_subjrelativa = $trace_subjrelativa.'['.(pos_in_ttree($newanchorage)).']';
						$trace_subjrelativa = (substr $trace_subjrelativa,0,(index $trace_subjrelativa,'/')).
														'33'.(substr $trace_subjrelativa,(index $trace_subjrelativa,'/'))."\n";
				}
				push(@sentence,$trace_subjrelativa);
				#attacca il VP e la traccia sotto anchorage
				$newvprelativa = substr $vprelativa,0,(rindex $vprelativa,'[');
				$newvprelativa = $newvprelativa.'['.(pos_in_ttree($newanchorage)).']'."\n";	
				@sentence = sostituisciinarray($vprelativa,$newvprelativa,@sentence);
		} 
############trattamento della relativa ridotta
		else{		
				if ($padre_anchorage =~ m/RELCL\+REDUC/){
						if ($vprelativa ne ''){
								if ($vprelativa !~ m/\*/){	#se il VP esiste costruisco il nuovo VP
										$newvprelativa = substr $vprelativa,0,(index $vprelativa,'[');
										$newvprelativa =$newvprelativa.'['.padre_in_quadre($padre_anchorage).']'."\n";
							#costruisco il nuovo soggetto che ha la forma (NP (-NONE- *))
										$newsubjrelativa = '(NP '.pos_in_ttree($subjrelativa).'/)['.pos_in_ttree($vprelativa).']'."\n";
										$figlionewsubjrelativa = (pos_in_spina($subjrelativa))-1;
										$figlionewsubjrelativa = spina($subjrelativa).'-'.$figlionewsubjrelativa;
										$figlionewsubjrelativa = '(NONE '.$figlionewsubjrelativa.'/)['.pos_in_ttree($subjrelativa).']'."\n";
										push(@sentence,$figlionewsubjrelativa);
										@sentence = cancelladaarray($anchorage,@sentence);
										@sentence = cancelladaarray($padre_anchorage,@sentence);
							#elimina i coindici che servivano alle tracce delle relative, ormai cancellate
										if ($subjrelativa =~ m/\*\[/){		#se il subj cancellato era una traccia
												$coindicesubj = substr $subjrelativa,0,(index $subjrelativa,']');
												$coindicesubj = substr $coindicesubj,(index $coindicesubj,'*[')+2;
												foreach $item(@sentence){
														if (($item =~ m/$coindicesubj/) and ($item !~ m/\*\[/)){
																$newitem = $item;
																$newitem =~ s/$coindicesubj//;
																$newitem =~ s/\*//;
																@sentence = sostituisciinarray($item,$newitem,@sentence);
														}
												}
										}
										foreach $hh(figliconstit($padre_anchorage,@sentence)){
												if (($hh ne $subjrelativa) and ($hh ne $vprelativa)){
														$newhh = substr $hh,0,(index $hh,'[')+1;
														$newhh = $newhh.pos_in_ttree($newvprelativa).']'."\n";#print $newhh," sotto newvprelativa",$newvprelativa,"\n";
														@sentence = sostituisciinarray($hh,$newhh,@sentence);
												}
										}
								}
								else{print "cucu\n";	#Gestione del caso specifico di ALB18
								#costruisco il nuovo VP
										$newvprelativa = substr $vprelativa,0,(rindex $vprelativa,'[')+1;
										$newvprelativa =$newvprelativa.padre_in_quadre($anchorage).']'."\n";
										@sentence = cancelladaarray($anchorage,@sentence);
										@fratelli_padre_anchor = fratelli($padre_anchorage,@sentence);
										foreach $yy(fratelli($padre_anchorage,@sentence)){
												if (nome_phrase($yy) eq nome_phrase(padre($padre_anchorage,@sentence))){
														foreach $tt(figliconstit($yy,@sentence)){
																$newtt = substr $tt,0,(rindex $tt,'[')+1;
																$newtt = $newtt.padre_in_quadre($padre_anchorage).']'."\n";
																@sentence = sostituisciinarray($tt,$newtt,@sentence);
														}
														@sentence = cancelladaarray($yy,@sentence);
												}
										}
								}
						}
				}	
				else{
						if (($padre_anchorage =~ m/\(S\s/) and ($padre_anchorage !~ m/RELCL/)){
								$nonno_anchorage = padre($padre_anchorage,@sentence);
								$newnonno_anchorage = $nonno_anchorage;
								$newnonno_anchorage =~ s/S\-RMOD\+RELCL\+REDUC/VP/;
								@sentence = sostituisciinarray($nonno_anchorage,$newnonno_anchorage,@sentence);
								@fratelli_anchorage = fratelli($padre_anchorage,@sentence);
								if (($nonno_anchorage =~ m/RELCL\+REDUC/) and 
									(join(" ",@fratelli_anchorage) =~ m/COORD2ND/)){
										$new_padreanchorage = '(S-RMOD+RELCL+REDUC'.
																			(substr $padre_anchorage,(index $padre_anchorage,' '));
										@sentence = sostituisciinarray($padre_anchorage,$new_padreanchorage,@sentence);
										foreach $fratello(@fratelli_anchorage){
												if ($fratello =~ m/COORD2ND/){
														$newcoordfratello =  '(S-RMOD+RELCL+REDUC'.(substr $fratello,(index $fratello,' '));
														@sentence = sostituisciinarray($fratello,$newcoordfratello,@sentence);
												}
										}
										$coordanchorage = substr $anchorage,0,(index $anchorage,' ');
										if (join(" ",figliconstit($newcoordfratello,@sentence)) !~ m/S\-COORD2ND/){
												$coordanchorage = $coordanchorage.' 150-5/)['.pos_in_ttree($newcoordfratello).']';
												push(@sentence,$coordanchorage);
										}
										else{	$newnewcoordfratello = $newcoordfratello;
												$newnewcoordfratello =~ s/S\-RMOD\+RELCL\+REDUC/VP/;
												@sentence = sostituisciinarray($newcoordfratello,$newnewcoordfratello,@sentence);
												foreach $ff(figliconstit($newcoordfratello,@sentence)){
														if ($ff =~ m/COORD2ND/){
																$newcoordff =  '(S-RMOD+RELCL+REDUC'.(substr $ff,(index $ff,' '));
																@sentence = sostituisciinarray($ff,$newcoordff,@sentence);
																$newsecondanchorage = $coordanchorage.' 160-5/)['.pos_in_ttree($newcoordff).']';
																push(@sentence,$newsecondanchorage);
														}
														else{if (($ff !~ m/COORD2ND/) and ($ff =~ m/S\s/)){
																		$newff = $ff;
																		$newff =  '(S-RMOD+RELCL+REDUC'.(substr $ff,(index $ff,' '));
																		@sentence = sostituisciinarray($ff,$newff,@sentence);		
																		$newfirstanchorage = $coordanchorage.' 170-5/)['.pos_in_ttree($newff).']';
																		push(@sentence,$newfirstanchorage);
																}
														}
												}
												@sentence = arrange_relclause($newsecondanchorage,@sentence);
												@sentence = arrange_relclause($newfirstanchorage,@sentence);	
										}
									
										@sentence = arrange_relclause($coordanchorage,@sentence);
										@sentence = arrange_relclause($anchorage,@sentence);
								}
							
						}
			}
	}
	@sentence = sostituisciinarray($subjrelativa,$newsubjrelativa,@sentence);
	@sentence = sostituisciinarray($vprelativa,$newvprelativa,@sentence);
	return @sentence;
}



### flat_doubleNP #####################################################
#Makes flat the structures where in CONStituency-TUT a double NP occurs, deleting it

sub flat_doubleNP{
	my $np = shift(@_);
	my @sent = @_;
	my @figli_np = ();
	my @figli_npfiglio = ();
	@figli_np = figliconstit($np,@sent);
	foreach $ee(@figli_np){
			if ($ee =~ m/\(PUNCT/){
					@figli_np = cancelladaarray($ee,@figli_np);
			}
	}
	if (scalar(@figli_np) ==1){	#se ha un figlio
			if (@figli_np[0] =~ m/^\(NP/){	#se il figlio e' un NP
					@figli_figli_np = figliconstit(@figli_np[0],@sent);
					foreach $ff(@figli_figli_np){
							@sent = one_level_up($ff,@figli_np[0],@sent);
					}
					@sent = cancelladaarray(@figli_np[0],@sent);
			}
	}
	elsif (scalar(@figli_np) >=2){#se ha due figli,
				$npfiglio = '';
				$numfiglio = '';
				foreach $yy(@figli_np){	#verifica se tra questi compaiono un NP e un NUMR
						if ($yy=~ m/^\(NP/){
								$npfiglio = $yy;
						}
						elsif ($yy=~ m/^\(NUMR2/){
									$numfiglio = $yy;
					   }
				}
				if (($npfiglio ne '') and ($numfiglio ne '')){	#se esistono sia num che npfiglio
						$newnumfiglio = substr $numfiglio,0,(index $numfiglio,'[')+1;	#anche num viene inserito in npfiglio
						$newnumfiglio = $newnumfiglio.pos_in_ttree($npfiglio).']'."\n";
						@sent = sostituisciinarray($numfiglio,$newnumfiglio,@sent);
				}	
				if (($npfiglio ne '') and ($advbfiglio ne '')){	#se esistono sia advb che npfiglio
						$newadvbfiglio = substr $advbfiglio,0,(index $advbfiglio,'[')+1;	#anche advb viene inserito in npfiglio
						$newadvbfiglio = $newadvbfiglio.pos_in_ttree($npfiglio).']'."\n";
						@sent = sostituisciinarray($advbfiglio,$newadvbfiglio,@sent);
				}	
	}
	return @sent;
}


### flat_doubleNP #####################################################
#Makes flat the structures where in CONStituency-TUT a double NP occurs which is referred to a name, deleting it

sub flat_NPnames{
	my $propernoun = shift(@_);
	my @sent = @_;
	my $padre_propernoun = '';
	my $nonno_propernoun = '';
	my $terminali = '';
	$padre_propernoun = padre($propernoun,@sent);
	$nonno_propernoun = padre($padre_propernoun,@sent);
	@figli_padre	= figliconstit($padre_propernoun,@sent);
	foreach $el(@figli_padre){
			if ($el ne $propernoun){
					if ($el =~ m/\/\)\[/){		#se il nomeproprio ha fratelli terminali 
								$terminali = $terminali.'si';
					}
					elsif ($el !~ m/\/\)\[/){$terminali = 'no';
					}
			}
	}
	@figlio_nonno = figliconstit($nonno_propernoun,@sent);
	if (($padre_propernoun =~ m/^\(NP/) and 
	    ($nonno_propernoun =~ m/^\(NP/)){
				if ((scalar(@figli_padre) == 1) and ($terminali !~ m/no/)){
						@sent = one_level_up($propernoun,padre($propernoun,@sent),@sent);
						@sent = cancelladaarray(padre($propernoun,@sent),@sent);
				}
	}
	return @sent;
}



## flat_CONJP ##############################################
#In CONStituency-TUT the coordination is introduced by CONJP and represented according the following pattern:
#(XP (XP-firstconjunct) (CONJP (conjunction) (XP-secondconjunct)))
#instead in penn: (XP (XP-firstconjunct) (conjunction) (XP-secondconjunct)) 
#where the XP of each conjuct is duplicated and the CONJP does not exist;
#The exception is the temporal coordination whici is represented in the same way in both formats,
#but in penn rather than CONJP an SBAR.
#Moreover, in CONStituency-TUT when CONJP is the tree root (and one of the conjuncts is missing), then the representation is 
#(CONJP (conjunction) (XP-secondconjuct)),
#while in penn is (XP (conjunction) (XP-secondconjunct)).
#This procedure applies the conversion of coordinative structures.

sub flat_CONJP{
	my $conjp = shift(@_);
	my @sentence = @_;
	my $head = '';
	my $newhead = '';
	my $newconjp = '';
	my $conjunction = '';
	my $newconjunction = '';
	my $punct = '';
	my $newpunct = '';
	if ($conjp =~ m/\]/){		#se CONJP non e' il nodo root della frase
#trattamento delle coordinazioni particolari come temporali ecc
			if (((substr $conjp,0,(index $conjp,' ')) =~ m/\-/) and ($conjp !~ m/\COORD/) and 
				($conjp !~ m/CONJP/)){
					$newconjp = $conjp;
					$newconjp =~ s/CONJP/SBAR/;
					@sentence = sostituisciinarray($conjp,$newconjp,@sentence);
			}
#trattamento di tutte le altre coordinazioni
			else{@figliconjp = figliconstit($conjp,@sentence);
					foreach $fi(@figliconjp){
							@sentence = one_level_up($fi,$conjp,@sentence);
					}
			}
	}
	else{@figliconjp = figliconstit($conjp,@sentence); 		#se CONJP e' nodo root della frase
			foreach $son(@figliconjp){		#cerca la congiunzione
					if ($son =~ m/\(CONJ/){
							$conjunction = $son;
					}
					else{if ($son =~ m/\(PUNCT/){	#cerca 
							$punct = $son;
							}
							else{if (($son !~ m/\(CONJ/) and ($son !~ m/\(PUNCT/) and ($son !~ m/VISITOR/)){
									$head = $son;
									}
									else{$otherson = $son;
									}
							}
					}
			}
			$newconjunction = $conjunction;
			$newconjunction = (substr $newconjunction,0,(index $newconjunction,'[')+1).(pos_in_ttree($head)).']'."\n";
			@sentence = sostituisciinarray($conjunction,$newconjunction,@sentence);
			if ($punct ne ''){
					$newpunct = $punct;
					$newpunct =  (substr $newpunct,0,(index $newpunct,'[')+1).(pos_in_ttree($head)).']'."\n";
					@sentence = sostituisciinarray($punct,$newpunct,@sentence);
			}
			$newhead = $head;				
			$newhead = (substr $newhead,0,(index $newhead,')')+1)."\n";
			@sentence = sostituisciinarray($head,$newhead,@sentence);
			if ($otherson ne ''){
					$newotherson = $otherson;
					$newotherson =  (substr $newotherson,0,(index $newotherson,'[')+1).(pos_in_ttree($head)).']'."\n";
					@sentence = sostituisciinarray($otherson,$newotherson,@sentence);
			}
	}
	@sentence = cancelladaarray($conjp,@sentence);
	return @sentence;
}


## find_update_coindexed #######################################
#In CONStituency-TUT the modifiers which are between the verb and some verbal complement 
#are represented by using a trace in the canonical position of the modifier;
#this procedure deletes the traces of the intraposed modifiers 

sub find_update_coindexed{
	my $nodo = shift(@_);
	my @frase = @_;
	my $newnodo = '';
	my $coindex = '';
	my $coindexincoord = '';
	my $node = '';
	if ($nodo =~ m/\*/){ #se il nodo e' una traccia
			$coindex = substr $nodo, ((index $nodo,'*')+1);	#trova l'indice che coindicizza la traccia
	}
	else{$phrasenodo = nome_phrase($nodo),"\n"; #altrimenti, se il nodo non e' traccia, ne calcola la categoria sintagmatica
		if ($phrasenodo eq 'S'){
 				foreach $figlio(subtreeconstit($nodo,@frase)){ 
						if (nome_phrase($figlio) =~ m/VP/){ 
								$coindex = substr $figlio, ((index $figlio,'*')+1);
								$coindexincoord = $figlio;
						}
				}
		}
		else{
		    foreach $figlio(figliconstit($nodo,@frase)){ #ne cerca tutti i figli
					if ((nome_phrase($figlio) =~ m/$phrasenodo/) and #se figlio ha la stessa categ sintag di nodo
					    ($figlio =~ m/\*/) and 										#e' una traccia e il sottoalbero di nodo contiene
						(join ("",figliconstit($nodo,@frase)) =~ m/CONJP\-COORD/)){  #un CONJP-COORD
							$coindex = substr $figlio, ((index $figlio,'*')+1);
							$coindexincoord = $figlio;
					}
			}
		}
	}
	$coindex = substr $coindex,0,(index $coindex,' ');
	if ($nodo =~ m/INTRAMD/){
			foreach $node(@frase){
					if (($node =~ m/\*\[/) and ($node ne $nodo) and 
						(nome_phrase($node) eq nome_phrase($nodo))){#se $node e' oggetto di coindicizzazione ed e' diverso da nodo
							$coindexnode = substr $node, (index $node,'*[')+2;
							$coindexnode = substr $coindexnode,0,(index $coindexnode,']');	#ne calcola il coindice
							if ($coindexnode eq $coindex){	#se il coindice di node corrisponde al coindicizzato di nodo
									if (nome_phrase($nodo) eq 'S'){
											$nonno_nodo = padre((padre($node,@frase)),@frase);
											$newnodo = (substr $nonno_nodo,0,(index $nonno_nodo,' ')).(substr $nodo,(index $nodo,' '));
											@frase = cancelladaarray($node,@frase);
											@frase = cancelladaarray($nonno_nodo,@frase);
											@frase = sostituisciinarray($nodo,$newnodo,@frase);
											if ($coindexincoord ne ''){
													$newcoindexincoord = $coindexincoord;
													$newcoindexincoord =~ s/$coindex//; 
													$newcoindexincoord =~ s/\*//;
													@frase = sostituisciinarray($coindexincoord,$newcoindexincoord,@frase);
											}
									}
									else{$newnodo = (substr $node,0,(index $node,'*')).(substr $nodo,(index $nodo,' '));
											@frase = cancelladaarray($node,@frase);
											@frase = sostituisciinarray($nodo,$newnodo,@frase);
											if ($coindexincoord ne ''){
													$newcoindexincoord = $coindexincoord;
													$newcoindexincoord =~ s/$coindex//; 
													$newcoindexincoord =~ s/\*//;
													@frase = sostituisciinarray($coindexincoord,$newcoindexincoord,@frase);
											}
									}
							}
					}
			}
	}
return @frase;
}


## find_update_coindexed #######################################
#the same as the previous procedure but for the extraposed subjects

sub find_update_extraposedsubj{
	my $nodo = shift(@_);
	my @frase = @_;
	my $newnodo = '';
	my $coindex = substr $nodo, ((index $nodo,'*')+1);
	$coindex = substr $coindex,0,(index $coindex,' ');
return @frase;
}


### remove_Xbar _from ##############################################
#removes all the intermediate projections (Dbar, Nbar, ADVbar, ADJbar,Vbar) that are not allowed 
#in the Penn since applies a minimal projection strategy of representation.
#ex.: (NP (Nbar (noun))) become (NP (noun)), 
#(NP (Nbar (Nbar (noun)) (PP...))) become (NP (NP (noun)) (PP...))

sub remove_Xbar_from{
	my $Xbar = shift(@_);		#un sintagma Xbar
	my @tree = @_;				#un albero
	my @figliXbar = ();
	@figliXbar= figliconstit($Xbar,@tree);	#trova i figli di Xbar dentro tree
	@figliXbartemp = @figliXbar;
	my $trovat = '';
	my $padreXbar = padre($Xbar,@tree);	#trova il padre di Xbar dentro tree
	if ((($Xbar =~ m/Sbar/) and ($padreXbar =~ m/S\-RMOD\+RELCL/)) or 
		(($Xbar =~ m/Sbar/) and ($padreXbar =~ m/S\s/) and 
		(padre($padreXbar,@tree) =~ m/S\-RMOD\+RELCL/))){
				$newXbar = $Xbar;
				if (padre($padreXbar,@tree) =~ m/S\-RMOD\+RELCL/){
						$newXbar =~ s/Sbar/Srespecialclbbaarr/;	
				}
				else{$newXbar =~ s/Sbar/Srelclbbaarr/;	
				}
				@tree = sostituisciinarray($Xbar,$newXbar,@tree);
	}
	if ((substr $padreXbar,0,(index $padreXbar,' '))=~ m/bar/){		#se il padre di Xbar e' un Xbar
			$trovat = 'sip';																#valorizza trovat
	}
	else{foreach $fig(@figliXbar){		#altrimenti, se Xbar ha un figlio Xbar
				if ((substr $fig,0,(index $fig,' ')) =~ m/bar/) {			#valorizza trovat
						$trovat = 'si';
				}
		  }
	} 
	if ($trovat eq ''){			#se element e' Xbar ma senza figli o padre Xbar
			if (($Xbar =~ m/Dbar/) and ($padreXbar !~ m/DP/)){		#caso particolare di alb17
					$newXbar = $Xbar;
					$newXbar =~ s/Dbar/DP/;
					@tree = sostituisciinarray($Xbar,$newXbar,@tree);
			}
			elsif (($Xbar !~ m/Vbar/) or (($Xbar =~ m/Vbar/) and (join(" ",@figliXbar) !~ m/VAU/))){	#caso di Xbar diversi da Vbar padre di ausiliare
					if ($Xbar =~ m/\*/){		#se Xbar porta un riferimento di traccia, sposta il riferimento sul padre (che e' un XP)
							$coindex = substr $Xbar,(index $Xbar,'*');
							$coindex = substr $coindex,0,(index $coindex,' ');
							$newpadreXbar = (substr $padreXbar,0,(index $padreXbar,' ')).$coindex.(substr $padreXbar,(index $padreXbar,' '));
							@tree = sostituisciinarray($padreXbar,$newpadreXbar,@tree);
					}
					if ($Xbar =~ m/]/){		#se Xbar non e' il nodo root del tree
							@tree = cancelladaarray($Xbar,@tree);	#lo cancella da tree	
						   while (scalar(@figliXbartemp) > 0){			#aggiorna tutti gli archi che puntavano a Xbar
									$figlio = shift(@figliXbartemp);
									$newfiglio = substr $figlio,0,(index $figlio,'[')+1;
									$newfiglio = $newfiglio.padre_in_quadre($Xbar).']';
									@tree = one_level_up($figlio,$Xbar,@tree);
						   }
					}
			}
			elsif ($Xbar =~ m/Vbar/){		#NB: qui si cancellano solo i Vbar, ma se la struttura di VP include aux 
					foreach $bb(@figliXbar){		#deve anche essere ruotata (vedi sub rotate_aux_structure)
							@tree = one_level_up($bb,$Xbar,@tree);
							@tree = cancelladaarray($Xbar,@tree); 
					}
			}
	}
	else{if ($trovat eq 'sip'){		#se element e' Xbar con padre Xbar
				if (($Xbar =~ m/Nbar/) and ($padreXbar =~ m/Nbar/)){		#se si tratta di Nbar
						$newXbar = $Xbar;		#rinomina entrambi i Nbar in NP
						$newXbar =~ s/Nbar/NP/;
						$newpadreXbar = $padreXbar;
						$newpadreXbar =~ s/Nbar/NP/;
						@tree = sostituisciinarray($Xbar,$newXbar,@tree);
						$padrepadreXbar = padre($padreXbar,@tree);		#cancella la testa da cui dipende Xbar						
						$newpadreXbar = propagarelaz($newpadreXbar,$padrepadreXbar);
						@tree = sostituisciinarray($padreXbar,$newpadreXbar,@tree);
						@figlipadrepadre = figliconstit($padrepadreXbar,@tree);
						@figlitemp = @figlipadrepadre;
						if ($padrepadreXbar !~ m/\-TOP/){
								foreach $fi(@figlitemp){
										@tree = one_level_up($fi,$padrepadreXbar,@tree);
								}
								@tree = cancelladaarray($padrepadreXbar,@tree);
						}
			   }
			   elsif ($Xbar =~ m/Vbar/){ 
						if (join(" ",@figlitemp) !~ m/VAU/){
								foreach $ff(@figliXbar){#one level up di tutti i figli di Xbar
											@tree = one_level_up($ff,$Xbar,@tree);		
								}
								@tree = cancelladaarray($Xbar,@tree);#cancellazione di Xbar
		   						@figlipadreXbar = figliconstit($padreXbar,@tree);#onelevelup di tutti ifigli di padreXbar
		   						foreach $ff(@figlipadreXbar){
											@tree = one_level_up($ff,$padreXbar,@tree);
								}
								@tree = cancelladaarray($padreXbar,@tree);#cancellazione del padreXbar							   				
			   			}
			   }
			   elsif (($Xbar =~ m/ADJbar/) and ($padreXbar =~ m/ADJbar/)){
							foreach $ff(figliconstit($Xbar,@tree)){#mette il terminale adj come figlio di ADJP
									if ($ff =~ m/ADJ~/){
											$terminale = $ff;
											$newterminale = substr $terminale,0,(index $terminale,'[')+1;
											$newterminale = $newterminale.padre_in_quadre($padreXbar).']'."\n";
									}
									elsif ($ff =~ m/PP\-ARG/){		#caso dell'argomento dell'aggettivo
											$argomento = $ff;
											$newargomento = substr $argomento,0,(index $argomento,'[')+1;
											$newargomento = $newargomento.padre_in_quadre($padreXbar).']'."\n";
									}
							}			
							@figlipadreXbar = figliconstit($padreXbar,@tree);#mette il dipendente dell'upper Adjbar come figlio di ADJP
							if (scalar(@figlipadreXbar) > 1){		#se esiste la doppia bar proiezione e' perche' esiste un modificatore dell'adjp
									foreach $fifi(@figlipadreXbar){
											if ($fifi ne $Xbar){
													$newfifi = substr $fifi,0,(index $fifi,'[')+1;
													$newfifi = $newfifi.padre_in_quadre($padreXbar).']'."\n";
													@tree = sostituisciinarray($fifi,$newfifi,@tree);
											}
									}
							}
							@tree = sostituisciinarray($terminale,$newterminale,@tree);
							@tree = sostituisciinarray($argomento,$newargomento,@tree);
							@tree = cancelladaarray($Xbar,@tree);#cancella entrambi gli adjbar
							@tree = cancelladaarray($padreXbar,@tree);
			   }
			   elsif (($Xbar =~ m/ADVbar/) and ($padreXbar =~ m/ADVbar/)){							
							foreach $ff(figliconstit($Xbar,@tree)){#mette il terminale adj come figlio di ADJP
									if ($ff =~ m/ADVB/){
											$terminale = $ff;
									}
							}
							$newterminale = substr $terminale,0,(index $terminale,'[')+1;
							$newterminale = $newterminale.padre_in_quadre($padreXbar).']'."\n";
							@figlipadreXbar = figliconstit($padreXbar,@tree);#mette il dipendente dell'upper Adjbar come figlio di ADJP
							if (scalar(@figlipadreXbar) == 2){		#se esiste la doppia bar proiezione e' perche' esiste un modificatore dell'adjp
									 if ($figlipadreXbar[0] ne $Xbar){
											$modificatoreXbar = $figlipadreXbar[0];
									}
									else{$modificatoreXbar = $figlipadreXbar[1];
									}
							}
							$newmodificatoreXbar = substr $modificatoreXbar,0,(index $modificatoreXbar,'[')+1;
							$newmodificatoreXbar = $newmodificatoreXbar.padre_in_quadre($padreXbar).']'."\n";
							@tree = sostituisciinarray($modificatoreXbar,$newmodificatoreXbar,@tree);
							@tree = sostituisciinarray($terminale,$newterminale,@tree);							
							@tree = cancelladaarray($Xbar,@tree);#cancella entrambi gli adjbar
							@tree = cancelladaarray($padreXbar,@tree);
			   }
			  else{ 
					$newXbar = $Xbar;
					$newpadreXbar = $padreXbar;
					$newXbar =~ s/\(Dbar/\(DP/;
					$newpadreXbar =~ s/\(Dbar/\(DP/;
					@tree = sostituisciinarray($padreXbar,$newpadreXbar,@tree);
					@tree = sostituisciinarray($Xbar,$newXbar,@tree);
			   }
		}
		else{if ($trovat eq 'si'){
			  }
		}
	}
	return @tree;	
}



### standardize_NOUN_ARG ##############################################
#modifies the noun arguments (annotated in TUT as NOUN-SUBJ, NOUN-OBJ, NOUN-INDOBJ) and reduces 
#them as modifiers of the noun, as in Penn

sub standardize_NOUN_ARG{
	my $nounarg = shift(@_);
	my @sentence = @_;
	my $fathernounarg = '';
	$fathernounarg = padre($nounarg,@sentence);
	my @fratelli = fratelli($fathernounarg,@sentence);
	my $newfather = '';		#creo una copia del padre C, se necessaria
	if ((join("",@fratelli) !~ m/PPSUBJ/) and (join("",@fratelli) !~ m/PP\-OBJ/) and
		(join("",@fratelli) !~ m/PP\-INDOBJ/)){
				if (pos_in_spina($fathernounarg) ne '1'){
						$newfather = '('.nome_phrase($fathernounarg).' '.spina($fathernounarg).
						                    '-1/)['.pos_in_ttree($fathernounarg).']'."\n";
						$newnounarg = substr $nounarg,0,(rindex $nounarg,'[')+1;
						$newnounarg = $newnounarg.padre_in_quadre($fathernounarg).']'."\n";
						$newnounarg =~ s/SUBJ/\-RMOD/; 
						$newnounarg =~ s/INDOBJ/RMOD/;
						$newnounarg =~ s/OBJ/RMOD/; 
						@sentence = sostituisciinarray($nounarg,$newnounarg,@sentence);
				}
				else{#significa che esiste un  Mod attaccato sotto il padre di NOUN-ARG
						$newnounarg = substr $nounarg,0,(index $nounarg,'[')+1;
						$newnounarg = $newnounarg.padre_in_quadre($fathernounarg).']'."\n";
						$newnounarg =~ s/SUBJ/\-RMOD/; 
						$newnounarg =~ s/INDOBJ/RMOD/;
						$newnounarg =~ s/OBJ/RMOD/; 
						@sentence = sostituisciinarray($nounarg,$newnounarg,@sentence);			
				}
	}
	else{	if (pos_in_spina($fathernounarg) ne '1'){
				$newfather = '('.nome_phrase($fathernounarg).' '.spina($fathernounarg).'-1/)['.pos_in_ttree($fathernounarg).']'."\n";
				push (@sentence,$newfather);
				foreach $elem(@figli){
						if (($elem !~ m/PP\-OBJ/) and ($elem !~ m/PP\-INDOBJ/) and  
							($elem !~ m/PPSUBJ/)){
									$newelem = substr $elem,o,(index $elem,'[')+1;
									$newelem = $newelem.pos_in_ttree($newfather).']'."\n";
						}
						else{$newelem = $elem;
								$newelem =~ s/SUBJ/\-RMOD/; 
								$newelem =~ s/INDOBJ/RMOD/;
								$newelem =~ s/OBJ/RMOD/; 
						}
						@sentence = sostituisciinarray($elem,$newelem,@sentence);
				}	
		}
		else{#significa che esiste un  Mod attaccato sotto il padre di NOUN-ARG
				foreach $elem(@figli){
						if (($elem =~ m/PP\-OBJ/) and ($elem =~ m/PP\-INDOBJ/) and  
							($elem =~ m/PPSUBJ/)){
									$newelem = $elem;
									$newelem = substr $newelem,0,(index $newelem,'[')+1;
									$newelem = $newelem.padre_in_quadre($fathernounarg).']'."\n";
									$newelem =~ s/SUBJ/\-RMOD/; 
									$newelem =~ s/INDOBJ/RMOD/;
									$newelem =~ s/OBJ/RMOD/; 
									@sentence = sostituisciinarray($elem,$newelem,@sentence);			
						}
				}
			
		}
	}
return @sentence;
}



### del_and_flat_DP#################################################
#deletes the DPs and modifies the structure like in Penn:
#ex.: (DP (Dbar (art) (NP (Nbar (noun))))) become (NP (art) (noun)). 

sub del_and_flat_DP{
	my $dp = shift(@_);	#un DP
	my @daftree = @_;
	my $trovato = 'no';
	my @subtree_dp = ();
	my @figli_dp = ();
	@subtree_dp = subtreeconstit($dp,$trovato,@daftree);
	@figli_dp = figliconstit($dp,@subtree_dp);
	my @nonterminali = ();
	my $det = '';
	my $np = '';
	my $npmod = '';
	my $ppmod = '';
	my $pparg = '';
	my $noun = '';
	my $num = '';
	my $coordinazione = '';
	my $srelativa = '';
	my $predet = '';
	my $dpfiglio = '';
	my $contin = '';
	my $parentetica = '';
	my $adv = '';
	my $adj = '';
	my $percent = '';
	my $pppartitive = '';
	my $dpmod = '';
	my $pronrel = '';
	my $verbindp = '';
	my $openquo = '';
	my $closequo = '';
	foreach $son(@figli_dp){
			if ($son =~ m/\/\)\[/){
					push(@nonterminali,$son);
			}
			if ($son =~ m/CONJP/){
					$coordinazione = 'si';
			}
			elsif ($son =~ m/OPEN\+QUOTE/){
					$openquo = $son;
			}
			elsif ($son =~ m/CLOSE\+QUOTE/){
					$closequo = $son;
			}
			elsif (($son =~ m/\(ART\~DE/) or ($son =~ m/\(ART\~IN/) or ($son =~ m/\(ADJ\~DE/) or 
					($son =~ m/\(ADJ\~DI/) or ($son =~ m/\(ADJ\~IN/) or ($son =~ m/\(ADJ\~IR/) or 
					($son =~ m/\(ADJ\~PO/) or ($son =~ m/\(NUMR2/)){
							$det = $son;
			}
			elsif (((nome_phrase($son) =~ m/NP/) or (nome_phrase($son) =~ m/Nbar/)) and ($son !~ m/APPOSITION/) and 
				    ($son !~ m/RMOD/) and ($son !~ m/DENOM/) and ($son !~ m/CONTIN/)){
					$np = $son;
			} 
			elsif ((nome_phrase($son) =~ m/NP/) and (($son =~ m/APPOSITION/) or
				    ($son =~ m/RMOD/))){
					$npmod = $son;
			} 
			elsif ($son =~ m/PP\-RMOD/){
					$ppmod = $son;
			}
			elsif ($son =~ m/S\-RMOD\+RELCL/){
					$srelativa = $son;
			}
			elsif (($son =~ m/NUMR/) and ($son =~ m/NUMR2/)){
					$num = $son;
			}
			elsif (($son =~ m/NUMR/) and ($son !~ m/NUMR2/)){
					$numero = $son;
			}
			elsif ($son =~ m/CONTIN/){
					$contin = $son;
			}
			elsif ($son =~ m/NOU\~/){
					$noun = $son;		
			}
			elsif (($son =~ m/PRDT/) or ($son =~ m/ADJP2\-RMOD/)){
					$predet = $son;
			}
			elsif (($son =~ m/\(DP/) and ($son !~ m/RMOD/)){
					$dpfiglio = $son;
			}
			elsif (($son =~ m/\(DP/) and ($son =~ m/RMOD/)){
					$dpmod = $son;
			}
			elsif ($son =~ m/SEPARATOR/){
					$separator = $son;
			}
			elsif ($son =~ m/ADV/){
					$adv = $son;
			}
			elsif ($son =~ m/ADJ/){
					$adj = $son;
			}
			elsif (($son =~ m/SPECIAL/) and ($son =~ m/PERCENT/)){
					$percent = $son;
			}
			elsif ($son =~ m/PP\-ARG\-PARTITIVE/){
					$pppartitive = $son;
			}
			elsif ($son =~ m/PRO\~RE/){
					$pronrel = $son;
			}
			elsif ($son =~ m/\(PRO\~OR/){
					$pronord = $son;
			}
			elsif ($son =~ m/S\-ARG/){
					$verbindp = $son;
			}
			elsif ($son =~ m/PRN/){
					$parentetica = $son;
			}
	}
	if (($dp !~ m/\/\)\[/) or ($dp =~ m/\*\[/) or ($dp =~ m/LISTPOS/) or ($verbindp ne '') or
		(($coordinazione eq 'si') and (($dpfiglio ne '') or ($np ne ''))) or (scalar(@figli_dp) == 0)){	#se DP e' la root del tree
			$newdp = $dp;
			$newdp =~ s/DP/NP/;
			@daftree = sostituisciinarray($dp,$newdp,@daftree);
	}		#i casi che seguono si realizzano se non e' la root del ttree
	else{
			if (($ppmod ne '') or ($coordinazione ne '')){	#se esiste un PP allora si deve tenere DP e trasformarlo in NP
					$newdp = $dp;
					$newdp =~ s/DP/NP/;
					@daftree = sostituisciinarray($dp,$newdp,@daftree);	
					if (($np eq '') and ($dpfiglio eq '') and ($det =~ m/NUMR/)){
							$newnum = substr $det,0,(rindex $det,'[')+1;
							$newnum = $newnum.padre_in_quadre($dp).']'."\n";
							@daftree = sostituisciinarray($det,$newnum,@daftree);	
					}
					elsif (($np eq '') and ($npmod ne '') and ($det ne '') and ($numero ne '')){	#caso delle date (Es. in alb474)
							$newnpmod = substr $npmod,0,(rindex $npmod,'[')+1;	#aggancia np sotto il resto dell'albero
							$newnpmod = $newnpmod.padre_in_quadre($dp).']'."\n";
							$newnpmod = propagarelaz($newnpmod,$npmod);
							@daftree = sostituisciinarray($npmod,$newnpmod,@daftree);	
							$newdet = substr $det,0,(rindex $det,'[')+1;		#aggancia det sotto suo fratello NP
							$newdet = $newdet.pos_in_ttree($newnpmod).']'."\n";		
							@daftree = sostituisciinarray($det,$newdet,@daftree);
							$newnumero  = substr $numero,0,(rindex $numero,'[')+1;		
							$newnumero = $newnumero.pos_in_ttree($newnpmod).']'."\n";		
							@daftree = sostituisciinarray($numero,$newnumero,@daftree);		
							@daftree = del_and_flat_DP($newnpmod,@daftree);	
					}
					elsif ($np ne ''){ 	#se ci sono det/predet e np metto det/predet dentro np
							if ($det ne ''){
									$newdet = substr $det,0,(rindex $det,'[')+1;		#aggancia det sotto suo fratello NP
									$newdet = $newdet.pos_in_ttree($np).']'."\n";		
									@daftree = sostituisciinarray($det,$newdet,@daftree);							
							}
							if ($pronrel ne ''){
									$newpronrel = substr $pronrel,0,(rindex $pronrel,'[')+1;		#aggancia det sotto suo fratello NP
							 		$newpronrel = $newpronrel.pos_in_ttree($np).']'."\n";		
									@daftree = sostituisciinarray($pronrel,$newpronrel,@daftree);
							}
							if ($openquo ne ''){
									$newopenquo = substr $openquo,0,(rindex $openquo,'[')+1;		#aggancia det sotto suo fratello NP
							 		$newopenquo = $newopenquo.pos_in_ttree($np).']'."\n";		
									@daftree = sostituisciinarray($openquo,$newopenquo,@daftree);
							}
							if ($closequo ne ''){
									$newclosequo = substr $closequo,0,(rindex $closequo,'[')+1;		#aggancia det sotto suo fratello NP
							 		$newclosequo = $newclosequo.pos_in_ttree($np).']'."\n";		
									@daftree = sostituisciinarray($closequo,$newclosequo,@daftree);
							}
							if ($predet ne ''){
									$newpredet = substr $predet,0,(rindex $predet,'[')+1;		#aggancia predet sotto suo fratello NP
									$newpredet = $newpredet.pos_in_ttree($np).']'."\n";		
									@daftree = sostituisciinarray($predet,$newpredet,@daftree);
							}
							if ($contin ne ''){
									@figliocontin = figliconstit($contin,@daftree);
									if (scalar(@figliocontin) ==1){ 
											$newcontin = substr @figliocontin[0],0,(rindex @figliocontin[0],'[')+1;
											$newcontin = $newcontin.pos_in_ttree($np).']'."\n";
											@daftree = sostituisciinarray($contin,$newcontin,@daftree);
											@daftree = cancelladaarray($contin,@daftree);
									}
							}
							if ($dpmod ne ''){
									@figliodpmod = figliconstit($dpmod,@daftree);
									if (scalar(@figliodpmod) ==1){ 
											$newdpmod = substr @figliodpmod[0],0,(rindex @figliodpmod[0],'[')+1;
											$newdpmod = $newdpmod.pos_in_ttree($np).']'."\n";
											@daftree = sostituisciinarray($dpmod,$newdpmod,@daftree);
											@daftree = cancelladaarray($dpmod,@daftree);
									}
							}
							foreach $figlio(@figli_dp){
									if (($figlio =~ m/\(ADJ\~/) or ($figlio =~ m/\(ADV/) or
										($figlio =~ m/NUM/)){
												$newfiglio = substr $figlio,0,(rindex $figlio,'[')+1;
												$newfiglio = $newfiglio.pos_in_ttree($np).']'."\n";
												@daftree = sostituisciinarray($figlio,$newfiglio,@daftree);													
									}
							}
							@daftree = del_and_flat_DP($np,@daftree);
					}
					elsif ($dpfiglio ne ''){
							if ($predet ne ''){
									$newpredet = substr $predet,0,(rindex $predet,'[')+1;		#aggancia predet sotto suo fratello NP
									$newpredet = $newpredet.pos_in_ttree($dpfiglio).']'."\n";		
									@daftree = sostituisciinarray($predet,$newpredet,@daftree);
							}
							foreach $figlio(@figli_dp){
									if (($figlio =~ m/\(ADJ/) or ($figlio =~ m/\(ADV/) or ($figlio =~ m/PRN/) or 
										(($figlio =~ m/NP/) and (($figlio =~ m/RMOD/) or ($figlio =~ m/APPOSITION/))) or
										($figlio eq $srelativa) or ($figlio eq $separator) or ($figlio =~ m/NUM/)){
												$newfiglio = substr $figlio,0,(rindex $figlio,'[')+1;
												$newfiglio = $newfiglio.pos_in_ttree($dpfiglio).']'."\n";
												@daftree = sostituisciinarray($figlio,$newfiglio,@daftree);													
									}
							}
							@daftree = del_and_flat_DP($dpfiglio,@daftree);								
					}
			}			
			else{	#altrimenti, se non esiste PP, si cancella DP
					if (($noun eq '') and ($adv eq '') and ($adj eq '') and (scalar(@nonterminali) >= 1) and 
					     ($percent eq '') and ($partitive eq '') and ($srelativa eq '') and ($verbindp eq '')){
								@daftree = cancelladaarray($dp,@daftree); 	#cancella dp
								if (($np eq '') and ($dpfiglio eq '') and ($det =~ m/NUMR/)){
											$newnum = substr $det,0,(rindex $det,'[')+1;
											$newnum = $newnum.padre_in_quadre($dp).']'."\n";
											@daftree = sostituisciinarray($det,$newnum,@daftree);	
								}
								elsif (($np eq '') and ($npmod ne '') and ($det ne '') and ($numero ne '')){	#caso delle date (Es. in alb474)
										$newnpmod = substr $npmod,0,(rindex $npmod,'[')+1;	#aggancia np sotto il resto dell'albero
										$newnpmod = $newnpmod.padre_in_quadre($dp).']'."\n";
										$newnpmod = propagarelaz($newnpmod,$npmod);
										@daftree = sostituisciinarray($npmod,$newnpmod,@daftree);	
										$newdet = substr $det,0,(rindex $det,'[')+1;		#aggancia det sotto suo fratello NP
										$newdet = $newdet.pos_in_ttree($newnpmod).']'."\n";		
										@daftree = sostituisciinarray($det,$newdet,@daftree);
										$newnumero  = substr $numero,0,(rindex $numero,'[')+1;		#aggancia predet sotto suo fratello NP
										$newnumero = $newnumero.pos_in_ttree($newnpmod).']'."\n";		
										@daftree = sostituisciinarray($numero,$newnumero,@daftree);		
										if ($predet ne ''){
												$newpredet = substr $predet,0,(rindex $predet,'[')+1;		#aggancia predet sotto suo fratello NP
												$newpredet = $newpredet.pos_in_ttree($newnpmod).']'."\n";
												@daftree = sostituisciinarray($predet,$newpredet,@daftree);
										}
										@daftree = del_and_flat_DP($newnpmod,@daftree);	
								}
								elsif ($np ne ''){ 	#se ci sono det/predet e np metto det/predet dentro np
										$newnp = substr $np,0,(rindex $np,'[')+1;	#aggancia np sotto il resto dell'albero
										$newnp = $newnp.padre_in_quadre($dp).']'."\n";
										$newnp = propagarelaz($newnp,$dp);
										if ($dp !~ m/CONTIN\+NUM/){
												$newnp = propagarelaz($newnp,$dp);
										}
										@daftree = sostituisciinarray($np,$newnp,@daftree);	
										if ($openquo ne ''){
												$newopenquo = substr $openquo,0,(rindex $openquo,'[')+1;		#aggancia det sotto suo fratello NP
							 					$newopenquo = $newopenquo.pos_in_ttree($np).']'."\n";		
												@daftree = sostituisciinarray($openquo,$newopenquo,@daftree);
										}
										if ($closequo ne ''){
												$newclosequo = substr $closequo,0,(rindex $closequo,'[')+1;		#aggancia det sotto suo fratello NP
										 		$newclosequo = $newclosequo.pos_in_ttree($np).']'."\n";		
												@daftree = sostituisciinarray($closequo,$newclosequo,@daftree);
										}
										if ($det ne ''){
												$newdet = substr $det,0,(rindex $det,'[')+1;		#aggancia det sotto suo fratello NP
												$newdet = $newdet.pos_in_ttree($newnp).']'."\n";	
												@daftree = sostituisciinarray($det,$newdet,@daftree);
										}
										if ($pronrel ne ''){
												$newpronrel = substr $pronrel,0,(rindex $pronrel,'[')+1;		#aggancia det sotto suo fratello NP
												$newpronrel = $newpronrel.pos_in_ttree($np).']'."\n";		
												@daftree = sostituisciinarray($pronrel,$newpronrel,@daftree);
										}
										if ($predet ne ''){
												$newpredet = substr $predet,0,(rindex $predet,'[')+1;
												$newpredet = $newpredet.pos_in_ttree($newnp).']'."\n";
												@daftree = sostituisciinarray($predet,$newpredet,@daftree);	
										}
										if ($num ne ''){
												$newnum = substr $num,0,(rindex $num,'[')+1;		#aggancia predet sotto suo fratello NP
												$newnum = $newnum.pos_in_ttree($newnp).']'."\n";		
												@daftree = sostituisciinarray($num,$newnum,@daftree);
										}								
										foreach $figlio(@figli_dp){
												if (($figlio =~ m/\(ADJ/) or ($figlio =~ m/\(ADV/) or ($figlio =~ m/PRN/) or 
													(($figlio =~ m/NP/) and (($figlio =~ m/RMOD/) or ($figlio =~ m/APPOSITION/))) or
													($figlio eq $srelativa) or ($figlio eq $separator) or ($figlio eq $npmod) or ($figlio =~ m/NUM/)
													or (($figlio =~ m/ART/) and ($figlio ne $det))){
															$newfiglio = substr $figlio,0,(rindex $figlio,'[')+1;
															$newfiglio = $newfiglio.pos_in_ttree($newnp).']'."\n";
															@daftree = sostituisciinarray($figlio,$newfiglio,@daftree);													
												}
										}
										@daftree = del_and_flat_DP($newnp,@daftree);
								}
								elsif ($dpfiglio ne ''){
										$newdpfiglio = substr $dpfiglio,0,(rindex $dpfiglio,'[')+1;	#aggancia np sotto il resto dell'albero
										$newdpfiglio = $newdpfiglio.padre_in_quadre($dp).']'."\n";
										$newdpfiglio = propagarelaz($newdpfiglio,$dp);
										@daftree = sostituisciinarray($dpfiglio,$newdpfiglio,@daftree);	
										if ($predet ne ''){
												$newpredet = substr $predet,0,(rindex $predet,'[')+1;		#aggancia predet sotto suo fratello NP
												$newpredet = $newpredet.pos_in_ttree($newdpfiglio).']'."\n";		
												@daftree = sostituisciinarray($predet,$newpredet,@daftree);
										}
										foreach $figlio(@figli_dp){
												if (($figlio =~ m/\(ADJ/) or ($figlio =~ m/\(ADV/) or ($figlio =~ m/PRN/) or 
													(($figlio =~ m/NP/) and (($figlio =~ m/RMOD/) or ($figlio =~ m/APPOSITION/))) or
													($figlio eq $srelativa) or ($figlio eq $separator) or ($figlio eq $npmod) or ($figlio =~ m/NUM/)){
															$newfiglio = substr $figlio,0,(rindex $figlio,'[')+1;
															$newfiglio = $newfiglio.pos_in_ttree($newdpfiglio).']'."\n";
															@daftree = sostituisciinarray($figlio,$newfiglio,@daftree);													
												}
								  		}
										@daftree = del_and_flat_DP($newdpfiglio,@daftree);								
								}
						}
						elsif (($verbindp ne '') and ($det ne '')){
								@figli_verbindp = figliconstit($verbindp,@daftree);
								foreach $figlioverbindp(@figli_verbindp){
										if ($figlioverbindp =~ m/VP/){
												@figli_figlioverbindp = figliconstit($figlioverbindp,@daftree);
												foreach $figliovp(@figli_figlioverbindp){
														if (($figliovp =~ m/VMA\~/) or ($figliovp =~ m/VMA\~/)){
																$verbo = $figliovp;
														}
												}
										}
								}
								$newverbo = substr $verbo,0,(index $verbo,'[')+1;
								$newverbo = $newverbo.padre_in_quadre($verbindp).']'."\n";
								@daftree = sostituisciinarray($verbo,$newverbo,@daftree);	
								@daftree = cancelladaarray($verbindp,@daftree);					
						}
						elsif (($noun eq '') and ($np eq '') and ($dpfiglio eq '') and 
								(($adv ne '') or ($adj ne ''))){#casi particolari come alb444
										$newdp = $dp;
										$newdp =~ s/DP/NP/;
										@daftree = sostituisciinarray($dp,$newdp,@daftree);	
						}
				}
	}
	return @daftree;
}



### flat_ADP#################################################
#deletes the ADJPs and ADVPs which are not allowed in the Penn minimal projection strategy
#which are those that have only one child
#ex.: (DP (Dbar (art) (NP (Nbar (Nbar (noun)) (ADJP (ADJbar (adj))))))) become (NP (NP (art) (noun) (adj))

sub flat_ADP{
	my $adp = shift(@_);	#un ADJP o un ADVP
	my @fatree = @_;
	my @figli_adp = ();
	my $verbmain_frat = '';
	my $vp_frat = '';
	my $advp_frat = '';
	my $adjp_frat = '';
	my $np_frat = '';
	my $num_frat = '';
	my $newadp = '';
	my $adpterm = '';
	my $predetfiglio = '';
	my $advpfiglio = '';
	my $adverbfiglio = '';
	my $ppmodfiglio = '';
	my $ppargfiglio = '';
	my $adjectivefiglio = '';
	my $adjpfiglio = '';
	my $dontflat = '';
	my $prep_frat = '';
	@figli_adp = figliconstit($adp,@fatree);		#adp ha sempre almeno un figlio
	if (scalar(@figli_adp) == 1){						#se adp ha un solo figlio 
			if (@figli_adp[0] !~ m/\/\)/){				#che e' un nodo terminale per forza adj o advb
					$adpterm = @figli_adp[0];
					@fratelli_adp= fratelli($adp,@fatree);	#cerca i fratelli di adp per metterci dentro adpterm
					foreach $fratello(@fratelli_adp){	
							if (($fratello =~ m/VMA/) or ($fratello =~ m/VMO/)){
											$verbmain_frat = $fratello;
							}
							elsif ($fratello =~ m/\(VP/){
											$vp_frat = $fratello;
							}
							elsif ($fratello =~ m/NP/){
									if (($fratello !~ m/RMOD/) and ($fratello !~ m/APPOSITION/)){
											$np_frat = $fratello;
									}
							}
							elsif ($fratello =~ m/ADVP/){
											$advp_frat = $fratello;
							}
							elsif ($fratello =~ m/ADJP/){
										if ($fratello !~ m/PRED/){
												$adjp_frat = $fratello;
										}
										else{$adjp_pred = $fratello;
										}
							}
							elsif ($fratello =~ m/NUMR2/){
											$num_frat = $fratello;
							}
							elsif ($fratello =~ m/PREP/){
											$prep_frat = $fratello;
							}
					}
					if ($adp =~ m/^\(ADVP/){		#se adp e' un avverbio
							if (($verbmain_frat eq '') and ($np_frat ne '')){		#se adp e' associato ad un np
									$newadpterm = substr $adpterm,0,(index $adpterm,'[')+1;
									$newadpterm = $newadpterm.pos_in_ttree($np_frat).']'."\n";
									@fatree = sostituisciinarray($adpterm,$newadpterm,@fatree);
									@fatree = cancelladaarray($adp,@fatree);
							}
							elsif (($prep_frat ne '') and (padre($adp,@fatree) =~ m/\(PP/)){#se adp e' associato a una prep (come in "neppure a Torino")
									$newadpterm = substr $adpterm,0,(index $adpterm,'[')+1;
									$newadpterm = $newadpterm.padre_in_quadre($prep_frat).']'."\n";
									@fatree = sostituisciinarray($adpterm,$newadpterm,@fatree);
									@fatree = cancelladaarray($adp,@fatree);
							}							
							elsif (($verbmain_frat eq '') and (($advp_frat ne '') or ($adjp_frat ne ''))){		#se adp e' associato ad un advp o adjp
									$newadpterm = substr $adpterm,0,(index $adpterm,'[')+1;
									if ($adjp_frat ne ''){
											$newadpterm = $newadpterm.padre_in_quadre($adjp_frat).']'."\n";
									}
									else{$newadpterm = $newadpterm.padre_in_quadre($advp_frat).']'."\n";}
									@fatree = sostituisciinarray($adpterm,$newadpterm,@fatree);
									@fatree = cancelladaarray($adp,@fatree);
							}
							elsif (($verbmain_frat eq '') and ($np_frat ne '') and ($advp_frat eq '') 	#se adp e' associato ad un np ed un numero
									and ($adjp_frat eq '') and ($num_frat ne '')){								#come in "almeno 10 gatti"
											$newadpterm = substr $adpterm,0,(index $adpterm,'[')+1;
											$newadpterm = $newadpterm.pos_in_ttree($np).']'."\n";
											@fatree = sostituisciinarray($adpterm,$newadpterm,@fatree);
											@fatree = cancelladaarray($adp,@fatree);
							}
							elsif ($verbmain_frat ne '') {	#se adp e' associato ad un verbmain deve diventare fratello del relativo VP
										if ($vp_frat eq ''){#tranne nel caso in cui advp sia tra aux e main
													$newadp = substr $adp,0,(index $adp,'[')+1;
													$newadp = $newadp.padre_in_quadre($verbmain_frat).']'."\n";
													@fatree = sostituisciinarray($adp,$newadp,@fatree);
										}
										elsif (($vp_frat ne '') and ((join ("",figliconstit($vp_frat,@fatree))) =~ m/\(VAU/)){
													if ((spina($adp) eq spina($vp_frat)+1) and (spina($adp) eq spina($verbmain_frat)-1)){
															$newadp = substr $adp,0,(index $adp,'[')+1;
															$newadp = $newadp.pos_in_ttree($vp_frat).']'."\n";
															@fatree = sostituisciinarray($adp,$newadp,@fatree);
													}
													else{$newadp = substr $adp,0,(index $adp,'[')+1;
															$newadp = $newadp.padre_in_quadre($verbmain_frat).']'."\n";
															@fatree = sostituisciinarray($adp,$newadp,@fatree);
													}
										}			
							}
							elsif (($vp_frat ne '') and ($verbmain_frat eq '')){	#se adp e' associato ad un vp deve diventare fratello del relativo VP
											$newadpterm = substr $adpterm,0,(index $adpterm,'[')+1;
											$newadpterm = $newadpterm.pos_in_ttree($vp_frat).']'."\n";
											@fatree = sostituisciinarray($adpterm,$newadpterm,@fatree);
											@fatree = cancelladaarray($adp,@fatree);
							}
					}
					else{
							if (($np_frat ne '') and ($adp !~ m/PRED/)){		#se adp e' associato ad un np
									$newadpterm = substr $adpterm,0,(rindex $adpterm,'[')+1;
									$newadpterm = $newadpterm.pos_in_ttree($np_frat).']'."\n";
									@fatree = sostituisciinarray($adpterm,$newadpterm,@fatree);
									@fatree = cancelladaarray($adp,@fatree);
							}
					}
			}
			else{		#se adp ha un solo figlio che non e' nodo terminale: NON SO SE ESISTE QUESTO CASO
			}
	}
	elsif (scalar(@figli_adp) >= 2){	#se adp ha piu' di un figlio
			foreach $figlioadp(@figli_adp){
					if ($figlioadp =~ m/PRDT/){
							$predetfiglio = $figlioadp;
					}
					elsif ($figlioadp =~ m/ADVP/){
							$advpfiglio = $figlioadp;
					}
					elsif ($figlioadp =~ m/ADVB/){
							$adverbfiglio = $figlioadp;
					}
					elsif ($figlioadp =~ m/\(ADVP/){
							$advpfiglio = $figlioadp; 
					}
					elsif ($figlioadp =~ m/\(PP\-RMOD/){
							$ppmodfiglio = $figlioadp;
					}
					elsif ($figlioadp =~ m/\(PP\-ARG/){
							$ppargfiglio = $figlioadp;
					}
					elsif ($figlioadp =~ m/\(ADJ~/){
							$adjectivefiglio = $figlioadp;
					}
					elsif ($figlioadp =~ m/ADJP/){
							$adjpfiglio = $figlioadp;
					}
			}
			if ($adp =~ m/\(ADJP/){		#se adp e' aggettivo allora ha sicuramente un figlio adjfiglio o adjpfiglio
					if ($advpfiglio ne ''){
							@figliadvpfiglio = figliconstit($advpfiglio,@fatree);
							foreach $son(@figliadvpfiglio){
									if ($son =~ m/\/\)\[/){		#se advpfiglio ha solo figli terminali 
											$dontflat = 'si';
									}
							}
							if ($dontflat ne 'si'){	#mette dentro adp tutti i figli di advpfiglio e cosi' appiattisce
									foreach $son(@figliadvpfiglio){
											$newson = substr $son,0,(index $son,'[')+1;
											$newson = $newson.pos_in_ttree($adp).']'."\n";
											@fatree = sostituisciinarray($son,$newson,@fatree);
									}
									@fatree = cancelladaarray($advpfiglio,@fatree);	#cancella advpfiglio (svuotato)	
							}
					}
		
			}
			elsif ($adp =~ m/\(ADVP/){		#se adp e' avverbio allora ha sicuramente un figlio adverbfiglio o advpfiglio
					if (($adverbfiglio ne '') and ($advpfiglio ne '')){		#se adp ha un advp annidato, advpfiglio
							foreach $dd(figliconstit($advpfiglio,@fatree)){		#appiattisce adp mettendo tutti i figli di advpfiglio sotto adp
									$newdd = substr $dd,0,(index $dd,'[')+1;
									$newdd = $newdd.pos_in_ttree($adp).']'."\n";
									@fatree = sostituisciinarray($dd,$newdd,@fatree);
							}
							@fatree = cancelladaarray($advpfiglio,@fatree);	#cancella advpfiglio (svuotato)
					}
			}
	}
	return @fatree;
}



### rotate_aux_structure #################################################
#In TUT and CONStituency-TUT the structure composed by main verb and auxiliary verb is headed by
#the main; if there is a double auxiliary verb, one tensed and one untensed, the untensed one is the 
#dependent of the main verb and the tensed depends on the untensed.
#In Penn instead the auxiliary verb (or the first of the auxiliary verbs) is the head.
#This procedure manages the conversion of the verbal structures including auxiliary verbs.

sub rotate_aux_structure{
	my $vp1 = shift(@_);
	my @frase = @_;
	my @figlivp1 = figliconstit($vp1,@frase);
	my $vp2 = '';
	my $vp3 = '';
	my $mainverb = '';
	my $aux1 = '';
	foreach $fig(@figlivp1){
			if ($fig =~ m/\(VP/){
					$vp2 = $fig;
					@figlivp2 = figliconstit($vp2,@frase);					
					foreach $fi(@figlivp2){
							if (($fi =~ m/VP/) and ($fi !~ m/ADVP/)){
									$vp3 = $fi;
									@figlivp3 = figliconstit($vp3,@frase);
									foreach $f(@figlivp3){
											if ($f =~ m/VAU~/){
													$aux1 = $f;
											}
									}
							}
							elsif ($fi =~ m/VAU~/){
									$aux2 = $fi;
							}
							elsif ($fi =~ m/ADVP/){
							}
					}
			}
			elsif (($fig =~ m/VMA~/) or ($fig =~ m/VMO~/)){
					$mainverb = $fig;
			}
	}
	
	if ((scalar(@figlivp1) >=2) and ($vp2 ne '') and ($mainverb ne '') and (scalar(@figlivp2 == 1)) and ($aux2 ne '')){
				$newvp1 = substr $vp1,0,(index $vp1,'[')+1;
				$newvp1 = $newvp1.pos_in_ttree($vp2).']'."\n";
				@frase = sostituisciinarray($vp1,$newvp1,@frase);
				$newvp2 = substr $vp2,0,(index $vp2,'[')+1;
				$newvp2 = $newvp2.padre_in_quadre($vp1).']'."\n";
				@frase = sostituisciinarray($vp2,$newvp2,@frase);
				foreach $el(@figlivp1){
						if ((pos_in_ttree($el) < pos_in_ttree($vp1)) and ($el ne $vp2)){
										$newel = substr $el,0,(index $el,'[')+1;
										$newel = $newel.padre_in_quadre($vp1).']'."\n";
										@frase = sostituisciinarray($el,$newel,@frase);
						}
						elsif (pos_in_ttree($el) < pos_in_ttree($vp1)){
										$newel = substr $el,0,(index $el,'[')+1;
										$newel = $newel.padre_in_quadre($vp2).']'."\n";
										@frase = sostituisciinarray($el,$newel,@frase);
						}
						
				}
	}
	elsif ((scalar(@figlivp1) >=2) and ($vp2 ne '') and ($mainverb ne '') and (scalar(@figlivp2 >= 2)) and 
			(scalar(@figlivp3) == 1) and ($aux1 ne '')){
					$newvp1 = substr $vp1,0,(index $vp1,'[')+1;
					$newvp1 = $newvp1.pos_in_ttree($vp2).']'."\n";
					@frase = sostituisciinarray($vp1,$newvp1,@frase);
					$newvp2 = substr $vp2,0,(index $vp2,'[')+1;
					$newvp2 = $newvp2.pos_in_ttree($vp3).']'."\n";
					@frase = sostituisciinarray($vp2,$newvp2,@frase);
					$newvp3 = substr $vp3,0,(index $vp3,'[')+1;
					$newvp3 = $newvp3.padre_in_quadre($vp1).']'."\n";
					@frase = sostituisciinarray($vp3,$newvp3,@frase);
					foreach $popo(@figlivp1){
							if (($popo ne $mainverb) and ($popo ne $vp2)){
							$newpopo = substr $popo,0,(rindex $popo,'[')+1;
							$newpopo = $newpopo.padre_in_quadre($newvp3).']'."\n";
							@frase = sostituisciinarray($popo,$newpopo,@frase);
							}
					}
	}
	elsif ((scalar(@figlivp1) >=2) and ($vp2 eq '') and ($mainverb ne '')){
			foreach $el(@figlivp1){
					if  (pos_in_ttree($el) < pos_in_ttree($vp1)){ 
							$newel = substr $el,0,(rindex $el,'[')+1;
							$newel = $newel.padre_in_quadre($vp1).']'."\n";
							@frase = sostituisciinarray($el,$newel,@frase);
					}
			}
	}
	return @frase;
}



###MODIFY-TRACING #################################################
#This procedure deletes the multiple traces (traces of traces) and modifies the indexes as needed.

sub modify_tracing{
	my $trace = shift(@_);
	my @sentence = @_;
	my $coindex = '';	
	my $coindexing = '';
	my $simpletrace = ''; 
	my $newtrace = '';
	my $newtraceson = '';
	my $posiznewtraceson = '';
	$simpletrace = substr $trace,0,(rindex $trace,'['); 	#trace senza il link al padre
	if ($simpletrace =~ m/\/\)/){#se e' traccia di un costituente
			$simpletrace = substr $simpletrace,(index $simpletrace,'*')+1;
			if ($simpletrace =~ m/\*/){ #allora e' coindexed e coindexer (non necessariamente in questo ordine)
					if ($simpletrace =~ m/^\[/){ 	#e' per prima cosa coindexed
							$coindex = substr $simpletrace,1,(index $simpletrace,']')-1;
							$coindexing = substr $simpletrace,(index $simpletrace,'*')+1;
							$coindexing = substr $coindexing,0,(index $coindexing,' ');
					}
					else{	#e' per prima cosa coindexer
							$coindexing = substr $simpletrace,0,(index $simpletrace,'*');
							$coindex = substr $simpletrace,(rindex $simpletrace,'[')+1;
							$coindex = substr $coindex,0,(index $coindex,']');
					}
					$newtrace = substr $trace,0,(index $trace,'*');
					$newtrace = $newtrace.'@'.$coindexing;
					$newtrace = $newtrace.(substr $trace,(index $trace,' '));
					$posiznewtraceson = (spina($trace)).'-'.(pos_in_spina($trace)+10);
					$newtraceson = '(NONCO '.$posiznewtraceson.'/'.$coindex.')['.(pos_in_ttree($trace)).']'."\n";
					push(@sentence,$newtraceson);
			} 
			else{#e' solo coindexed o coindexer
					if ($simpletrace =~ m/^\[/){ 	#e' coindexed
							$coindex = substr $simpletrace,1,(index $simpletrace,']')-1;
							$newtrace = substr $trace,0,(index $trace,'*');
							$newtrace = $newtrace.(substr $trace,(index $trace,' '));
							$posiznewtraceson = (spina($trace)).'-'.(pos_in_spina($trace)+10);
							$newtraceson = '(NONCO '.$posiznewtraceson.'/'.$coindex.')['.(pos_in_ttree($trace)).']'."\n";
							push(@sentence,$newtraceson);
					}
					else{	#e' coindexer
							$coindexing = substr $simpletrace,0,(index $simpletrace,' ');
							$newtrace = $trace;
							$newtrace =~ s/\*/\@/;
					}
			}
	}
	else{#se e' traccia di morfo-cat
			if ($simpletrace =~ m/\]\)/){
					$coindex = substr $simpletrace,(index $simpletrace,'/*[')+3;
					if ($coindex eq '])'){	#se e' traccia di morfo-cat non coindexed
							$coindex = '';
							$simpletrace = substr $simpletrace,0,(index $simpletrace,'/');
							if (($simpletrace =~ m/\*/) and ($simpletrace !~ m/LOCUT/) and 	#se e' traccia di morfo-cat 
								($simpletrace !~ m/CLEFT/)){													#non coindexed coindexer
										$coindexing = substr $simpletrace,(index $simpletrace,'*');
										$coindexing = substr $coindexing,0,(index $coindexing,' ');
										$coindexing =~ s/\*//;
										$newtrace = '(NONN@'.(substr $trace,(index $trace,'*'));
										$newtrace =~ s/\*//g;
										$newtrace =~ s/\[//;
										$newtrace =~ s/\]//;
							}
							else{#se e' traccia di morfo-cat non coindexed e non coindexer
									$newtrace = '(NONE'.(substr $trace,(index $trace,' ')); 
									$newtrace =~ s/\*//;
									$newtrace =~ s/\[//;
									$newtrace =~ s/\]//;
							}
					}
					else{$coindex = substr $coindex,0,(index $coindex,']'); #se e' traccia di morfo-cat coindexed
							$simpletrace = substr $simpletrace,0,(index $simpletrace,'/');
							if (($simpletrace =~ m/\*/) and ($simpletrace !~ m/LOCUT/) and 	#se traccia di morfo-cat coindexed
								($simpletrace !~ m/CLEFT/)){													#e coindexer
										$coindexing = substr $simpletrace,(index $simpletrace,'*');
										$coindexing = substr $coindexing,0,(index $coindexing,' ');
										$coindexing =~ s/\*//;
										$newtrace = (substr $trace,(index $trace,'*'));
										$newtrace =~ s/\[/\-/g;
										$newtrace =~ s/\]//;
										$newtrace =~ s/\*//g;
										$newtrace = '(NONN@'.$newtrace;
							}
							else{#se e' traccia di morfo-cat coindexed e non coindexer
									$newtrace = '(NONCO'.(substr $trace,(index $trace,' '));
									$newtrace =~ s/\[//;
									$newtrace =~ s/\]//;
									$newtrace =~ s/\*//;
							}
					}
			}
			else{	#altrimenti e' una categoria morfologica (con parola) che funge da coindexer (ex tracce w)
					$coindexing = substr $simpletrace,(index $simpletrace,'*')+1;
					$coindexing = substr $coindexing,0,(index $coindexing,' ');
					$newtrace = $trace;
					$newtrace =~ s/\*/\@/;
			}
	}
	@sentence = sostituisciinarray($trace,$newtrace,@sentence);
	return @sentence;
}


########### one_level_up #####################################
#given a node, the father of the node and a tree, put in the higher level of the tree the node

sub one_level_up{
	my $arg = shift(@_);
	my $fatherarg = shift(@_);
	my @argtree = @_;
	my $newarg = $arg;
	$newarg = substr $newarg,0,(rindex $newarg,'[') +1;	
	$newarg = $newarg.(padre_in_quadre($fatherarg))."]\n";	#arg prende il padre di suo padre
	@argtree = sostituisciinarray($arg,$newarg,@argtree);	
return @argtree;
}


########### figliconstit #####################################
#finds the children of a given node in a given contituency tree

sub figliconstit{
	my $nodo = shift(@_);
	my @albero = @_;
	my @array = ();
	my $podelpadre = '';
	my $posnodo = pos_in_ttree($nodo);
	foreach $it(@albero){
		$posdelpadre = padre_in_quadre($it);
		if ($posdelpadre eq $posnodo){
			push(@array,$it);
		}
	}
return @array;
}


########### fratelli #####################################
#given a node and a tree, ouput the nodes having the same father than the node

sub fratelli{
	my $nodo = shift(@_);
	my @albero = @_;
	my @array = ();
	foreach $it(@albero){
		if ((padre_in_quadre($it) eq padre_in_quadre($nodo)) and ($it ne $nodo)){
			push(@array,$it);
		}
	}
return @array;
}


### padre_in_quadre ###########################################################
#finds the father of a node argument

sub padre_in_quadre{
	my $nodo = shift(@_);
	my $padre = substr $nodo,0,(rindex $nodo,']');
	$padre = substr $padre,((rindex $padre,'[') + 1);
	return $padre;
}


### pos_in_ttree ###########################################################
#finds the position of a node within a constituency tree

sub pos_in_ttree{
	my $nodo = shift(@_);
	my $posinttree = substr $nodo,((index $nodo,' ')+1);
	$posinttree = substr $posinttree, 0,(index $posinttree,'/');
	return $posinttree;
}


#### subtreeconstit ##################################################################################
#finds the subtree of a given node

sub subtreeconstit{
	my $nodo = $_[0];
	$tro = $_[1];   #$tro `e inizializzata all'esterno
	my @T_tree = @_;
	if ($tro eq 'no'){  #solo quando si entra in subtree da una chiamata esterna
		@subtreeconstituenti = ();  #l'array del subtree deve essere vuoto, ma deve non deve essere
	}                       #svuotato nelle chiamate ricorsive
	$tro = 'si';
	if (scalar(figliconstit($nodo,@T_tree)) > 0){     #se il nodo ha figli
		push(@subtreeconstituenti,$nodo);              #viene messo nell'array subtree e la routine
		foreach $figlio(figliconstit($nodo,@T_tree)){     #viene richiamata su ogni suo figlio
			subtreeconstit($figlio,$tro,@T_tree);
		}
	}
	else{push(@subtreeconstituenti,$nodo);                 #se invece il nodo non ha figli, viene messo nell'array
	}
return @subtreeconstituenti;
}


#### sostituisciinarray ##################################################################################
#given A and B and and array, substitues B with A in the array (if occurs)

sub sostituisciinarray {
	my $vecchio = shift(@_);
	my $nuovo = shift(@_);
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elemento(@array){
		if ($elemento ne $vecchio){
			push (@arraytemp1,$elemento);
		}
		else {push (@arraytemp1,$nuovo);
		}
	}
	@array = @arraytemp1;
return @array;
}


#### cancelladaarray ##################################################################################
#given and argument and an array, removes the argument from the array

sub cancelladaarray {
	my $elemento = shift(@_);
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elem(@array){
		if ($elem ne $elemento){
			push (@arraytemp1,$elem);
		}
	}
	@array = @arraytemp1;
return @array;
}



######### padre ##################################################################################
#given a line in tut format and a sentence as an array, output the line of the head word of the given line in the sentence

sub padre{
	my $arg =  shift(@_);
	my @frase = @_;
	my $padre = '';
	my $pospadrediarg = '';
	$pospadrediarg = substr $arg,((rindex $arg, '[') + 1);    #calcola la posizione del padre
	$pospadrediarg = substr $pospadrediarg, 0,(rindex $pospadrediarg, ']');
	foreach $item(@frase){
		$positem = substr $item, ((index $item, ' ') + 1);
		$positem = substr $positem,0,(index $positem,'/');
		if ($positem eq $pospadrediarg){
			$padre = $item;
		}
	}
return $padre;
}



#### nome_phrase ###############################################################################
#give a node, output the name of the phrase of such a node

sub nome_phrase{
	my $phrase = shift(@_);
	my $nome = substr $phrase,1,((index $phrase,' ') - 1);
	if (($nome =~ m/\-/) and ($nome !~ m/PUNCT\-COORD/)){
		$nome = substr $nome,0,(index $nome, '-');
	}
	else{if ($nome =~ m/PUNCT\-COORD/){
			$nome = 'PUNCTCOORD+BASE';
		}
	}
return $nome;}



######### spina ##################################################################################
#given a node, ouptut the spine where the node is positioned within the tree

sub spina{
	my $nodo = shift(@_);
	my $spina = '';
	my $posnodo = pos_in_ttree($nodo);
	if ($posnodo =~ m/\-/){ #calcola la spina di nodo
		$spina = substr $posnodo,0,(index $posnodo,'-');
	}
	else{$spina = $posnodo;
	}
return $spina;
}



######### pos_in_spina ##################################################################################
#given a node, ouptut the position of the node within the spine where the node is positioned within the tree

sub pos_in_spina{
	my $nodo = shift(@_);
	my $posnodo = pos_in_ttree($nodo);
	my $indice = 0;
	if  ($posnodo =~ m/\-/){ #calcola l'indice di nodo
		$indice = substr $posnodo,((index $posnodo,'-') + 1);
	}
return $indice;
}



#### propagarelaz ########################################################################################
#given a node with a grammatical relation, annotates the same relation on the child node

sub propagarelaz{
	my $figlio = shift(@_);
	my $padre = shift(@_);
	my $tracing = '';
	my $relaz = substr $padre,1,(index $padre, ' ');
	$relaz =~ s/\s//g;
	if ($relaz =~ m/\*\d/){
			$tracing = substr $relaz,(index $relaz,'*');
	}
	if ($relaz =~ m/\-/){
			$relaz = '-'.substr $relaz,(index $relaz, '-')+1;
	}
	else{if ($relaz =~ m/SUBJ/){
				$relaz = 'SUBJ'.$tracing;
		  }
		  else{$relaz = ''.$tracing;
		  }
	}
	if ($figlio !~ m/SUBJ/){
			$figlio = '('.nome_phrase($figlio).$relaz.(substr $figlio,(index $figlio,' '));
	}
	return $figlio;
}



return 1;
END {}

