#/*  #Copyright 2011 Cristina Bosco [bosco] AT di DOT unito DOT it

    #This file is part of TUTtoPENN converter rel. 1.0.

    #TUTtoPENN converter is free software: you can redistribute it and/or modify
    #it under the terms of the GNU General Public License as published by
    #the Free Software Foundation, either version 3 of the License, or
    #(at your option) any later version.

    #TUTtoPENN converter is distributed in the hope that it will be useful,
    #but WITHOUT ANY WARRANTY; without even the implied warranty of
    #MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    #GNU General Public License for more details.

    #You should have received a copy of the GNU General Public License
    #along with TUTtoPENN converter. If not, see <http://www.gnu.org/licenses/>.
#*/


#This file is part of the TUTtoPENN converter, which implements a conversion from TUT 
#native format to Penn Treebank format adapted to Italian (called TUT-Penn).
#The TUTtoPENN converter User MANUAL, which can be dowloaded from 
#http://www.di.unito.it/~tutreeb/TUTtoPENN/TUTtoPENNconverter-manual.zip), describes 
#the organization and use of the converter.
#
#This is the SECOND MODULE, i.e. the module the module that is called to convert the 
#the dependency syntactic structures into constituency structures Xbar like, i.e. the 
#CONStituency-TUT. The input of this module is in the format where the PoS tagging has 
#been converted in a Penn-like PoS tagging, i.e. the format produced by the FIRST module
#preprocessor and is in the file 'outpreproc.out'.
#The call of this module is done by the MAIN module after the preprocessing (FIRST module) 
#and is done when the parameter is -c or -a or -p.
#The output of this module are sentences in form of array and in CONStituency-TUT format, 
#and is in the file called 'outconstut.out'.

#The main comments are in English, the minor in Italian

package package_constut;	
BEGIN {}

#################################### MAIN ################################################
#manages the call of the rest of the module on the single sentences included in the input file
#The main procedure put each sentence in an array (D_frase) where each word is an alement of the array
#It put the output in another array (T_frase), where each terminal and non terminal node (that the conversion
#generates) is an element of the array.
#At the end of the processing of each sentence, both the array are emptied and the procedure is repeated until 
#the end of the input file.

sub main_constut{
	print "in constuttifier\n";
	@D_frase = ();       	#l'array che conterra' la frase in formato TUT
	@T_frase = ();			#l'array che conterra' la frase in formato Penn
	@balancedpunct = ();	#l'array che contiene punteggiatura bilanciata (parentetiche e quote)
	@separator = ();		#l'array che contiene i punti separatori
	$finefrase = 0;         	#la variabile che segnala il raggiungimento della fine della frase
	$contit = 0;            	#la variabile che segnala che il titolo e' stato trovato
	$file_T_frasi = '';  		#l'otput del programma

#tables reading ##############################################################################
#the tables are three, according to the algorithm used (written by Fei Xia): modifiers, arguments and head-projection
#that will be read later in the program in the subroutine proiezione
	@arg_table = ();  		#la tabella degli argomenti
	@mod_table = ();		#la tabella dei modificatori
	open(FILEHANDLEarg, '<TUTtoPENN1_0-arg-table.txt');
	while (<FILEHANDLEarg>){
			$ccc = $_;
   			push (@arg_table,$ccc);
	}
	close(FILEHANDLEarg);
	open(FILEHANDLEmod, '<TUTtoPENN1_0-mod-table.txt');
	while (<FILEHANDLEmod>){
			$ddd = $_;
   			push (@mod_table,$ddd);
	}
	close(FILEHANDLEmod);
	$contarighe = scalar(@_);
	while ($contarighe > 0){         #fino a quando ci sono righe di input
    	$test = 0;  						#inizializza la variabile di uscita $test
    	while ($test == 0){			#fino quando test vale 0
			$riga = shift(@_);					#legge la riga di input
   			$stringa = sprintf "%s", $riga;	 	#la converte in stringa
			$inizio = substr $stringa, 0,5; 	#legge i primi 5 caratteri
    		if ($inizio =~ m/\d/){        			#se la riga inizia con un carattere numerico
        	   	push (@D_frase,$stringa);	#la mette nell'array
			}
	       	else {if ($inizio !~ m/\*/){ 		#altrimenti se riga vuota
						if ($contit == 0){					#se non trovato il titolo (prima frase)
							$contit = ++$contit;   		#aggiorna la variabile $contit
							$file_T_frasi = $file_T_frasi.$titolo;
							print $titolo,"\n";
						}
		    			$finefrase = ++$finefrase;     	#aggiorna la var che segnala la fine della frase
		      		}
        	      	if ($inizio =~ m/\*/){       	#se la riga e' di **
       					$titolo = $stringa;     	#assegna il titolo della nuova frase a $titolo
						if ($finefrase != 0){           #se raggiunta la fine della frase precedente
           					@T_frase = translatesentence(\@D_frase,\@arg_table,\@mod_table);
							@T_frase = duplicaintermedie(@T_frase);
							@T_frase = sollevapuntofinale(@T_frase);
							@T_frase = sollevatracce(@T_frase);
							@T_frase = inserisciseparator(\@T_frase,\@separator);
							@T_frase = bilanciapunti(\@T_frase,\@D_frase,\@balancedpunct); 
							$T_frase = join("\n",@T_frase);	#prepara la frase per il programma tree-to-par
							$file_T_frasi = $file_T_frasi.$T_frase."\n\n".$titolo;
							print $titolo,"\n";
							@D_frase = ();			#svuota D_frase per la nuova frase
							@T_frase = ();			#svuota T_frase per la nuova frase
							@balancedpunct = ();		#svuota l'array della punteggiatura bilanciata
							@separator = ();
							$finefrase = 0;			#riporta la var di finefrase a 0
						}
	      			}
			}
    		$test = ++$test;      #aggiorna la var di uscita
    	}
   	$contarighe = --$contarighe;
 	}
	@T_frase = translatesentence(\@D_frase,\@arg_table,\@mod_table); #richiama translatesentence sull'ultima frase
	@T_frase = duplicaintermedie(@T_frase);
	@T_frase = sollevapuntofinale(@T_frase);
	@T_frase = sollevatracce(@T_frase);
	@T_frase = inserisciseparator(\@T_frase,\@separator);
	@T_frase = bilanciapunti(\@T_frase,\@D_frase,\@balancedpunct);
	$T_frase = join("\n",@T_frase);	#prepara la frase per il programma 
	$file_T_frasi = $file_T_frasi.$T_frase."\n\n";
	return $file_T_frasi;
}


######################## TRANSLATESENTENCE #####################################################
#makes the conversion in an Xbar-like constituent-structure 
#NB: the capital letters between [] are references to the Fei Xia algorithm

sub translatesentence{
   	my ($ref1,$ref2,$ref3) = @_;  	#dereferenziazione degli array argomenti
   	my @D_tree = @{$ref1};             	#l'albero di dipendenze (input)
   	my @arg_table = @{$ref2};          	#la tabella degli argomenti
   	my @mod_table = @{$ref3};        	#la tabella dei modificatori
   	my @T_tree = ();		      		#l'albero di costituenti
   	my $root_T = '';				#la radice dell'albero a costituenti
  	my $root_D = dependencyroot(@D_tree);   #la radice di D_tree (posiz/parola/categoria/padre/relaz)
   	my $numfigli_root_D = scalar(figli($root_D,@D_tree));  #il numero di figli di root_D

#[A] build the root node for the constituency tree ################################
   	if (($root_D =~ m/OPEN\+/) or ($root_D =~ m/CLOSE\+/)){
		push(@balancedpunct,$root_D); 	#mette in un array le punteggiature bilanciate, che tratta dopo (sub bilanciapunti)
   	}
   	else{if ($root_D =~ m/SEPARATOR/){		#mette in un array le punteggiature che tratta dopo (sub inserisciseparator)
	  			push(@separator,$root_D);
	     }
	     else{$root_T = build_node($root_D); 	#costruisce il nodo T per il formato a costituenti e
			if (($root_T =~ m/\-TOP/) and ($root_T !~ m/\-TOPIC/)){
				$root_T = (substr $root_T,0,(index $root_T,' ')).(substr $root_T,(index $root_T,' '));
			}
			push(@T_tree,$root_T);    			#lo mette in T_tree  
         }
   	}

#[B] if the root is a leaf, end ###########################################################
   	if ($numfigli_root_D == 0){		#se il numero di figli di root_D e' zero
		return @T_tree;		#esce da translate restituendo il T_tree
   	}
  	else{	#questo e' else grande

#[C] find the projection chain for the grammatical category (X) of the $root_D of the dependendy tree #################
	     	my $X_chain = proiezione($root_D,$root_T);

#[D] fix the position of the root_T of the constituency tree on the projection chain of its category (X) ###########
	  		my $X_chain_root_index = 2;    #fissa la posizione sulla catena
   			$root_T = proietta($X_chain,$X_chain_root_index);  #assegna a root_T la proiezione relativa a X_chain_root_index

#[E] begin the attachment of the left dependents in the spine of the T_tree #####################
	   		$X_chain_lowvis_index = 2;  #fissa la posizione del nodo piu' basso visibile di X_chain
				  #e' il nodo piu' basso a cui si possono fare attacchi
				  #deve essere aggiornato man mano che i dipendenti si attaccano

#[F] continues the attachment #####################################################################
			my @dependent_sin = dipsinistra($root_D,figli($root_D,@D_tree));  #dipendenti sinistri di root_D
 			my $loop_index_sin = 0;						  #ordinati da destra a sinistra
 	 		for ($loop_index_sin = scalar(@dependent_sin); $loop_index_sin >= 1; $loop_index_sin--){
   					my $word = shift(@dependent_sin);           #il dipendente da attaccare avente categoria Y
      				my $Y_chain = proiezione($word,$word);		 #la catena di proiezione di Y
					my $word_dep_type = dependency_type($word); #il tipo di dipendenza di word (arg o mod)
   					my @word_D_tree = ();                    #l'albero di dipendenze di cui word e' radice
	   				my $trovato = 'no';  			 #$trovato serve a far pulire il D_tree solo
   					my @word_D_tree = subtree($word,$trovato,@D_tree);  #quando si calcola un sottoalbero nuovo
					my @word_T_tree = ();
					my @word_T_tree = translatesentence(\@word_D_tree,\@arg_table,\@mod_table);
					@T_tree = attach_dep_to_head(\'LEFT',\$word_dep_type,\$X_chain,\$Y_chain,\@T_tree,\@word_T_tree,\@arg_table,\@mod_table);#
			}
	
#[G] begin the attachment of the right dependents in the spine of the T_tree ###########################
			$root_T = proietta($X_chain, $X_chain_root_index);  #assegna a root_T la proiezione relativa a X_chain_root_index

#[H] continues the attachment #####################################################################
	    	my @dependent_des = dipdestra($root_D,figli($root_D,@D_tree));  #dipendenti di destra di root_D
    		my $loop_index_des = 0;						#ordinati da sinistra a destra
    		for ($loop_index_des = scalar(@dependent_des); $loop_index_des >= 1; $loop_index_des--){
    			my $word = shift(@dependent_des);  	 #il dipendente da attaccare avente categoria Y
				my $Y_chain = proiezione($word,$word);	 #la catena di proiezione di Y
				my $word_dep_type = dependency_type($word); #il tipo di dipendenza di word (arg, mod o coord)
				my @word_D_tree = ();                   #l'albero di dipendenze di cui word e' radice
				my $trovato = 'no';                         #$trovato serve a far pulire il D_tree solo
				my @word_D_tree = subtree($word,$trovato,@D_tree);  #quando si calcola un sottoalbero nuovo
				my @word_T_tree = ();
				my @word_T_tree = translatesentence(\@word_D_tree,\@arg_table,\@mod_table);
				@T_tree =  attach_dep_to_head(\'RIGHT',\$word_dep_type,\$X_chain,\$Y_chain,\@T_tree,\@word_T_tree,\@arg_table,\@mod_table);#
			}
	}#chiude else grande
  	return @T_tree;
}


####################################### SUBROUTINE of TRANSLATESENTENCE #############################

########### build_node ##############################################################################
#for each of the words that takes as input in tut format (previously modified by the preprocessor), it builds the node for the 
#constituency tree

sub build_node{
  	my $D_node = shift(@_);
    my $T_node = '';
	my $gramcategory = '';
	my $extrainfo = '';
	my $subextrainfo = '';
	my $top = '';
	$extrainfo = substr $D_node,((rindex $D_node,'/') + 1);
	if ($extrainfo =~ m/\%/){
		$extrainfo =~ s/VERB-//g;
		$extrainfo = '-'.$extrainfo;
		if ($extrainfo =~ m/-AGENT/){	#se e' una trasformazione
			$extrainfo = '-LGS';
		}
		if ($extrainfo =~ m/OBJ\%SUBJ/){	#se e' una trasformazione
			$extrainfo = '-SUBJ';
		}
	}
	else{if (($extrainfo =~ m/TOP\-/) or ($extrainfo =~ m/AUX/)){	#se e' top 
				if ($extrainfo =~ m/TOP/){
						$top = 'si';
						$extrainfo = '-TOP';}
				else{$extrainfo = '';
				}
	       }
	       else{if ($extrainfo !~ m/\-/){	#altrimenti elimina tutta la parte morfo di extrainfo
						$extrainfo = '-'.$extrainfo;  #se extrainfo non contiene trattino tiene tutto
			      }
		    	  else{$subextrainfo = substr $extrainfo,(index $extrainfo, '-');#se contiene 2 trattini toglie cio' che precede il primo trattino
						if (($extrainfo =~ m/\-/) and ($subextrainfo =~ m/\-/)){
							$extrainfo = $subextrainfo;
						}			#se contiene 1 trattino toglie la prima parte solo se e' morfologica
						else{if (($extrainfo =~ m/^DET/) or ($extrainfo =~ m/^VERB/) or ($extrainfo =~ m/^NOUN/)
				        	   or ($extrainfo =~ m/^PREP/) or ($extrainfo =~ m/^PRON/) or ($extrainfo =~ m/^ADVB/)
							   or ($extrainfo =~ m/^CONJ/) or ($extrainfo =~ m/^PREDET/) or ($extrainfo =~ m/^ADJC/)
							   or ($extrainfo =~ m/^NUM/)){
									$extrainfo = $subextrainfo;
 				       		   }
						       else{$extrainfo = $extrainfo;
						       }
						}
				  }
		   }
	}
	if ($extrainfo =~ m/-SUBJ/){
		$extrainfo = 'SUBJ';
	}
	chomp($extrainfo); #elimina eventuali a capo nella stringa
	$D_node = substr $D_node,0,(rindex $D_node,'/');  #estrae dal nodo posizione/parola/categoria/padre
	$D_node = substr $D_node,0,(rindex $D_node,'/');  #estrae posizione/parola/categoria
	$gramcategory = substr $D_node,((rindex $D_node,'/') + 1);#assegna a category la categoria del nodo
	if ($extrainfo eq ''){
			$T_node = '('.$gramcategory.' '.$extrainfo.(substr $D_node,0,(rindex $D_node,'/')).')';
	}
	else{$T_node = '('.$gramcategory.$extrainfo.' '.(substr $D_node,0,(rindex $D_node,'/')).')';
	}
	return $T_node;
}


############## attach_dep_to_head ###################################################################
#attaches the phrase built for the node Y to the phrase built for the head X on which Y depends

sub attach_dep_to_head {
	my ($ref1,$ref2,$ref3,$ref4,$ref5,$ref6,$ref7,$ref8) = @_;   #dereferenziazione degli argomenti
	my $dep_pos = $$ref1;
    my $dep_type = $$ref2;
	@T_tree = @{$ref5};
	my @dep_T_tree = @{$ref6};
	my @arg_table = @{$ref7};
	my @mod_table = @{$ref8};
	my $list1 = '';
	my $list2 = '';

#[A] setting of the parameters of the head #########################################
	my $X_chain = $$ref3;		#catena di proiezione della categoria (X) della radice dell'albero testa
	my $root_T = root(@T_tree);     	#radice dell'albero testa 
 	my $X_proj_cat = proietta($X_chain,$X_chain_lowvis_index); 	#proiezione della categoria X

#[B]  setting of the parameters of the dependent ######################################
	my $Y_chain = $$ref4;	#proiezione della categoria (Y) radice dell'albero dipendente
	my $root_dep_T = root(@dep_T_tree);		#radice dell'albero dipendente
	my $Y_proj_cat = nome_phrase($root_dep_T);	#proiezione di Y disponibile per l'attacco
								                #= posizione che coincide con la categoria sintagmatica del dipendente
	$list1 = substr $Y_chain,(index $Y_chain,nome_phrase($root_dep_T));  #la catena di proiezione di Y a partire
	$list1 = substr $list1,0,(rindex $list1,')');                        #dal sintagma radice del dipendente
	if (($X_chain eq '') or ($Y_chain eq '')){
		print "a projection chain is missing:\n",$X_chain,"=Xchain for ",$root_T,"\n",$Y_chain,"=Ychain for ",$root_dep_T,"\n";}

#[C] look for the lower possible attachment for the dependent on the projection chain of the head

# TRATTAMENTO DELLA COORDINAZIONE
if ($dep_type !~ m/COORD/){#####APRE IF DELLA NON-COORD
	my $loop_index = 0;
	for ($loop_index = position_in_chain($X_chain,$X_proj_cat); $loop_index <= (chain_length($X_chain)); $loop_index++){
			#il loop va da B = la posizione che sta immediatamente sopra la categoria sintagmatica dell'albero testa nella
			#catena di proiezione X_chain, a K = l'ultima posizione di X_chain
		$list2 = '';		#svuota list2
		if ($dep_type =~ m/ARG/){		#se il dipendente e' argomento della testa (allora deve stare sotto la testa)
			$list2 = proietta($X_chain,($loop_index -1)); 	#trova la categoria sintagmatica avente posizione = loop-index dentro X_chain
			$list2 = arg_list(\$list2,\@arg_table,\$dep_pos);	#calcola la lista di argomenti associati nella arg_table a tale categoria
		}
		else{$list2 = proietta($X_chain,$loop_index); 	#altrimenti, se il dipendente e' modificatore della testa (sta al suo stesso livello), trova la categoria sintagmatica avente
			$list2 = mod_list(\$list2,\@mod_table,\$dep_pos);    #posizione = loop-index - 1 dentro X_chain, e la relativa lista di 	modificatori in mod_table
		}
		my $voidstring = '';
	        if (($list1 ne $voidstring) and ($list2 ne $voidstring)){	#se le due liste non sono vuote
	        	my $testconfronta = 'no';
				my $confronto = confronta($list1,$list2,$testconfronta);	#le confronta per trovare il primo elemento comune
				if ($confronto ne 'no'){					#se trova tale elemento
						$X_proj_cat = proietta($X_chain,$loop_index);		#aggiorna
						$Y_proj_cat = $confronto;
					$loop_index = 100;
			  }
		}
	}#chiude il loop

#[D] further projection of the dependent on the projection chain (Y_chain) of the head, if needed ##########################
	my $counter1 = 0;
	my $pos_in_dep_T = '';
	my $old_root_dep_T = '';
	while ((position_in_chain($Y_chain,nome_phrase($root_dep_T)))
		< (position_in_chain($Y_chain,$Y_proj_cat))){ #se la categoria radice del dipendente e' piu' in basso della proiezione 											
#necessaria per l'attacco, esce dal while quando le 2 position sono uguali
		if (pos_in_ttree($root_dep_T) =~ m/\-/){#se il nodo e' non terminale
			$counter1 = pos_in_spina($root_dep_T);   #substr $counter1, 0, (index $counter1, '/');
			$counter1= ++$counter1;
		}
		else{$counter1 = ++$counter1;
		}
		$pos_in_dep_T = count_index($root_dep_T,$counter1);
		$old_root_dep_T = $root_dep_T.'['.$pos_in_dep_T.']'; #assegna a oldroot la radice + il riferimento al suo padre (=nuova radice di depT)
		@dep_T_tree = sostituisciinarray($root_dep_T,$old_root_dep_T,@dep_T_tree); 	#mette il link alla nuova root nella vecchia
		$root_dep_T = '('.proietta($Y_chain,(position_in_chain($Y_chain,nome_phrase($root_dep_T))) + 1).' '.$pos_in_dep_T.'/'.')';
		$root_dep_T = propagarelaz($root_dep_T,$old_root_dep_T);
		@dep_T_tree = sostituisciinarray($old_root_dep_T,(puliscirelaz($old_root_dep_T)),@dep_T_tree);
		push(@dep_T_tree,$root_dep_T); 	#mette la nuova root nell'array che contiene l'albero
	}

#[E] further projection of the head on the projection chain (X_chain) if needed ###############################
	my $counter = 0;
	my $pos_in_T = '';
	my $old_root_T = '';
	my $counter2 = 0;
	my $fatherofdep = '';
	while ((position_in_chain($X_chain,nome_phrase($root_T))) < ((position_in_chain($X_chain,$X_proj_cat)) )){#- 1)){
			#se la categoria radice della testa sta piu' in basso della proiezione necessaria per l'attacco
	if (($root_T =~ m/\/\)/) and ($root_T !~ m/\*\[/)){#se la radice della testa e' non terminale e non e' traccia
			$counter2 = substr $root_T, ((rindex $root_T, '-') + 1);
			$counter2 = substr $counter2, 0, (index $counter2, '/');
			$counter2= ++$counter2;
		}
		else{$counter2 = ++$counter2;
		}
		$pos_in_T = count_index($root_T,$counter2);	#calcola l'indice numerico da attribuire al nodo
		$old_root_T = $root_T.'['.$pos_in_T.']';	#assegna a oldroot la radice di T + il riferimento al suo padre (=nuova radice di T)
		@T_tree = sostituisciinarray($root_T,$old_root_T,@T_tree);		#mette il link alla nuova root nella vecchia
		$root_T = '('.proietta($X_chain,((position_in_chain($X_chain,nome_phrase($root_T)))) + 1).' '.$pos_in_T.'/'.')';	#costruisce la nuova root
		$root_T = propagarelaz($root_T,$old_root_T);
		@T_tree = sostituisciinarray($old_root_T,(puliscirelaz($old_root_T)),@T_tree);
		push(@T_tree,$root_T);	#mette la nuova root nell'array che contiene l'albero
		if (($root_T =~ m/\-TOP/) and ($root_T !~ m/CONJP/)){		#nel caso della root che ha relazione TOP si forza una proiezione ulteriore
				$pos_in_T = count_index($root_T,++$counter2);	#calcola l'indice numerico da attribuire al nodo
				$old_root_T = $root_T.'['.$pos_in_T.']';	#assegna a oldroot la radice di T + il riferimento al suo padre (=nuova radice di T)
				@T_tree = sostituisciinarray($root_T,$old_root_T,@T_tree);		#mette il link alla nuova root nella vecchia
				$root_T = '('.proietta($X_chain,((position_in_chain($X_chain,nome_phrase($root_T)))) + 1).' '.$pos_in_T.'/'.')';	#costruisce la nuova root
				$root_T = propagarelaz($root_T,$old_root_T);
				@T_tree = sostituisciinarray($old_root_T,(puliscirelaz($old_root_T)),@T_tree);
				push(@T_tree,$root_T);	#mette la nuova root nell'array che contiene l'albero
		}
	}
    $counter = ++$counter2;
	$pos_in_T = count_index($root_T,$counter);	#calcola l'indice numerico da attribuire al nodo
    if ((position_in_chain($X_chain,(nome_phrase($root_T)))) >= (position_in_chain($X_chain,$X_proj_cat))){
			$fatherofdep = $X_proj_cat;
	}
	else{$old_root_T = $root_T.'['.$pos_in_T.']';	#assegna a oldroot la radice di T e gli aggiunge il riferimento al suo padre (=nuova radice di T)
			$fatherofdep = $X_proj_cat;
			@T_tree = sostituisciinarray($root_T,$old_root_T,@T_tree);	#mette il link alla nuova root nella vecchia
			$root_T = '('.$X_proj_cat.' '.$pos_in_T.'/'.')';	#costruisce la nuova root
			$root_T = propagarelaz($root_T,$old_root_T);
			@T_tree = sostituisciinarray($old_root_T,(puliscirelaz($old_root_T)),@T_tree);
			push(@T_tree,$root_T);				#mette la nuova root nell'array che contiene l'albero
        	$fatherofdep = $root_T;
	}
	
#[F] update the value for lowvisibindex and of the root on x-chain ###############################
	$X_chain_root_index = $X_chain_lowvis_index;

#[G] attachment of the subtree of the dependent (dep_T) to the tree on the head (T) #####################
	foreach $item(@dep_T_tree){	#cerco la testa dell'albero (a costituenti) dipendente
		if ($item eq $root_dep_T){
				if ($fatherofdep !~ m/\/\)/){    #se il nodo a cui l'albero dipendente va attaccato non e'  foglia prende la posizione del costituente nell'albero
						$pos_in_T = trova_father_in_chain($fatherofdep,$X_chain,$root_T,@T_tree);
				}
				else{#altrimenti prende la posizione della parola nella frase
						$pos_in_T = substr $fatherofdep,((index $fatherofdep,' ') + 1),
					  	((index $fatherofdep,'/') - (index $fatherofdep,' ') - 1);
				}
 				push(@T_tree,($item.'['.$pos_in_T.']'));
		}
		else{	push(@T_tree,$item);
		}
	}
}#####CHIUDE IF DELLA NON-COORD
else{####APRE ELSE DELLA COORD
#inserisce un nodo nuovo (che e' un duplicato del nodo testa del primo coordinato) come testa dell'intera coordinazione
#prima, nel while, costruisce la catena fino alla proiezione max del primo coordinato
#poi sopra la proiezione max costruisce il nodo nuovo e gli attacca i due coordinati
		while ((nome_phrase($root_T) ne 'S') and (nome_phrase($root_T) ne 'VP') and (nome_phrase($root_T) ne 'DP') and
			    (nome_phrase($root_T) ne 'NP') and (nome_phrase($root_T) ne 'PP') and
	    	    (nome_phrase($root_T) ne 'ADJP') and (nome_phrase($root_T) ne 'ADVP') and
	    	    (nome_phrase($root_T) ne 'PRN') and (nome_phrase($root_T) ne 'PPSUBJ') and 
	    	    (nome_phrase($root_T) ne 'DPSUBJ') and (nome_phrase($root_T) ne 'SSUBJ') and 
	    	    (nome_phrase($root_T) ne 'CONJP') and  (nome_phrase($root_T) ne 'NPSUBJ')){	
	    	    #proietta la radice  del primo coordinato fino alla proiez massim
						$pos_in_T = (spina($root_T)).'-'.((pos_in_spina($root_T) + 1));	#calcola l'indice numerico da attribuire alla nuova root
						$old_root_T = $root_T.'['.$pos_in_T.']';	#a cui attacca la root ora obsoleta (=oldroot)
						@T_tree = sostituisciinarray($root_T,$old_root_T,@T_tree);
						$root_T = '('.proietta($X_chain,((position_in_chain($X_chain,nome_phrase($root_T)))) + 1).' '.$pos_in_T.'/)';
						$root_T = propagarelaz($root_T,$old_root_T); 	#costruisce il nodo nuovo (=root)
						@T_tree = sostituisciinarray($old_root_T,(puliscirelaz($old_root_T)),@T_tree);
						push(@T_tree,$root_T);	#mette la nuova root nell'array che contiene l'albero
				}#costruisce il nodo nuovo duplicato della root del primo coordinato come testa della coordinazione
				$pos_in_T = (spina($root_T)).'-'.(pos_in_spina($root_T) + 1);	#calcola l'indice numerico da attribuire al nodo nuovo
				$old_root_T = $root_T.'['.$pos_in_T.']';	#attacca al nodo nuovo la root (=radice del primo coordinato)
				@T_tree = sostituisciinarray($root_T,$old_root_T,@T_tree);
				$root_T = '('.nome_phrase($root_T).' '.$pos_in_T.'/'.')'; 	#costruisce il nodo nuovo
				$root_T = propagarelaz($root_T,$old_root_T);
				@T_tree = sostituisciinarray($old_root_T,(puliscirelaz($old_root_T)),@T_tree);
				push(@T_tree,$root_T);	#mette la nuova root nell'array che contiene l'albero
				$new_root_dep_T = $root_dep_T.'['.$pos_in_T.']';	#attacca la radice del depTtree al nodo nuovo
				foreach $item(@dep_T_tree){	#mette nel Ttree tutti gli elementi del depTtree (=dipendenti del secondo coordinato)
				push(@T_tree,$item);
		}
		@T_tree = sostituisciinarray($root_dep_T,$new_root_dep_T,@T_tree);
	}####CHIUDE ELSE DELLA COORD
	return @T_tree;
}



#### root ########################################################################################
#find the root of the dependency tree

sub root{
	my @albero = @_;
	my $trovato = '';
	foreach $nodo(@albero){
		if ($nodo !~ m/\]$/){
				$trovato = $nodo;
		}
	}
return $trovato;
}


#### nome_phrase ###############################################################################
#give a node, output the name of the phrase of such a node

sub nome_phrase{
	my $phrase = shift(@_);
	my $nome = substr $phrase,1,((index $phrase,' ') - 1);
	if (($nome =~ m/\-/) and ($nome !~ m/PUNCT\-COORD/)){
		$nome = substr $nome,0,(index $nome, '-');
	}
	else{if ($nome =~ m/PUNCT\-COORD/){
			$nome = 'PUNCTCOORD+BASE';
		}
	}
return $nome;}


#### count_index ###############################################################################
##give a node and a counter, output the index that should be used to link the node to the father

sub count_index{
	my $root = shift(@_);
	my $counter = shift(@_);
	my $index = '';
	$index = substr $root,(index $root,' '); 	#trova l'indice numerico della radice (la stringa compresa tra spazio e /)
	$index = substr $index,1,((index $index,'/') - 1);
	if ($index !~ m/-/){
		$index = $index.'-'.$counter;		#gli aggiunge - e livello di proiezione (counter)
	}
	else{ $index = (substr $index,0,(index $index,'-')).'-'.$counter;
	}
return $index;
}



############# trova_father_in_chain($fatherofdep,$X_chain,@T_tree) #####################################
#gievn a phrase category, find it in the tree spineof the tree T

sub trova_father_in_chain {
	my $fatherofdep = shift(@_);
	my $X_chain = shift(@_);
	my $root = shift(@_);
	my @T_tree = @_;
	my $posoffather = '';
	my @trovati = ();
	my @stessaspina = ();
	my $indexroot = '';
	my $indextrov = '';
	foreach $el(@T_tree){		#se el e' un sintagma dello stesso tipo di quello cercato come father
		if (nome_phrase($el) eq $fatherofdep){		#lo mette in un array trovati
				push(@trovati,$el);
		}
	}
	if (scalar(@trovati) == 1){		#se trova un solo candidato father 
			$posoffather = @trovati[0];		#lo restituisce
			$posoffather = substr $posoffather,((index $posoffather,' ') + 1),((index $posoffather,'/') - (index $posoffather,' ') - 1);
	}
	else{if (scalar(@trovati) > 1){		#altrimenti, se esistono piu' candidati father
		   			foreach $trov(@trovati){		#per ognuno cerca se la spina e' la stessa del figlio
							if (spina($trov) eq spina($root)){	#mette ogni candidato con stessa spina del figlio dentro l'array stessaspina
									push(@stessaspina,$trov);
							}
					}
					if (scalar(@stessaspina) == 1){		#se esiste un solo candidato che appartiene alla stessa spina
							$posoffather = @stessaspina[0];		#lo restituisce
							$posoffather = substr $posoffather,((index $posoffather,' ') + 1),((index $posoffather,'/') - (index $posoffather,' ') - 1);
					}
					else{if(scalar(@stessaspina) == 0){	#altrimenti, se nessuno dei trovati appartiene alla stessa spina
									$posoffather = trovanearest(@$root,@trovati);	#confronta gli elementi di trovati sulla vicinanza a root
						  }
						  else{if (scalar(@stessaspina) > 1){		#altrimenti se piu' di uno dei trovati appartiene alla stessa spina
						  				foreach $spi(@stessaspina){
												$posoffather = trovanearest($root,@stessaspina);#confronta gli elementi di stessaspina sulla vicinanza al figlio
						  				}
						  		}
						  }	  
					}
				
			}
	}
return $posoffather;
}


############trovanearest ###########################################################
#given a node and a tree in form of array, finds the element of the array that is closer to the node, or the lower

sub trovanearest{
	my $node = shift(@_);
	my @array = @_;
	my $result = '';
	my @positions = ();
	foreach $el(@array){
			push(@positions,pos_in_ttree($el));
	}
	@sortedpositions = sort {comparpos($a,$b)}@positions; #ordina in base alla posizione
	$result = @sortedpositions[0];	
	return $result;
}


sub comparpos{return (pos_in_ttree(shift(@_)) <=> pos_in_ttree(shift(@_)));
}



##### dependencyroot #####################################################################################
#find the root node of the dependency tree

sub dependencyroot{
	my @albero = @_;
	my $trovato = '';
	my $nodo = '';
	my $posizione_nodo = '';
	my $scartato = '';
	if (scalar(@albero) == 1){		#se l'albero contiene un solo elemento X, allora X ? la radice
		$trovato = shift(@albero);
	}
	else{						#altrimenti
	foreach $nodo(@albero){		#si confrontano i nodi dell'albero
		$posizione_nodo = substr $nodo,0,(index $nodo,'/');
		foreach $item(@albero){
			if ($posizione_nodo eq pospadre($item)){	#e per ogni nodo che risulta avere un padre nell'albero
				@albero = sostituisciinarray($item,$item.'*scartato*',@albero);	#si scrive *scartato*
				}
			}
		}
		while (scalar(@albero) > 0){				#facendo un ciclo while sui nodi dell'albero
			$scartato = shift(@albero);
			if ($scartato !~ m/\*scartato\*/){		#si trova l'unico nodo che non contiene *scartato*
				$trovato = $scartato;			#(l'unico che non ha padre nell'albero)
			}
		}
	}
return $trovato;
}


###### figli ########################################################################################
#find all the children of a node in a given tree

sub figli{
	my $arg =  $_[0];
	my @frase = @_[1..$#_];
	my $posiz = '';
	my @arrayfigli = ();
	$posiz = substr $arg, 0, (index $arg, '/');
	foreach $elem (@frase){                #per ogni elemento della frase
		$padreelem = substr $elem, 0, (rindex $elem, '/');    #calcola la posizione del padre
		$padreelem = substr $padreelem, ((rindex $padreelem, '/') + 1);
		if ($padreelem eq $posiz){     #e la confronta con la posizione di $arg
			push(@arrayfigli,$elem);  #se corrispondono mette l'elemento nell'array dei figli
		}
	}
return @arrayfigli;
}


######### padre ##################################################################################
#given a line in tut format and a sentence as an array, output the line of the head word of the given line in the sentence

sub padre{
	my $arg =  shift(@_);
	my @frase = @_;
	my $padre = '';
	my $pospadrediarg = '';
	$pospadrediarg = substr $arg,((rindex $arg, '[') + 1);    #calcola la posizione del padre
	$pospadrediarg = substr $pospadrediarg, 0,(index $pospadrediarg, ']');
	foreach $item(@frase){
		$positem = substr $item, ((index $item, ' ') + 1);
		$positem = substr $positem,0,(index $positem,'/');
		if ($positem eq $pospadrediarg){
			$padre = $item;
		}
	}
return $padre;
}


######### pospadre ##################################################################################
#find the position of the father

sub pospadre{
	my $arg =  $_[0];
	$pospadre = substr $arg, 0, (rindex $arg, '/');
	$pospadre = substr $pospadre, ((rindex $pospadre, '/') + 1);
return $pospadre;
}


######### rel #######################################################################################
#find the relation linking the node with its father

sub rel{
	my $arg =  $_[0];
	$rel = substr $arg, ((rindex $arg, '/') + 1), (length($arg) - (rindex $arg, '/'));
return $rel;
}


######## dipsinistra ##############################################################################
#given a node and the array of its children, output an array which includes only the left dependents of the node 

sub dipsinistra {
	my $nodo = shift(@_);
	my @figli = @_;
	my $posnodo = substr $nodo,0,(index $nodo,'/');  #posizione del nodo
	my @figlisinistra = ();
	foreach $figlio(@figli){
		if ((substr $figlio,0,(index $figlio,'/')) < $posnodo){  #se la posizione del figlio ? minore di quella del padre
			push (@figlisinistra,$figlio);                   #mette il figlio nell'array
		}
	} #ordina i figli di sinistra partendo da quello pi? vicino al padre (cio? con index maggiore)
	@figlisinistra = sort {(substr $b,0,(index $b,'/')) cmp (substr $a,0,(index $a,'/'))} @figlisinistra;
return @figlisinistra;
}



######## dipdestra ##############################################################################
#given a node and the array of its children, output an array which includes only the right dependents of the node 

sub dipdestra {
	my $nodo = shift(@_);
	my @figli = @_;
	my $posnodo = substr $nodo,0,(index $nodo,'/');  #posizione del nodo
	my @figlidestra = ();
	foreach $figlio(@figli){
		if ((substr $figlio,0,(index $figlio,'/')) > $posnodo){  #se la posizione del figlio ? maggiore di quella del padre
			push (@figlidestra,$figlio);                   #mette il figlio nell'array
		}
	} #ordina i figli di destra partendo da quello pi? vicino al padre (cio? con index minore)
	@figlidestra = sort {(substr $a,0,(index $a,'/')) cmp (substr $b,0,(index $b,'/'))} @figlidestra;
return @figlidestra;
}


###### subtree #######################################################################################
##given a node and a tree, output the subtree of the node 

sub subtree{
	my $nodo = $_[0];
	$trovato = $_[1];   #$trovato `e inizializzata all'esterno
	my @albero = @_;
	if ($trovato eq 'no'){  #solo quando si entra in subtree da una chiamata esterna
		@subtree = ();  #l'array del subtree deve essere vuoto, ma deve non deve essere
	}                       #svuotato nelle chiamate ricorsive
	$trovato = 'si';
	if (scalar(figli($nodo,@albero)) > 0){     #se il nodo ha figli
		push(@subtree,$nodo);              #viene messo nell'array subtree e la routine
		foreach $figlio(figli($nodo,@albero)){     #viene richiamata su ogni suo figlio
			subtree($figlio,@albero);
		}
	}
	else{push(@subtree,$nodo);                 #se invece il nodo non ha figli, viene messo nell'array
	}
return @subtree;
}



###### proiezione #####################################################################################
#given a node of a dependency tree, find the morphological category of the node and the projection chain looking in 
#the head-pro-table

sub proiezione {
	my $arg = shift(@_);
	my $secondarg = shift(@_);
	my @headp_table = ();
	my $proiez = '';
	my $projectedcateg = '';
	open(FILEHANDLEheadp, '<TUTtoPENN1_0-head-pro-table.txt');  #legge la tabella di proiezione della testa
	while (<FILEHANDLEheadp>){         	   #e mette i dati in un array
		$ccc = $_;
		push (@headp_table,$ccc);}
	close(FILEHANDLEheadp);
	my $category = substr $arg,((index $arg,'/') + 1);   	#trova la categoria del nodo dato come argomento
	$category = substr $category,((index $category,'/') + 1);
	my $posslash = index $category,'/';
	$category = substr $category,0,(index $category,'/');
	#if (($secondarg !~ m/IMPERS/) and 
	
		if (($secondarg !~ m/AGENT/) and ($secondarg !~ m/PREDCOMPL/) and
	   ($secondarg !~ m/RMODPRED/) and ($secondarg !~ m/EXTRAPOSED/) and
	    ($secondarg =~ m/SUBJ/)){
		$category = $category.'SUBJ';
	}
	if ($secondarg =~ m/END/){
		$category = $category.'END';
	}
	if (($secondarg =~ m/PUNCT/) and ($secondarg =~ m/COORD\+BASE/)){
		$category = $category.'COORD+BASE';
	} #questa modifica del 16 agosto 05 serve solo a gestire i punti che fungono da coord, distinguendoli dagli altri punct, 
	#non gestisce le coord
	foreach $item(@headp_table){                            	#per ogni proiezione registrata nella table
		$projectedcateg = substr $item,1,((index $item,',') - 1);     #trova la categoria della proiezione
		if ($projectedcateg eq $category){		#e se questa corrisponde alla categoria del
			$proiez = $item;                   	#nodo la restituisce (NB: la table contiene
		}                         			#una sola proiezione per ogni categoria)
	}
return $proiez;
}



####### proietta ######################################################################################
#given a projection chain (like (xxx, yyy, zzz)) and a number n, output the n element of the chain , and if the chain is shorter
#than n output the last element of the chain

sub proietta {
	my $chain = shift(@_);
	my $num = shift(@_);
	my $posclosetonda = (rindex $chain,')') - 1;
	my $n_proiezione = substr $chain,1,$posclosetonda;
	my $posfinale = 0;
	my $loop_index = 0;
	for ($loop_index = 1; $loop_index <= $num; $loop_index++){
		if ($n_proiezione !~ m/,/){      #se la proiezione non contiene piu' virgole vuol dire
		       $n_proiezione = $n_proiezione;     #che ha raggiunto l'ultimo elemento della catena
		}
		else{$n_proiezione = substr $n_proiezione,$posfinale;   #altrimenti va avanti sull'elemento
		     $posfinale = (index $n_proiezione,',') + 1;        #successivo della catena
		}
	}
	if ($n_proiezione =~ m/,/){                     #se la proiezione contiene virgole cancella tutto
		$posfinale = index $n_proiezione, ',';  #quello che segue la prima virgola
		$n_proiezione = substr $n_proiezione,0,$posfinale;
	}
return $n_proiezione;
}


###### position_in_chain ####################################################################################
#given a projection chain and a phrase category, output the number that corresponds to the position of the phrase category 
#in the projection chain

sub position_in_chain{
	my $chain = shift(@_);
	$chain = substr $chain, 1; 	#elimina la parentesi iniziale della catena
	my $syntcategory = shift(@_);
	my $position = 0;
	my $firstinchain = '';
	my $test = ')';
	while ($chain ne $test){		#fino a che la catena non e' vuota
		$position = ++$position;	#aggiorna il valore di position
		$firstinchain = substr $chain,0,(index $chain,',');	#assegna a firstinchain la prima categoria di chain
		if ($firstinchain =~ m/\)/){    #se firstinchain e' l'ultimo elemento della catena deve rimuovere anche la parentesi
			$firstinchain = substr $firstinchain,0,(index $firstinchain,')');
		}
		if ($syntcategory eq $firstinchain){		#se syntcategory corrisponde a firstinchain
			$chain = ')';				#esce
		}
		else{ if ($chain =~ m/\,/){
				$chain = substr $chain, ((index $chain,',') + 1);	#toglie il primo elemento e la prima virgola dalla catena
			}
			else{$chain = ')';}
		}
	}
return $position;
}


###### chain_length ########################################################################################
#given a chain, count the number of elements of the chain

sub chain_length{
	my $chain = shift(@_);
	my $test = ')';
	my $length = 0;
	while ($chain ne $test){	#fino a che chain non ? vuota
		$length = ++$length;	#aggiorna il valore di length
		if ($chain =~ m/\,/){	#se chain contiene virgole (= ha pi? di una categoria sintagmatica)
			$chain = substr $chain, ((index $chain,',') + 1);	#toglie il primo elemento e la prima virgola dalla catena
		}
		else{$chain = ')';		#altrimenti esce
		}
	}
return $length;
}


###### dependency_type #####################################################################################
#given a node of the dependency tree, finds the name of the grammatical relation annotated in the node, read in the 
#rel-table and output modifier or argument

sub dependency_type {
	my $arg = shift(@_);
	chomp $arg;
	my $relaz_arg = rel($arg);
	chomp($relaz_arg);
	my @relaz_table = ();
	my $type = '';
	open(FILEHANDLErel, '<TUTtoPENN1_0-rel-table.txt');  #legge la tabella delle relazioni, dove e' specificato per
	while (<FILEHANDLErel>){            #ogni rel se e' arg o mod, e mette i dati in un array
			$ccc = $_;
			push (@relaz_table,$ccc);
	}
	close(FILEHANDLErel);
	my $relaz = shift(@relaz_table);
	my $name_relaz = substr $relaz, 0, (rindex $relaz, '>');
	chomp($name_relaz);
	my $type_relaz = substr $relaz, ((rindex $relaz, '>') + 1);
	chomp($type_relaz);
	my $trovato = 0;                    #inizializza la var di uscita dal while
	while ($trovato == 0){
	       	if($relaz_arg eq $name_relaz){
					$trovato = ++$trovato;
					$type = $type_relaz;
			}
			else {$relaz = shift(@relaz_table);
					$name_relaz = substr $relaz, 0, (rindex $relaz, '>');
					chomp($name_relaz);
					$type_relaz = substr $relaz, ((rindex $relaz, '>') + 1);
					chomp($type_relaz);
            }
	}
	return $type;
}


#### arg_list ##########################################################################################
#given a grammatical category and a position (left or right), output the list of the arguments that the category 
#can assume in such a position

sub arg_list{
	my ($ref1,$ref2,$ref3) = @_;
	my $syntcategory = $$ref1;
	my @arg_table = @{$ref2};
	my $dep_pos = $$ref3;
	my $list = '';
	my $itemcateg = '';
	my $choplist = '';
	my $assegnato = '';
	foreach $item(@arg_table){       					  #nella arg_table cerca per ogni riga
		$itemcateg = substr $item,1,((index $item,':') - 1);  #la categoria sintagmatica della riga
		if ($syntcategory eq $itemcateg){    			  #trova la riga relativa a syntcategory
			$list = $item;               					  #e la assegna a $list
			$assegnato = 'si';
		}
	}
	if ($assegnato eq 'si'){
		$list = substr $list,((index $list,'<') + 1);	#estrae da $list gli argomenti
		$chop = chomp($list);  #elimina i caratteri vuoti e le parentesi a fine lista
		$chop = chop($list);
		$chop = chop($list);
	}
return $list;
}


#### mod_list ##########################################################################################
#given a grammatical category and a position (left or right), output the list of the modifiers that the category 
#can assume in such a position

sub mod_list{
	my ($ref1,$ref2,$ref3) = @_;
	my $syntcategory = $$ref1;
	my @mod_table = @{$ref2};
	my $dep_pos = $$ref3;
	my $list = '';
	my $itemcateg = '';
	my $endleftmod = '';
	my $choplist = '';
	my $assegnato = '';
        foreach $item(@mod_table){       #nella mod_table cerca per ogni riga
		$itemcateg = substr $item,1,((index $item,':') - 1);  #la categoria sintagmatica della riga
		if ($syntcategory eq $itemcateg){    #trova la riga relativa a syntcategory
			$list = $item;               #e la assegna a $list
			$assegnato = 'si';
		}
	}
	if ($assegnato eq 'si'){
		if ($dep_pos eq 'LEFT'){
			$endleftmod = (index $list,'>RIGHT') - (index $list,'<') - 1;    #calcola la stringa dei mod di sinistra
        		$list = substr $list,((index $list,'LEFT<') + 5),$endleftmod;
        	}
        	else{$list = substr $list,((index $list,'RIGHT<') + 6);    #calcola la stringa dei mod di destra
		      $chop = chomp($list);  #elimina i caratteri vuoti a fine lista
		      $chop = chop($list);   #elimina la parentesi ) che chiude la lista argomenti
		      $chop = chop($list);   #elimina la parentesi > che chiude la lista
		}
	}
return $list;
}


#### confronta #########################################################################################
#given 2 list (which are the projection chain of a dependent and the list of the arguments that a head can assume),
#output 'no' if there are no common element, else the first common element of the two lists

sub confronta{
	my $list1 = shift(@_);
	my $list2 = shift(@_);
	my $ok = '';
	my $result = shift(@_);
	my @list1array = (); #crea un array con i valori contenuti nella lista1
	while ($list1 =~ m/\,/){
		$item = substr $list1,0,(index $list1,',');
		push (@list1array,$item);
		$list1 = substr $list1,((index $list1,',') + 1);
	}
	push (@list1array,$list1);
	my @list2array = (); #crea un array con i valori contenuti nella lista2
	while ($list2 =~ m/\,/){
		$item = substr $list2,0,(index $list2,',');
		push (@list2array,$item);
		$list2 = substr $list2,((index $list2,',') + 1);
	}
	push (@list2array,$list2);
	foreach $el(@list1array){
		foreach $ele(@list2array){
			if ($el eq $ele){
				$ok = 'ok';
				$result = $el;
				return $result;
			}
			else {$ok = 'errore nella proiezione';
			}
		}
	}
	if ($ok ne ''){
	return $result;
	}
	else {
	return $ok;
	}
}


#### sostituisciinarray ##################################################################################
#given A and B and and array, substitues B with A in the array (if occurs)

sub sostituisciinarray {
	my $vecchio = shift(@_);
    my $nuovo = shift(@_);
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elemento(@array){
		if ($elemento ne $vecchio){
			push (@arraytemp1,$elemento);
		}
		else {push (@arraytemp1,$nuovo);
		}
	}
	@array = @arraytemp1;
return @array;
}


#### sostituisci_unavolta_inarray ##################################################################################
#as the previous but only for the first occurrence of B

sub sostituisci_unavolta_inarray {
	my $vecchio = shift(@_);
    my $nuovo = shift(@_);
    my $unavolta = '';
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elemento(@array){
		if ($elemento ne $vecchio){
			push (@arraytemp1,$elemento);
		}
		else {if ($unavolta eq ''){
						push (@arraytemp1,$nuovo);
						$unavolta = 'si';
				}
				else{push (@arraytemp1,$elemento);}
		}
	}
	@array = @arraytemp1;
return @array;
}


#### cancelladaarray ##################################################################################
#given and argument and an array, removes the argument from the array

sub cancelladaarray {
	my $elemento = shift(@_);
       	my @array = @_;
	my @arraytemp1 = ();
	foreach $elem(@array){
		if ($elem ne $elemento){
			push (@arraytemp1,$elem);
		}
	}
	@array = @arraytemp1;
return @array;
}



#### propagarelaz ########################################################################################
#given a node with a grammatical relation, annotates the same relation on the child node

sub propagarelaz{
	my $figlio = shift(@_);
	my $padre = shift(@_);
	my $relaz = substr $padre, 0, (index $padre, ' ');
	if ($relaz =~ m/\-/){
		$relaz = (substr $relaz, (index $relaz, '-')).' ';
		$figlio =~ s/\s/$relaz/;
	}
	return $figlio;
}


##### puliscirelaz #######################################################################################
#deletes the relation included in the node

sub puliscirelaz{
	my $nodo = shift(@_);
	my $relaz = '';
	if (($nodo =~ m/\,/) and ($nodo =~ m/COORD\+BASE/)){
			$nodo =~ s/\,/\,C/;   
	}										
			$relaz = substr $nodo, 0, (index $nodo, ' ');
	if ($relaz =~ m/\-/){
		$relaz = (substr $relaz, (index $relaz, '-')).' ';
		$piu = '\+';
		$asterix = '\*';
		$relaz =~ s/\+/$piu/g;
		$relaz =~ s/\*/$asterix/;
		$nodo =~ s/$relaz/ /;
	}
return $nodo;
}


### duplicaintermedie ####################################################################################
#introduces the intermediate projections needed in the representation of modifiers

sub duplicaintermedie{
my @T_frase = @_;
my @figli = ();
my @figlinod = ();
my $newXbar = '';
my @arraymodifiers = ();
my @arrayarguments = ();
my @newarray = ();
my @arraytemp = ();
my @dasostituire = ();
my $test = '';
my @newarray = @T_frase;
while (scalar(@newarray) > 0){ 
	$nodo = shift(@newarray);
	if ($nodo =~ m/bar/){ #se il nodo e' proiezione intermedia e ha figli
			@figli = figliconstit($nodo,@T_frase);
			if (scalar(@figli) > 1){		#NB ogni nodo Xbar ha sicuramente come figlio il nodo terminale di cui e' proiezione
					foreach $figlio(@figli){ #trova ogni modificatore di nodo
							if (($figlio =~ m/RMOD/) or ($figlio =~ m/APPOSITION/) or 
							    (($figlio =~ m/CONTIN/) and ($figlio !~ m/CONTIN\+NUM/))){# or 
								#($figlio =~ m/OBJ\%EXTRAPOSED/)){
									push(@arraymodifiers,$figlio); #e lo mette in un array
							}
					}
			}
			if (scalar(@arraymodifiers) > 0){#se il nodo ha dei modificatori (che al passo precedente sono stati messi in arraymodifiers
					$newXbar = alza($nodo); # prende nodo e crea una nuova proiezione Xbar:######duplicazione intermedia
					push (@T_frase,$newXbar);
					foreach $modifier(@arraymodifiers){#per ogni modificatore
							$newmod = $modifier; #aggiorna il padre del modificatore
							$padreoldmod = (padre_in_quadre($modifier));#per agganciare il modificatore
							$padreoldmod = '\['.$padreoldmod;
							$posnewXbar = pos_in_ttree($newXbar);
							$posnewXbar = "\[".$posnewXbar;
							$newmod =~ s/$padreoldmod/$posnewXbar/;#alla newXbar (creata a causa del modificatore stesso)
							@T_frase = sostituisciinarray($modifier,$newmod,@T_frase);
					}
					@arraytemp = @T_frase;
					while (scalar(@arraytemp) > 0){	###########aggiornamento dei nodi attorno all'intermedia
						$nod = shift(@arraytemp);
						if ($nod ne $newXbar){#se nod non e' newXbar
								if (pos_in_ttree($nod) =~ m/\-/){#se e' un non terminale
										if((spina($nod) == spina($newXbar)) and #se appartiene alla stessa spina di newxbar
					   	   					(pos_in_spina($nod) >= pos_in_spina($newXbar))){ #con un index e' = or > di quello di newxbar
												if ($nod =~ m/\)\[/){#se non e' la radice dell'albero
													if (spina(padre_in_quadre($nod)) == spina($nod)){#e non e' l'ultimo della sua spina
															$newnod = alza($nod); #allora lo alza di una posizione nel T_tree
															push(@dasostituire,$nod.'|'.$newnod);
													}
													else{$newnod = $nod;#altrimenti, se nod e' l'ultimo della sua spina
        													$posnewnod = newposition_in_tree($nod);	#calcola la posizione di newnod
															$posnod = pos_in_ttree($nod); #e la sostituisce a quella di nod
															$newnod =~ s/$posnod/$posnewnod/;
															push(@dasostituire,$nod.'|'.$newnod);
													}
											}
											else{$newnod = $nod;#altrimenti, se nod e' la radice dell'albero
        											$posnewnod = newposition_in_tree($nod);	#calcola la posizione di newnod
													$posnod = pos_in_ttree($nod); #e la sostituisce a quella di nod
													$newnod =~ s/$posnod/$posnewnod/;
													push(@dasostituire,$nod.'|'.$newnod);
											}
											@figlinod = figliconstit($nod,@T_frase);#calcola i figli di nod
											@arrayarguments = ();
											if (scalar(@figlinod) > 0){#se nod ha figli
													foreach $figlio(@figlinod){ #trova ogni elemento che sia argomento di nod
															if (($figlio !~ m/RMOD/) and ($figlio !~ m/APPOSITION/)
									     						and ($figlio !~ m/CONTIN/) and ($figlio !~ m/OBJ\%EXTRAPOSED/)){
																		$posfiglio = pos_in_ttree($figlio);
																		if (spina($nod) != spina($figlio)){#ma che non appartenga alla spina di nod
																				push(@arrayarguments,$figlio); #e lo mette in un array
																		}#(NB: gli argomenti appartenenti alla stessa spina sono stati gia' aggiornati)
															}
													}
											}
											if (scalar(@arrayarguments) > 0){#se il nodo ha degli argomenti
													foreach $argument(@arrayarguments){#per ogni argomento
															if ($argument !~ m/^up/){
																	$newarg = $argument; #aggiorna il padre dell'argomento
																	$padreoldarg = padre_in_quadre($argument);#per agganciare l'argomento
																	$posnewnod = pos_in_ttree($newnod);
																	$newarg =~ s/$padreoldarg/$posnewnod/;#al newnod
																	@T_frase = sostituisciinarray($argument,'up'.$newarg,@T_frase);
																	@arrayarguments = sostituisciinarray($argument,'up'.$argument,@arrayarguments);
															}
													}
											}
									}
							}
						}
				}
				foreach $item(@dasostituire){
						$first = substr $item,0,(index $item,'|');
						$second = substr $item,(index $item,'|')+1;
						@T_frase = sostituisci_unavolta_inarray($first,$second,@T_frase);
				}
				@dasostituire = ();
		}
		@arraymodifiers = ();
	}
}
foreach $tok(@T_frase){
	if ($tok =~ m/^up/){
		$newtok = substr $tok,2;
		@T_frase = sostituisci_unavolta_inarray($tok,$newtok,@T_frase);
	}
}
return @T_frase;
}


#### newposition_in_tree ########################################################################################
#find for a given node a lower position on the same tree

sub newposition_in_tree{
	my $arg =  shift(@_);
	my $posiz = pos_in_ttree($arg);
	$posiz = (substr $posiz,0,(index $posiz,'-')).'-'.((substr $posiz,(index $posiz,'-') + 1) + 1);
return $posiz;
}


#### figliconstit ########################################################################################
#finds the children of a given node in a constituency tree

sub figliconstit{
	my $nodo = shift(@_);
	my @albero = @_;
	my @array = ();
	my $podelpadre = '';
	my $posnodo = pos_in_ttree($nodo);
	foreach $it(@albero){
		$posdelpadre = padre_in_quadre($it);
		if ($posdelpadre eq $posnodo){
			push(@array,$it);
		}
	}
return @array;
}


### padre_in_quadre ###########################################################
#finds the father of a node argument

sub padre_in_quadre{
	my $nodo = shift(@_);
	my $padre = '';
	if ($nodo =~ m/\)\[/){
			$padre = substr $nodo,0,(rindex $nodo,']');
			$padre = substr $padre,((rindex $padre,'[') + 1);
	}
	return $padre;
}


### pos_in_ttree ###########################################################
#finds the position of a node within a constituency tree

sub pos_in_ttree{
	my $nodo = shift(@_);
	my $posinttree = substr $nodo,((index $nodo,' ')+1);
	$posinttree = substr $posinttree, 0,(index $posinttree,'/');
	return $posinttree;
}


### pos_in_ttree ###########################################################
#put a given node in higher position within the constituency tree

sub alza{
	my $nodo = shift(@_);
	my $newnodo = '';
	$newnodo = $nodo; # sulla base di nodo crea newnodo:
	$padrenodo = padre_in_quadre($nodo); #calcola il padre di nodo e il padre di newnodo
	$padrenewnodo =  (substr $padrenodo,0,(index $padrenodo,'-')).'-'.((substr $padrenodo,(index $padrenodo,'-') + 1) + 1);
	$newnodo =~ s/$padrenodo/$padrenewnodo/;#sostituisce il padre nuovo a quello vecchio
	$posnewnodo = newposition_in_tree($nodo);	#calcola la posizione di newnodo
	$posnodo = pos_in_ttree($nodo); #e la sostituisce a quella di nodo
	$newnodo =~ s/$posnodo/$posnewnodo/;
return $newnodo;
}


######### spina ##################################################################################
#given a node, ouptut the spine where the node is positioned within the tree

sub spina{
	my $nodo = shift(@_);
	my $spina = '';
	my $posnodo = pos_in_ttree($nodo);
	if ($posnodo =~ m/\-/){ #calcola la spina di nodo
		$spina = substr $posnodo,0,(index $posnodo,'-');
	}
	else{$spina = $posnodo;
	}
return $spina;
}


######### pos_in_spina ##################################################################################
#given a node, ouptut the position of the node within the spine where the node is positioned within the tree

sub pos_in_spina{
	my $nodo = shift(@_);
	my $posnodo = pos_in_ttree($nodo);
	my $indice = 0;
	if  ($posnodo =~ m/\-/){ #calcola l'indice di nodo
		$indice = substr $posnodo,((index $posnodo,'-') + 1);
	}
return $indice;
}



### sollevapuntofinale #####################################################################
#attach the final punctuation marker to the head of the sentence

sub sollevapuntofinale{
	my @T_frase = @_;
	my $senthead = '';
	my $finalpunct = '';
	my $newfinalpunct = '';
	foreach $nodo(@T_frase){
		if (($nodo !~ m/\]/) and ($nodo !~ m/\[/)){
			$senthead = $nodo;
		}
		else{if ($nodo =~ m/PUNCT\-END/){
				$finalpunct = $nodo;
			}
		}
	}
	$newfinalpunct = (substr $finalpunct,0,(index $finalpunct,')') + 1).'['.pos_in_ttree($senthead).']';
	@T_frase = sostituisciinarray($finalpunct,$newfinalpunct,@T_frase);
return @T_frase;
}



### sollevatracce ############################################################################
#put the full and partial trace references on the maximal and intermediate projections respectively

sub sollevatracce{
	my @T_frase = @_;
	my $indice = '';
	my $coreferentepartial = '';
	my $coreferentefull = '';
	my $coreferenteword = '';
	foreach $costituente(@T_frase){
		if (($costituente =~ m/\*\d/) and (($costituente =~ m/\*\[w\-/) or ($costituente =~ m/\*\[f\-/) 
			or ($costituente =~ m/\*\[p\-/) or ($costituente =~ m/\*\[\]/))){
				$indice = substr $costituente, ((index $costituente,'/*') + 2);#calcolo l'indice di referenza
				$indice = substr $indice, 0,(index $indice,'-');
				$coreferentepartial = '';
				$coreferentefull = '';
				$coreferenteword = '';
				foreach $tra(@T_frase){#cerco l'elemento traccia coindicizzato con costituente
						if ($tra =~ m/\*\[([f|p|w])\-/){
								if ($tra =~ m/\*\[f\-$indice\]/){#se la traccia e' coindicizzata con costituente ed e' full
										$coreferentefull = $tra;#valorizzo coreferentefull
								}
								elsif ($tra =~ m/\*\[p\-$indice\]/){#se la traccia coindicizzata con costituente e' partial
										$coreferentepartial = $tra;#valorizzo coreferentepartial
				      			}
								elsif ($tra =~ m/\*\[w\-$indice\]/){#se la traccia coindicizzata con costituente e' word
											$coreferenteword = $tra;#valorizzo coreferenteword
								}
						}
				}
				if ($coreferenteword ne ''){
						if ((substr $costituente,0,(index $costituente,' ')) !~ m/\-/){
								$newcostituente = (substr $costituente, 0,(index $costituente,' ')).'*'.$indice.'33 '.
															$indice.'/'.(substr $costituente,(index $costituente,'-') + 1);
						}
						else{$newcostituente = (substr $costituente, 0,(index $costituente,' ')).'*'.$indice.'33 '.$indice.'/';
								$subcostit = substr $costituente,(index $costituente,' ') + 1;
								$subcostit = substr $subcostit,(index $subcostit,'-')+1;
								$newcostituente = $newcostituente.$subcostit;
						}
						@T_frase = sostituisciinarray($costituente,$newcostituente,@T_frase);
				}
				else{$proiezfirst = padre($costituente,@T_frase);#cerco le proiezioni di costituente:
						if (($proiezfirst =~ m/^\(PP/) or ($proiezfirst =~ m/^\(CONJP/)
				     		or ($proiezfirst =~ m/^\(PRN/)){		#se il costituente non puo' avere proiezione intermedia (caso1)
									$proiezint = $proiezfirst;
									$proiezmax = $proiezfirst;
						}
						else{$proiezsecond = padre($proiezfirst,@T_frase);	#se il costituente puo' avere proiezione intermedia
								if (($proiezsecond =~ m/^\(VP/) or ($proiezsecond =~ m/^\(NP/)
				    				or ($proiezsecond =~ m/^\(DP/) or ($proiezsecond =~ m/^\(ADVP/) or
				    				($proiezsecond =~ m/^\(ADJP/)){#o ne ha una sola (caso2)
				    						$proiezint = $proiezfirst;
				  					  		$proiezmax = $proiezsecond;
								}
								else{$proiezint = $proiezsecond;#oppure ne ha due (caso3)
										$proiezmax = padre($proiezsecond,@T_frase);
								}
						}
						$newcostituente = $costituente;
						$newcostituente =~ s/\*$indice\-//;#cancello la referenza dal terminale
						@T_frase = sostituisciinarray($costituente,$newcostituente,@T_frase);
						if ($coreferentefull ne ''){#se la traccia e' full
								$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*'.$indice.'33'.(substr
			       											$proiezmax,(index $proiezmax,' '));#mette la rerefenza nella massimale
			       				$indiceplus = $indice.'33';
			       				if ($proiezmax !~ m/$indiceplus/){
										@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
								}
						}
						elsif ($coreferentepartial ne ''){#altrimenti, se la traccia e' partial
									$newproiezint = (substr $proiezint, 0,(index $proiezint,' ')).'*'.$indice.'33'.(substr
															$proiezint,(index $proiezint,' '));#mette la rerefenza nella intermedia
									$indiceplus = $indice.'33';
			       					if ($proiezint !~ m/$indiceplus/){
											@T_frase = sostituisciinarray($proiezint,$newproiezint,@T_frase);
									}
						}
				}
				$indicetr = substr $newcostituente, ((index $newcostituente,'*[') + 4);#calcolo l'indice di referenza della traccia
				$indicetr = substr $indicetr, 0,(index $indicetr,']');
				if ($costituente !~ m/\*\[w\-/){#se non e' word
					$proiezfirst = padre($costituente,@T_frase);#cerco le proiezioni di costituente:
					if (($costituente =~ m/PREP/) or ($proiezfirst =~ m/^\(PP/) or ($proiezfirst =~ m/^\(CONJP/)
			 	    	or ($proiezfirst =~ m/^\(PRN/)){		#se il costituente non puo' avere proiezione intermedia (caso1)
								$proiezint = $proiezfirst;
								$proiezmax = $proiezfirst;
								$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*['.$indicetr.'33]'.(substr
				        						          $proiezmax,(index $proiezmax,' '));#metto la rerefenza nella massimale
								$indiceplus = $indicetr.'33';
			       				if ($proiezmax !~ m/$indiceplus/){
			       						@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       				}
					}
					else{$proiezsecond = padre($proiezfirst,@T_frase);	#se il costituente puo' avere proiezione intermedia
							if (($proiezsecond =~ m/^\(VP/) or ($proiezsecond =~ m/^\(NP/)
			    				or ($proiezsecond =~ m/^\(DP/) or ($proiezsecond =~ m/^\(ADVP/) or
		    	    			($proiezsecond =~ m/^\(ADJP/)){#o ne ha una sola (caso2)
			   							$proiezint = $proiezfirst;
			   							$proiezmax = $proiezsecond;
										if ($newcostituente =~ m/\/\*\[f\-/){#se e' full
												@T_frase = cancelladaarray($proiezint,@T_frase);#cancella l'intermedia
												$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*['.$indicetr.'33]'.(substr
				            		            				          $proiezmax,(index $proiezmax,' '));#mette la rerefenza nella massimale
												$indiceplus = '\*'.$indicetr.'33';
			       								if ($proiezmax !~ m/$indiceplus/){
			       										@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       								}
										}
										elsif ($newcostituente =~ m/\/\*\[p\-/){#se e' partial
													$newproiezint = (substr $proiezint, 0,(index $proiezint,' ')).'*['.$indicetr.'33]'.(substr
						    	        							        $proiezint,(index $proiezint,' '));#mette la rerefenza nella intermedia
													$indiceplus = $indicetr.'33';
			       									if ($proiezint !~ m/$indiceplus/){
			       											@T_frase = sostituisciinarray($proiezint,$newproiezint,@T_frase);
			       									}
										}
							}
							else{$proiezintfirst = $proiezfirst; #oppure ha due proiezioni intermedie (caso3)
									$proiezintsecond = $proiezsecond;
									$proiezmax = padre($proiezsecond,@T_frase);
									@T_frase = cancelladaarray($proiezintfirst,@T_frase);#cancella la prima intermedia
									if ($newcostituente =~ m/\/\*\[f\-/){#se e' full
											@T_frase = cancelladaarray($proiezintsecond,@T_frase);#cancella la seconda intermedia
											$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*['.$indicetr.'33]'.(substr
				    		            			                  $proiezmax,(index $proiezmax,' '));#mette la rerefenza nella massimale
											$indiceplus = $indicetr.'33';
			       							if ($proiezmax !~ m/$indiceplus/){			
			       									@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       							}
									}
									elsif ($newcostituente =~ m/\/\*\[p\-/){#se e' partial
											$newproiezintsecond = (substr $proiezintsecond, 0,(index $proiezintsecond,' ')).
																			'*['.$indicetr.'33]'.(substr $proiezintsecond,(index $proiezintsecond,' '));#mette la rerefenza nella seconda intermedia
											$indiceplus = $indicetr.'33';
			       							if ($proiezintsecond !~ m/$indiceplus/){
			       									@T_frase = sostituisciinarray($proiezintsecond,$newproiezintsecond,@T_frase);
			       							}
									}
							}
					}
					@T_frase = cancelladaarray($costituente,@T_frase);#cancello il terminale
			}
			else{$newnewcostituente = (substr $newcostituente,0,(index $newcostituente,'/') + 1).'*['.$indicetr.'33]'.
													 (substr $newcostituente,(index $newcostituente,')'));
					@$indiceplus = $indicetr.'33';
			       	if ($newcostituente !~ m/$indiceplus/){
			       			@T_frase = sostituisciinarray($newcostituente,$newnewcostituente,@T_frase);
			       	}
			}				
		}
		elsif (($costituente =~ m/\*\d/) and ($costituente !~ m/\*\[/)){#se e' il referente di una traccia, ma non e' una traccia
				$firstindex = substr $costituente,(index $costituente,' ')+1;
				$firstindex = substr $firstindex,0,(index $firstindex,'/');
				$secondindex = substr $costituente,(index $costituente,'*')+1;
				$secondindex = substr $secondindex,0,(index $secondindex,'-');
				if ($secondindex =~ m/LOCUT/){
						$secondindex = substr $secondindex,(index $secondindex,'*')+1;
				}
				if (($costituente !~ m/33 /) and ($costituente =~ m/\*/) and ($firstindex eq $secondindex)){	#escludo il caso del referente gia' trattato
						$indice = substr $costituente, ((index $costituente,'*') + 1);#calcolo l'indice di referenza
						$indice = substr $indice, 0,(index $indice,'-');
						if ($costituente =~ m/LOCUT/){
								$indice = substr $indice,(rindex $indice,'*')+1;
						}
						$coreferentepartial = '';
						$coreferentefull = '';
						$coreferenteword = '';
						foreach $tra(@T_frase){#cerco l'elemento traccia coindicizzato con costituente
												#e a seconda del tipo (p,f) sposto la tracciatura (su Xbar, XP) o lla lascio sulla parola (w)
								if ($tra =~ m/\*\[([f|p|w])\-/){
										if ($tra =~ m/\*\[f\-$indice\]/){#se la traccia e' coindicizzata con costituente ed e' full
												$coreferentefull = $tra;#valorizzo coreferentefull
										}
										elsif ($tra =~ m/\*\[p\-$indice\]/){#se la traccia coindicizzata con costituente e' partial
													$coreferentepartial = $tra;#valorizzo coreferentepartial
				      					}
										elsif ($tra =~ m/\*\[w\-$indice\]/){#se la traccia coindicizzata con costituente e' word
												$coreferenteword = $tra;#valorizzo coreferentepartial
										}
					      		}
						}
						if ($coreferenteword ne ''){
								if ((substr $costituente,0,(index $costituente,' ')) !~ m/\-/){
										$newcostituente = (substr $costituente, 0,(index $costituente,' ')).'*'.$indice.'33 '.
																	$indice.'/'.(substr $costituente,(index $costituente,'-') + 1);
								}	
								else{$newcostituente = (substr $costituente, 0,(index $costituente,' ')).'*'.$indice.'33 '.$indice.'/';
										$subcostit = substr $costituente,(index $costituente,' ') + 1;
										$subcostit = substr $subcostit,(index $subcostit,'-')+1;
										$newcostituente = $newcostituente.$subcostit;
								}
								$indiceplus = $indice.'33';
			       				if ($costituente !~ m/$indiceplus/){
										@T_frase = sostituisciinarray($costituente,$newcostituente,@T_frase);
								}
						}
						else{$proiezfirst = padre($costituente,@T_frase);#cerco le proiezioni di costituente:
								if (($proiezfirst =~ m/^\(PP/) or ($proiezfirst =~ m/^\(CONJP/)
				     				or ($proiezfirst =~ m/^\(PRN/)){		#se il costituente non puo' avere proiezione intermedia (caso1)
											$proiezint = $proiezfirst;
											$proiezmax = $proiezfirst;
								}
								else{$proiezsecond = padre($proiezfirst,@T_frase);	#se il costituente puo' avere proiezione intermedia
										if (($proiezsecond =~ m/^\(VP/) or ($proiezsecond =~ m/^\(NP/)
				    						or ($proiezsecond =~ m/^\(DP/) or ($proiezsecond =~ m/^\(ADVP/) or
				    						($proiezsecond =~ m/^\(ADJP/)){#o ne ha una sola (caso2)
				   						 			$proiezint = $proiezfirst;
				    								$proiezmax = $proiezsecond;
										}
										else{$proiezint = $proiezsecond;#oppure ne ha due (caso3)
												$proiezmax = padre($proiezsecond,@T_frase);
										}
								}
								$newcostituente = $costituente;
								$newcostituente =~ s/\*$indice\-//;#cancello la referenza dal terminale
								@T_frase = sostituisciinarray($costituente,$newcostituente,@T_frase);
								if ($coreferentefull ne ''){#se la traccia e' full
										$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*'.$indice.'33'.(substr
			       													$proiezmax,(index $proiezmax,' '));#mette la rerefenza nella massimale
										$indiceplus = $indice.'33';
			       						if ($proiezmax !~ m/$indiceplus/){
			       								@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       						}
								}
								elsif ($coreferentepartial ne ''){#altrimenti, se la traccia e' partial
											$newproiezint = (substr $proiezint, 0,(index $proiezint,' ')).'*'.$indice.'33'.(substr
																	$proiezint,(index $proiezint,' '));#mette la rerefenza nella intermedia
											$indiceplus = $indice.'33';
			       							if ($proiezint !~ m/$indiceplus/){
			       									@T_frase = sostituisciinarray($proiezint,$newproiezint,@T_frase);
			       							}
								}
						}
				}
		}
	}
##fine trattamento delle referenze delle tracce - inizio trattamento delle tracce
	foreach $costituente(@T_frase){
		if (($costituente !~ m/\*\d/) and ($costituente =~ m/\*\[w\-/) or ($costituente =~ m/\*\[f\-/)
			or ($costituente =~ m/\*\[p\-/)){#altrimenti se costituente e' traccia coindicizzata
				$indice = substr $costituente, ((index $costituente,'*') + 4);#calcolo l'indice di referenza
				$indice = substr $indice, 0,(index $indice,']');
				if (($costituente =~ m/LOCUT/) or ($indice =~ m/\*/)){
							$indice = substr $indice,(rindex $indice,'-')+1;
					}
				if ($costituente !~ m/\/\*\[w\-/){#se non e' word
					$proiezfirst = padre($costituente,@T_frase);#cerco le proiezioni di costituente:
					if (($costituente =~ m/PREP/) or ($proiezfirst =~ m/^\(PP/) or ($proiezfirst =~ m/^\(CONJP/)
				     	    or ($proiezfirst =~ m/^\(PRN/)){		#se il costituente non puo' avere proiezione intermedia (caso1)
						$proiezint = $proiezfirst;
						$proiezmax = $proiezfirst;
#FULL e PARTIAL: indice su XP
						$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*['.$indice.'33]'.(substr
						                  			$proiezmax,(index $proiezmax,' '));#metto la rerefenza nella massimale
						$indiceplus = $indice.'33';
						if ($proiezmax !~ m/$indiceplus/){			       		
			       				@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       		}
					}
					else{$proiezsecond = padre($proiezfirst,@T_frase);	#se il costituente puo' avere proiezione intermedia
						if (($proiezsecond =~ m/^\(VP/) or ($proiezsecond =~ m/^\(NP/)
					    	or ($proiezsecond =~ m/^\(DP/) or ($proiezsecond =~ m/^\(ADVP/) or
				    	    	($proiezsecond =~ m/^\(ADJP/)){#o ne ha una sola (caso2)
				    			$proiezint = $proiezfirst;
				    			$proiezmax = $proiezsecond;
							if ($costituente =~ m/\/\*\[f\-/){#se e' full
								@T_frase = cancelladaarray($proiezint,@T_frase);#cancella l'intermedia
								$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*['.$indice.'33]'.(substr
						                                  $proiezmax,(index $proiezmax,' '));#mette la rerefenza nella massimale
								$indiceplus = $indice.'33';
			       				if ($proiezmax !~ m/$indiceplus/){
			       						@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       				}
							}
							else{if ($costituente =~ m/\/\*\[p\-/){#se e' partial
									$newproiezint = (substr $proiezint, 0,(index $proiezint,' ')).'*['.$indice.'33]'.(substr
								    	                $proiezint,(index $proiezint,' '));#mette la rerefenza nella intermedia
									$indiceplus = $indice.'33';
			      					if ($proiezint !~ m/$indiceplus/){
			       							@T_frase = sostituisciinarray($proiezint,$newproiezint,@T_frase);
			       					}
								}
							}
						}
						else{$proiezintfirst = $proiezfirst; #oppure ha due proiezioni intermedie (caso3)
							$proiezintsecond = $proiezsecond;
							$proiezmax = padre($proiezsecond,@T_frase);
							@T_frase = cancelladaarray($proiezintfirst,@T_frase);#cancella la prima intermedia
							if ($costituente =~ m/\/\*\[f\-/){#se e' full
								@T_frase = cancelladaarray($proiezintsecond,@T_frase);#cancella la seconda intermedia
								$newproiezmax = (substr $proiezmax, 0,(index $proiezmax,' ')).'*['.$indice.'33]'.(substr
						                	                  $proiezmax,(index $proiezmax,' '));#mette la rerefenza nella massimale
					  			$indiceplus = $indice.'33';
			       				if ($proiezmax !~ m/$indiceplus/){
			       						@T_frase = sostituisciinarray($proiezmax,$newproiezmax,@T_frase);
			       				}
							}
							else{if ($costituente =~ m/\/\*\[p\-/){#se e' partial
									$newproiezintsecond = (substr $proiezintsecond, 0,(index $proiezintsecond,' ')).
																		'*['.$indice.'33]'.(substr $proiezintsecond,(index $proiezintsecond,' '));#mette la rerefenza nella seconda intermedia
									$indiceplus = $indice.'33';
			       					if ($proiezintsecond !~ m/$indiceplus/){
			       							@T_frase = sostituisciinarray($proiezintsecond,$newproiezintsecond,@T_frase);
			       					}
								}
							}
						}
					}
					@T_frase = cancelladaarray($costituente,@T_frase);#cancello il terminale
				}
				else{$newcostituente = (substr $costituente,0,(index $costituente,'/') + 1).'*['.$indice.'33]'.(substr $costituente,(index 										$costituente,')'));
					@T_frase = sostituisciinarray($costituente,$newcostituente,@T_frase);
				}
			}
		}
	return @T_frase;
}



### bilanciapunti ###############################################################################
#given a sentence in CONStituency-TUT and an array that includes all the cases of balanced punctuation occurring in the 
#sentence, put together each couple OPEN and CLOSE dependents of the same father, and call the procedure that builds 
#the parenthetical and quoted clauses

sub bilanciapunti{
	my ($ref1,$ref2,$ref3) = @_;  	#dereferenziazione degli array argomenti
   	my @T_frase = @{$ref1};    		#l'albero di costituenti
   	my @D_frase = @{$ref2};    		#l'albero a dipendenze
   	my @bilanciati = @{$ref3};  		#l'array della punteggiatura bilanciata
	my @bilanciatitemp = ();
	my $open = 0;
	my $close = 0;
	foreach $it(@bilanciati){
		push(@bilanciatitemp,$it);
	}
	while ((scalar(@bilanciati) > 0) and (scalar(@bilanciatitemp) > 0)){
			foreach $elem(@bilanciati){
					if ($elem =~ m/OPEN/){			#trova open
							$close = cerca_close($elem,@bilanciatitemp);
							if ($close == 0){
									$close = '150'.(substr $elem, (index $elem,'/'));
									$close =~ s/OPEN/CLOSE/;
							}
							else{@bilanciatitemp = cancelladaarray($close,@bilanciatitemp);
									@bilanciati = cancelladaarray($close,@bilanciati);
							}
							@T_frase = costruisci_parentetica_o_quotata(\$elem,\$close,\@D_frase,\@T_frase);
							@bilanciatitemp = cancelladaarray($elem,@bilanciatitemp);
							@bilanciati = cancelladaarray($elem,@bilanciati);
					}
					else {			#se invece trova close		
							$open = cerca_open($elem,@bilanciatitemp);
							if ($open == 0){		#costruisce un open fittizio da mandare a costruisciparentetica
									$open = '150'.(substr $elem, (index $elem,'/'));
									$open =~ s/CLOSE/OPEN/;
							}
							else{@bilanciatitemp = cancelladaarray($open,@bilanciatitemp);
									@bilanciati = cancelladaarray($open,@bilanciati);
							}
							@T_frase = costruisci_parentetica_o_quotata(\$open,\$elem,\@D_frase,\@T_frase);
							@bilanciatitemp = cancelladaarray($elem,@bilanciatitemp);
							@bilanciati = cancelladaarray($elem,@bilanciati);
					}
			}
	}
	foreach $el(@T_frase){
			if (($el =~ m/\(PRN /) and ($el =~ m/\)\[\]/)){
					$newel = substr $el,0,(rindex $el,'[');
					$prn = $el;
					@T_frase = sostituisciinarray($el,$newel,@T_frase);
			}
	}
	foreach $el(@T_frase){
			if (($el =~ m/\-TOP/) and ($el =~ m/\/\[/)){
					$newel = (substr $el,0,(rindex $el,'[')).')['.pos_in_ttree($prn).']';
					@T_frase = sostituisciinarray($el,$newel,@T_frase);
			}
	}
	return @T_frase;
}


### cerca_close ###############################################################################
#find the close in the array of the balanced punctuation

sub cerca_close{
	my $open = shift(@_);
	my @arraybilanciati = @_;
	my @arrayclose = ();
	my @arrayopen = ();
	my $close = '';
	foreach $item(@arraybilanciati){	#trova tutti gli item che hanno stesso padre di open ma sono >open
			if ((pospadre($item) eq pospadre($open)) and ($item ne $open) and 
			    (position_in_Dtree($item) > position_in_Dtree($open))){
					if ($item =~ m/CLOSE/){	#li divide in un array di close ed uno di open
							push(@arrayclose,$item);
					}
					else{push(@arrayopen,$item);
					}
			}
	}
	if (scalar(@arrayclose)==1){
			$close = shift(@arrayclose);
	}
	elsif (scalar(@arrayclose)>1){	#se i close sono piu' di uno
			@arrayopen = sort {position_in_Dtree($a) <=> position_in_Dtree($b)} @arrayopen;
			@arrayclose = sort {position_in_Dtree($a) <=> position_in_Dtree($b)} @arrayclose;
			$first_open = shift(@arrayopen); #prende il piu' piccolo da ognuno dei due array
			$first_close = shift(@arrayclose);
			if ((position_in_Dtree($first_open) < position_in_Dtree($first_close)) and 
			    ($first_open ne '')){	#se first_open e' piu' piccolo di first_close
					@arraybilanciati = cancelladaarray($first_open,@arraybilanciati);	#allora first_open apre una PRN chiusa da first_close ed 
					@arraybilanciati = cancelladaarray($first_close,@arraybilanciati);	#annidata dentro quella aperta da open
					$close = cerca_close($open,@arraybilanciati);	#chiama ricorsivamente cercaclose sui restanti elementi di arraybilanciati
			}
			else{$close = $first_close;
			}
	}
	elsif (scalar(@arrayclose)==0){
			$close = 0;
	}
	return $close;
}


### cerca_open ###############################################################################
#find the open in the array of the balanced punctuation

sub cerca_open{
	my $close = shift(@_);
	my @arraybilanciati = @_;
	my @arrayclose = ();
	my @arrayopen = ();
	my $open = '';
	foreach $item(@arraybilanciati){	#trova tutti gli item che hanno stesso padre di close ma sono <close
			if ((pospadre($item) eq pospadre($close)) and ($item ne $close) and 
			    (position_in_Dtree($item) < position_in_Dtree($close))){
					if ($item =~ m/CLOSE/){	#li divide in un array di close ed uno di open
							push(@arrayclose,$item);
					}
					else{push(@arrayopen,$item);
					}
			}
	}
	if (scalar(@arrayopen)==1){
			$open = shift(@arrayopen);
	}
	elsif (scalar(@arrayopen)>1){	#se gli open sono piu' di uno
			@arrayopen = sort {position_in_Dtree($b) <=> position_in_Dtree($a)} @arrayopen;
			@arrayclose = sort {position_in_Dtree($b) <=> position_in_Dtree($a)} @arrayclose;
			$first_open = shift(@arrayopen); #prende il piu' piccolo da ognuno dei due array
			$first_close = shift(@arrayclose);
			if ((position_in_Dtree($first_close) > position_in_Dtree($first_open)) and
			    ($first_close ne '')){	#se first_close e' piu' grande di first_open
					@arraybilanciati = cancelladaarray($first_open,@arraybilanciati);	#allora first_close chiude una PRN aperta da first_open ed 
					@arraybilanciati = cancelladaarray($first_close,@arraybilanciati);	#annidata dentro quella chiusa da close
					$open = cerca_open($close,@arraybilanciati);	#chiama ricorsivamente cercaopen sui restanti elementi di arraybilanciati
			}
			else{$open = $first_open;
			}
	}
	elsif (scalar(@arrayopen)==0){
			$open = 0;
	}
	return $open;
}


### position_in_Dtree ###############################################################################
#find the position in the dependency tree of a given node

sub position_in_Dtree{
	my $arg =  shift(@_);
	my $posiz = '';
	$posiz = substr $arg, 0, (index $arg,'/');
	return $posiz;
}



########### costruisci_parentetica_o_quotata ##################################################
#Build the representation of parenthetical and quoted clauses on the previously built constituency tree 

sub costruisci_parentetica_o_quotata{
	my ($ref1,$ref2,$ref3,$ref4) = @_;  	#dereferenziazione degli array argomenti
	my $open = $$ref1;    						#l'elemento che apre la PRN/quotata
   	my $close = $$ref2;    						#l'elemento che chiude la PRN/quotata
   	my @D_frase = @{$ref3};  				#la frase a dipendenze
	my @T_frase = @{$ref4}; 					#la frasea costituenti
	my $posizione_open = '';
	my $posizione_close = '';   	
	my $posizione_head_inTtree = '';		#la posizione dell'elento piu' alto della spina che parte da $prn_head_inTtree
	my $prn_head_inDtree = '';				#la testa della PRN nel Dtree 
   	my $prn_head_inTtree = '';					#la testa della PRN nel Ttree
	my $padre_head_inTtree = ''; 			
	my $prn_anomala = '';
	my $new_PRN = '';								#la nuova testa della PRN (con categoria PRN)
	my $new_prnhead = '';						#$prn_head_inTtree agganciata sotto newPRN
	my $newopen = '';								#$open agganciato sotto newPRN
	my $newclose = '';								#$close agganciato sotto newPRN
	my $trovatolivello = 0;
	my @fratelli_padre_head = ();
	$posizione_open = substr $open,0,(index $open,'/'); 
	$posizione_close = substr $close,0,(index $close,'/');
	if (($open !~ m/\+QUOTES/) and ($close !~ m/\+QUOTES/)){
			foreach $it(@D_frase){
					if ((pospadre($it) == 0) and ($it ne $open) and ($it ne $close) and 
			    		((substr $it,0,(index $it,'/')) >  (substr $open,0,(index $open,'/'))) and 
			    		((substr $it,0,(index $it,'/')) <  (substr $close,0,(index $close,'/')))){
			    				$prn_anomala = $it;
					}
					if ((pospadre($it) eq pospadre($open)) and ($it ne $open) and ($it ne $close) and 
			    		((substr $it,0,(index $it,'/')) >  (substr $open,0,(index $open,'/'))) and 
			    		((substr $it,0,(index $it,'/')) <  (substr $close,0,(index $close,'/')))){ 
								if ($prn_anomala ne ''){#per i casi come ALB-39
										$prn_head_inDtree = $prn_anomala;
								}
								else{$prn_head_inDtree = $it;	#la testa della PRN nel Dtree
								}
								foreach $elem(@T_frase){	#cerca in Ttree la parola che corrisponde alla testa della PRN nel Dtree
										if (pos_in_ttree($elem) eq (substr $prn_head_inDtree,0,(index $prn_head_inDtree,'/'))){
												$prn_head_inTtree = $elem;
												$posizione_head_inTtree = padre($prn_head_inTtree,@T_frase);
												while ((spina($prn_head_inTtree) eq spina($posizione_head_inTtree)) and
														 ($trovatolivello == 0)){
														if ($prn_anomala ne ''){		
																foreach $fratello(figliconstit($posizione_head_inTtree,@T_frase)){
																		if (($fratello ne $prn_head_inTtree) and 
											    							((pos_in_ttree($fratello) > (substr $close,0,(index $close,'/'))) or 
											    							(pos_in_ttree($fratello) < (substr $open,0,(index $open,'/'))))){
																				$trovatolivello = ++$trovatolivello;
																		} 
																}														
														}
														$prn_head_inTtree = $posizione_head_inTtree;
														$posizione_head_inTtree = padre($prn_head_inTtree,@T_frase);
												}
												$posizione_head_inTtree = $prn_head_inTtree;
										}
								}
					}
			}	
			if (($prn_anomala ne '') and ($trovatolivello != 0)){
					$new_PRN =  '(PRN '.$posizione_open.'-1/)['.
								        padre_in_quadre($posizione_head_inTtree).']';
			}
			else{$new_PRN =  '(PRN '.spina($posizione_head_inTtree).'-'.(pos_in_spina($posizione_head_inTtree) + 1).'/)['.
								        padre_in_quadre($posizione_head_inTtree).']';
			}
			push(@T_frase,$new_PRN);
			$new_prnhead = substr $posizione_head_inTtree,0,(rindex $posizione_head_inTtree,'[');
			$new_prnhead = $new_prnhead.'['.(pos_in_ttree($new_PRN)).']';
			@T_frase = sostituisciinarray($posizione_head_inTtree,$new_prnhead,@T_frase);
			if ($open !~ m/^150/){
					$newopen = build_node($open).'['.(pos_in_ttree($new_PRN)).']';
					push(@T_frase,$newopen);
			}
			if ($close !~ m/^150/){
					$newclose = build_node($close).'['.(pos_in_ttree($new_PRN)).']';
					push(@T_frase,$newclose);
			}
	}
	elsif (($open =~ m/\+QUOTES/) and ($close =~ m/\+QUOTES/)){
#cerca nel Dtree la parola testa della quotata (=$prn_head_inDtree, cioe' l'elemento del Dtree che ha un indice 
#compreso tra open e close, e che e' padre di open e di close), risale fino al piu' alto elemento della spina di 
#$prn_head_inDtree (=$prn_head_inTtree), attacca sotto $prn_head_inDtree open e close  (creando cosi' 
#newopen, newclose e new_prnhead)
			foreach $it(@D_frase){
					if (($it ne $open) and ($it ne $close) and (pospadre($open) eq (substr $it,0,(index $it,'/'))) and 
						((substr $it,0,(index $it,'/')) >  (substr $open,0,(index $open,'/'))) and 
			    		((substr $it,0,(index $it,'/')) <  (substr $close,0,(index $close,'/')))){ 
							$prn_head_inDtree = $it;
					}
			}
			foreach $elem(@T_frase){	#cerca in Ttree la parola che corrisponde alla testa della quotata nel Dtree
					if (pos_in_ttree($elem) eq (substr $prn_head_inDtree,0,(index $prn_head_inDtree,'/'))){
							$prn_head_inTtree = $elem;
							$posizione_head_inTtree = padre($prn_head_inTtree,@T_frase);
							while ((spina($prn_head_inTtree) eq spina($posizione_head_inTtree)) and 
									  ($trovatolivello == 0)){
									foreach $fratello(figliconstit($posizione_head_inTtree,@T_frase)){
											if (($fratello ne $prn_head_inTtree) and 
											    ((pos_in_ttree($fratello) > (substr $close,0,(index $close,'/'))) or 
											    (pos_in_ttree($fratello) < (substr $open,0,(index $open,'/'))))){
													$trovatolivello = ++$trovatolivello;
											} 
									}	
									$prn_head_inTtree = $posizione_head_inTtree;
									$posizione_head_inTtree = padre($prn_head_inTtree,@T_frase);								
							}
							$posizione_head_inTtree = $prn_head_inTtree;
					}
			}
			if ($open !~ m/^150/){
					$newopen = build_node($open).'['.(pos_in_ttree($posizione_head_inTtree)).']';
					push(@T_frase,$newopen);
			}
			if ($close !~ m/^150/){
					$newclose = build_node($close).'['.(pos_in_ttree($posizione_head_inTtree)).']';
					push(@T_frase,$newclose);	
			}
	}
	return @T_frase;
}



###### subtreeconstit #######################################################################################
#given a node, output the subtree of the node in a constituency tree

sub subtreeconstit{
	my $nodo = $_[0];
	$trovto = $_[1];   #$trovto `e inizializzata all'esterno
	my @T_tree = @_;
	if ($trovto eq 'no'){  #solo quando si entra in subtree da una chiamata esterna
		@subtreeconstituenti = ();  #l'array del subtree deve essere vuoto, ma deve non deve essere
	}                       #svuotato nelle chiamate ricorsive
	$trovto = 'si';
	if (scalar(figliconstit($nodo,@T_tree)) > 0){     #se il nodo ha figli
		push(@subtreeconstituenti,$nodo);              #viene messo nell'array subtree e la routine
		foreach $figlio(figliconstit($nodo,@T_tree)){     #viene richiamata su ogni suo figlio
			subtreeconstit($figlio,$trovto,@T_tree);
		}
	}
	else{push(@subtreeconstituenti,$nodo);                 #se invece il nodo non ha figli, viene messo nell'array
	}
return @subtreeconstituenti;
}


#####compar #################################################################
sub comparspine{
	return (spina(shift(@_)) <=> spina(shift(@_)));
}



##### inserisciseparator ######################################################################################
#put the separator punct in the constituency tree

sub inserisciseparator{
	my ($ref1,$ref2) = @_;  			#dereferenziazione degli array argomenti
   	my @T_frase = @{$ref1};    		#l'albero di costituenti
   	my @separatori = @{$ref2};  	#l'array della punteggiatura
	my $padre_sep = '';					#il padre di separ
	my $proiez_padre = ''; 				#la proiezione massimale di padresepar
	my $newsep = '';						#il separatore nella forma idonea alla T_frase
	my $listpos = '';
	foreach $sep(@separatori){
			foreach $item(@T_frase){		  #trova il padre
					if (pos_in_ttree($item) eq pospadre($sep)){
							$padre_sep = $item;
					}
			}
			$proiez_padre = padre($padre_sep,@T_frase);		#trova il padre del padre di separatore
			$newsep = build_node($sep).'['.(sali_e_cercafratelli_separator($sep,$proiez_padre,@T_frase)).']';
			push(@T_frase,$newsep);
	}
return @T_frase;
}


##########################################################
#looks for the best attachment position for the separator

sub sali_e_cercafratelli_separator{
	my $nodoref = shift(@_);
	my $nodo = shift(@_);
	my @albero = @_;
	my @fratelli = ();
	my $fratello = '';
	my $fratminore = 0;
	my $fratmaggiore = 0;
	@fratelli = figliconstit($nodo,@albero);
	if (scalar(@fratelli) > 0){
			foreach $frat(@fratelli){
					$fratello = $frat;
					if (spina($frat) < pos_in_ttree($nodoref)){
							$fratminore = ++$fratminore;
					}
					else{
							$fratmaggiore = ++$fratmaggiore;
					}
			}
			if (($fratminore >0) and ($fratmaggiore >0)){
					$padreconstit = padre_in_quadre($fratello);
			}
			else{
					sali_e_cercafratelli_separator($nodoref,padre($nodo,@albero),@albero);
			}
	}
	return $padreconstit;
}


########### one_level_up #####################################
#given a node, the father of the node and a tree, put in the higher level of the tree the node

sub one_level_up{
	my $arg = shift(@_);
	my $fatherarg = shift(@_);
	my @argtree = @_;
	my $newarg = $arg;
	$newarg = substr $newarg,0,(rindex $newarg,'[') +1;	
	$newarg = $newarg.(padre_in_quadre($fatherarg))."]";	#arg prende il padre di suo padre
	@argtree = sostituisciinarray($arg,$newarg,@argtree);	
return @argtree;
}


return 1;
END {}
