#/*  #Copyright 2011 Cristina Bosco [bosco] AT di DOT unito DOT it

    #This file is part of TUTtoPENN converter rel. 1.0.

    #TUTtoPENN converter is free software: you can redistribute it and/or modify
    #it under the terms of the GNU General Public License as published by
    #the Free Software Foundation, either version 3 of the License, or
    #(at your option) any later version.

    #TUTtoPENN converter is distributed in the hope that it will be useful,
    #but WITHOUT ANY WARRANTY; without even the implied warranty of
    #MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    #GNU General Public License for more details.

    #You should have received a copy of the GNU General Public License
    #along with TUTtoPENN converter. If not, see <http://www.gnu.org/licenses/>.
#*/


#This file is part of the TUTtoPENN converter, which implements a conversion from TUT 
#native format to Penn Treebank format adapted to Italian (called TUT-Penn).
#The TUTtoPENN converter User MANUAL, which can be dowloaded from 
#http://www.di.unito.it/~tutreeb/TUTtoPENN/TUTtoPENNconverter-manual.zip), describes 
#the organization and use of the converter.
#
#This is the FOURTH MODULE, i.e. the module that is called to convert the 
#the strutures of the AUGmented-Penn into Penn structures.
#The call of this module is done by the MAIN module after the preprocessing (FIRST module), 
#the constuttifier (SECOND module) and the augpennifier (THIRD module) if the parameter is -p.

#Il programma PENNIFIER  prende un albero di costituenti di tipo AUGMENTED-PENN in notazione
#ad albero (dove ogni nodo e' seguito da una coppia di parentesi quadre che includono l'indice posizionale
#del padre del nodo stesso), lo trasforma in un albero di costituenti di tipo PENN.
#Esso pertanto prende in input l'output prodotto da aug-pennifier (che a sua volta lavora sull'output del
#constuttifier).

#The main comments are in English, the minor in Italian

package package_penn;
BEGIN{}

### main_penn #############################################################
#makes the call of the procedures that manage the conversion in Penn of the sentences in the 
#AUGmented-Penn format

sub main_penn{
	print "in pennifier\n";
	$contarighe = 0; 		#contiene il numero righe del file ancora da leggere
	@frase_pnn = ();		#conterra' l'albero augpen
	$frase_pnn = '';
	$file_frasi_penn = '';
	$titolo = '';
	$contarighe = scalar(@_);	#numero di righe del file di input 
	@tobedeleted = ();				#conterra' le relazioni da cancellare
	while ($contarighe > 0){        #fino a quando ci sono righe di input
			$riga = shift(@_);
			@frase_penn = ();
			$contarighe = --$contarighe;
			while ($riga !~ m/^\s/){		#se la riga non inizia con carattere vuoto
					$contarighe = --$contarighe;
					if ($riga =~ m/\*\*\*\*/){       	#se la riga inizia con *
							$titolo = $riga;     			#assegna riga a $titolo
					}
					else{if ($riga =~ m/^\(/){        #se la riga inizia con parentesi aperta
								$riga =~ s/PREDCOMPL\+SUBJ/PREDCOMPLSUB/g;
								$riga =~ s/PREDCOMPL\+OBJ/PREDCOMPLOB/g;	
								$riga =~ s/RMODPRED\+SUBJ/MODPRED\+SUBJ/g;
								$riga =~ s/RMODPRED\+OBJ/MODPRED\+OBJ/g;
								push(@frase_penn,$riga);	#la mette nell'array
	     	    		   }
					}
					$riga = shift(@_);
			}	
			$file_frasi_penn = $file_frasi_penn."\n".$titolo.pennify(\@frase_penn,\@tobedeleted,\@tobechanged);
			print $titolo,"\n";
			@tobedeleted = removeduplicatefrom(@tobedeleted);
			@tobechanged = removeduplicatefrom(@tobechanged);
			$frase_penn = '';
	}
	@tobedeleted = sort{$b cmp $a}@tobedeleted;
	foreach $item(@tobedeleted){
			$item = '\-'.$item; 
			$item =~ s/\+/\\\+/;
			$file_frasi_penn =~ s/$item//g;
	}
	push(@tobechanged,'SUBJ>SBJ');
	foreach $item(@tobechanged){
			$penntag = substr $item,(index $item,'>')+1;
			$tuttag = substr $item,0,(index $item,'>');
			if ($tuttag =~ m/\+/){
					$tuttag =~ s/\+/\\\+/g;
			}
			if ($tuttag =~ m/\-/){
					$tuttag =~ s/\-/\\\-/g;
			}
			if ($tuttag ne 'SUBJ'){
					$file_frasi_penn =~ s/\-$tuttag/\-$penntag/g;
			}
			else{$file_frasi_penn =~ s/$tuttag/\-$penntag/g;
			}
	}
	$file_frasi_penn =~ s/\*LOCUT//g;
	$file_frasi_penn =~ s/^\n//;
	$file_frasi_penn =~ s/\-\-/\-/g;
	return $file_frasi_penn."\n";
}



### PENNIFY ###########################################################
#removes from the sentences all the grammatical relations not accepted in Penn

sub pennify{
	my ($ref1,$ref2) = @_;  				#dereferenziazione degli array argomenti
   	my @frase_input = @{$ref1};        #l'albero di input
   	@tobedeleted = @{$ref2};   		#le relazioni da cancellare
	my @frase_penn = ();
	my $trovato = 0;
	my $relfunc = '';
	my $outtable = '';
	while (scalar(@frase_input) > 0){	#mette le righe della frase-albero in un array in cui le modifica
			$riga = shift(@frase_input);
			push (@frase_penn,$riga);
	}
	@penn_table = ();  			#crea l'array per la tabella delle label funzionali di Penn
	open(FILEHANDLEarg, '<TUTtoPENN1_0-penn-tag-table.txt');		#legge il file che contiene la table
	while (<FILEHANDLEarg>){ 							#e ne mette tutto il contenuto nell'array
			$ccc = $_;
			$ccc =~ s/\s//g;
   			push (@penn_table,$ccc);
	}
	close (FILEHANDLEarg);
	foreach $it(@frase_penn){		#per ogni nodo della frase-albero
			@relfuncs = ();
			$relfunc = substr $it, 1, (index $it, ' ');		#estrae la stringa sintagma-relazione funzionale
			if ($relfunc =~ m/\@/){
					$relfunc = substr $relfunc,0,(index $relfunc,'@');
			}
			if ($relfunc =~ m/\-/){
					$singlerelfunc = substr $relfunc,(rindex $relfunc,'-') +1;		#estrae l'ultima relazione funzionale
					push (@relfuncs,$singlerelfunc);		#la mette in un array
					$singlerelfunc = '-'.$singlerelfunc;
					$singlerelfunc =~ s/\s//;
					$relfunc =~ s/$singlerelfunc//; #substr $relfunc,(index $relfunc,'-')+1;				#eventuale rimanente relazione funzionale
					while ($relfunc =~ m/\-/){
							$singlerelfunc = substr $relfunc,(rindex $relfunc,'-') +1;		#gestisce il caso di relazioni funzionali multiple
							push (@relfuncs,$singlerelfunc);										
							$relfunc = substr $relfunc,(index $relfunc,'-')+1;
					}
			}
			if (scalar(@relfuncs) >0){
					foreach $rel(@relfuncs){
							$rel =~ s/\s//g;
							$outtable = '';
							foreach $item(@penn_table){
									$tuttag_in_penntable = substr $item,0,(index $item,'>');
									if ($rel eq $tuttag_in_penntable){			#se la relazione appartiene al tagset di Penn 
											$outtable = $item;	    #valorizzo la var outtable
									}
							}
							if ($outtable eq ''){		#se la relazione non appartiene al tagset di Penn
									push(@tobedeleted,$rel);
							}
							else{push(@tobechanged,$outtable);
							}
					}
			}
		}
	return join("",@frase_penn);
}


### figliconstit ###########################################################
#finds the children of a node within a constituency tree

sub figliconstit{
	my $nodo = shift(@_);
	my @albero = @_;
	my @array = ();
	my $podelpadre = '';
	my $posnodo = pos_in_ttree($nodo);
	foreach $it(@albero){
			$posdelpadre = padre_in_quadre($it);
			if ($posdelpadre eq $posnodo){
					push(@array,$it);
			}
	}
	return @array;
}


### fratelli ###########################################################
#given a node and a tree, finds all the sister nodes in the tree

#trova i nodi che hanno lo stesso padre del primo argomento, nell'albero secondo argomento
sub fratelli{
	my $nodo = shift(@_);
	my @albero = @_;
	my @array = ();
	foreach $it(@albero){
		if ((padre_in_quadre($it) eq padre_in_quadre($nodo)) and ($it ne $nodo)){
			push(@array,$it);
		}
	}
return @array;
}


### padre_in_quadre ###########################################################
#finds the father of a node argument
 
 sub padre_in_quadre{
	my $nodo = shift(@_);
	my $padre = substr $nodo,0,(rindex $nodo,']');
	$padre = substr $padre,((rindex $padre,'[') + 1);
	return $padre;
}


### pos_in_ttree ###########################################################
#finds the position of a node within a constituency tree

sub pos_in_ttree{
	my $nodo = shift(@_);
	my $posinttree = substr $nodo,((index $nodo,' ')+1);
	$posinttree = substr $posinttree, 0,(index $posinttree,'/');
	return $posinttree;
}




#### sostituisciinarray ##################################################################################
#given A and B and and array, substitues B with A in the array (if occurs)

sub sostituisciinarray {
	my $vecchio = shift(@_);
	my $nuovo = shift(@_);
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elemento(@array){
		if ($elemento ne $vecchio){
			push (@arraytemp1,$elemento);
		}
		else {push (@arraytemp1,$nuovo);
		}
	}
	@array = @arraytemp1;
return @array;
}



#### cancelladaarray ##################################################################################
#given and argument and an array, removes the argument from the array

sub cancelladaarray {
	my $elemento = shift(@_);
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elem(@array){
		if ($elem ne $elemento){
			push (@arraytemp1,$elem);
		}
	}
	@array = @arraytemp1;
return @array;
}


#### removeduplicatefrom ##################################################################################
#given and argument and an array, removes the duplication of the argument from the array

sub removeduplicatefrom{
	my @array = @_;
	@array = sort(@array);
	my @finalarray = ();
	my $elem1 = shift(@array);
	my $elem2 = shift(@array);
	while (scalar(@array) >0){
			if ($elem1 eq $elem2){
					$elem1 = $elem2;
					$elem2 = shift(@array);
		    }
		    else{push(@finalarray,$elem1);
					$elem1 = $elem2;
					$elem2 = shift(@array);
		    }
	}
	if ($elem1 ne $elem2){
			push(@finalarray,$elem1);
			push(@finalarray,$elem2);
	}
	else{push(@finalarray,$elem1);
	}
return @finalarray;
}

return 1;
END {}

