﻿#/*  #Copyright 2011 Cristina Bosco [bosco] AT di DOT unito DOT it

    #This file is part of TUTtoPENN converter rel 1.0.

    #TUTtoPENN converter is free software: you can redistribute it and/or modify
    #it under the terms of the GNU General Public License as published by
    #the Free Software Foundation, either version 3 of the License, or
    #(at your option) any later version.

    #TUTtoPENN converter is distributed in the hope that it will be useful,
    #but WITHOUT ANY WARRANTY; without even the implied warranty of
    #MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    #GNU General Public License for more details.

    #You should have received a copy of the GNU General Public License
    #along with TUTtoPENN converter. If not, see <http://www.gnu.org/licenses/>.
#*/


#This file is part of the TUTtoPENN converter, which implements a conversion from TUT 
#native format to Penn Treebank format adapted to Italian (called TUT-Penn).
#The TUTtoPENN converter User MANUAL, which can be dowloaded from 
#http://www.di.unito.it/~tutreeb/TUTtoPENN/TUTtoPENNconverter-manual.zip), describes 
#the organization and use of the converter.
#
#This is the FIRST MODULE, i.e. the module that is called to convert the PoS tagging of TUT
#into a Penn-Style PoS tagging. Therefore, the input of this module is in native TUT format.
#The call of this module is done by the MAIN module as a preprocessing before all the other 
#modules (parameter = -c/-a/-p) or alone (no parameter). 
#The output of this module are sentences in form of array and in CONStituency-TUT format, 
#and is in the file called 'outconstut.out'.is in 
#the file called 'outpreproc.out'.

#The main comments are in English, the minor in Italian


package package_preproc;
BEGIN {}

######## main_preproc #####################################################################
#manages the call of the rest of the module on the single sentences included in the input file
#	While end of file is false
#		while the line is not a title of sentence
#			read the input line
#				if the line is not empty, call buildinfoword and put the result in the sentence array 
#      When the line is empty (=end of sentence) call addindex on the sentence
#		Print the sentence
#	Make the sentence array empty for the next sentence

sub main_preproc{
print "in preprocessor\n";

@arrayfrase = ();       	#inizializza l'array che conterra le parole della frase,
$finefrase = 0;         		#la var di fine frase e
$contit = 0;            		#la var che segnala titolo trovato
$filefrasi = '';
while (<>){         				#fino a quando ci sono righe di input
	$test = 0;           	       		#inizializza la variabile di uscita $test
	while ($test == 0){			#fino quando test vale 0
		$riga = $_;			                        #legge la riga di input
		$stringa = sprintf "%s", $riga;     	        #la converte in stringa
		$stringa = senzacommento($stringa);     #elimina l eventuale parte commentata
		$annotata = (rindex $stringa, ']') - (rindex $stringa, '[');   #calcola la distanza tra le ultime [ ] (>2 se la riga annotata)
		$inizio = substr $stringa, 0,5;      	        #legge i primi 5 caratteri
		if (($inizio =~ m/\d/) and ($annotata >2) and ($stringa !~ m/MARKER/)){  #se la riga inizia con numero, annotata e non locuzione rigida
	       	if (relazcolpadre($stringa) !~ m/CONTIN\+LOCUT/){    	             #(per le locuzioni rigide tiene solo la prima parola)
	       		push (@arrayfrase,buildinfoword($stringa,@arrayfrase)); 		            #richiama buildinfoword e ne mette il risultato in arrayfrase
			}
		}
		else {if ($inizio !~ m/\*/){ 		           #altrimenti se riga vuota
					if ($contit == 0){  	           #se non ancora  trovato il titolo (prima frase)
						$contit = ++$contit; 	   #aggiorna la variabile $contit
						$filefrasi = $filefrasi.$titolo;
						print $titolo,"\n";
					}
	      			$finefrase = ++$finefrase;    	        #aggiorna la var che segnala la fine della frase
	      		}
				if ($inizio =~ m/\*/){            		#se la riga e' di **
					$titolo = $stringa;     	      	#assegna il titolo della nuova frase a $titolo
					if ($finefrase != 0){            	#se raggiunta fine della frase precedente
						$filefrasi = $filefrasi.adaptsentence(@arrayfrase)."\n\n".$titolo;	#richiama adaptsentence sulla frase
						print $titolo,"\n";
						@arrayfrase = ();			 	#svuota arrayfrase per la nuova frase
						$finefrase = 0;				#riporta la var di finefrase a 0
					}
	      		}
		}
		$test = ++$test;     			#aggiorna la var di uscita
	}
}
$filefrasi = $filefrasi.adaptsentence(@arrayfrase)."\n\n"; 	#richiama adaptsentence sull'ultima frase
return $filefrasi;
}



######################## ADAPTSENTENCE #####################################################
#call the subroutines that add the traces for the extraposed (on the right of the verb) subjects, for intraposed modifiers (modifiers 
#between the head and its argument/s)

sub adaptsentence {
	my @frase = @_;
	@frase = tracerightsubj(@frase);		#aggiunge le tracce per i soggetti che si trovano a destra del verb
	@frasetemp = @frase;
	while (scalar(@frasetemp) > 0){
		$it = shift(@frasetemp);
		if ((rel($it) =~ m/VERB-SUBJ/) and ($it !~ m/\*\[/) and ($it !~ m/PRED/)){
			@frase = traceintraposedsubj($it,@frase);
		}
	}#aggiunge le tracce per i soggetti che si trovano a sinistra del verbo preceduti da mod o arg
	@frase = addindex(@frase);   		#crea i riferimenti per i coindici
	@frase = traceintraposedmodifier(@frase);	#aggiunge le tracce per i modificatori che si trovano tra una testa ed i suoi complementi
	return join("\n",@frase);
}


###### tracerightsubj ##############################################################################
#add the traces for the subjects which are on the right of the verb

sub tracerightsubj{
	my @frase = @_;
	my $posizio = '';
	my $newtrace = '';
	my $datracciare = '';
	foreach $elem(@frase){   	#per ogni stringa dell'array frase
		$posizio = substr $elem,0,(index $elem,'/');	#se e' un soggetto di un verbo e si trova alla destra del verbo stesso
		if ((rel($elem) =~ m/VERB-SUBJ/) and ($elem !~ m/VERB-INDCOMPL-AGENT/) and	
		    ($elem !~ m/\*\[/) and (pospadre($elem) <= $posizio)){
				foreach $figlio(figli(padre($elem,@frase)),@frase){
					if ((substr $figlio,0,(index $figlio,'/')) < $posizio){
						$datracciare = 'si';
					}
				}
				if ((pospadre($elem) !~ m/\./) or ($datracciare eq 'si')){
					$newtrace = substr $elem,((index $elem,'/') + 1);		#crea una traccia
					$newtrace = substr $newtrace,(index $newtrace,'/');	#condicizzata con il soggetto
					if ($datracciare ne 'si'){
						if (pospadre($elem) ne ''){
							$newtrace = pospadre($elem).'.15/*[f-'.$posizio.']'.$newtrace;
						}
						else{$newtrace = '0.15/*[f-'.$posizio.']'.$newtrace;
						}
					}
					else{$posipadre = pospadre($elem);
						if ($posipadre =~ m/\./){
							$posipadre = substr $posipadre,0,(index $posipadre,'.');
						}
						if ($posipadre ne ''){
							$newtrace = $posipadre.'.15/*[f-'.$posizio.']'.$newtrace;
						}
						else{$indice = 15+$posizio;
							$newtrace = '0.'.$indice.'/*[f-'.$posizio.']'.$newtrace;
						}
					}
					push(@frase,$newtrace);
					$newelem = $elem;
					$newelem =~ s/VERB-SUBJ/VERB-EXTRAPOSEDSUBJ/;
					@frase = sostituisciinarray($elem,$newelem,@frase);
				}	
		}
	}
	@frase = sort {compar($a,$b)}@frase;
	return @frase;
}


####### addindex ###################################################################################
#If the line is a trace, find the coindexed element and mark it with * and the position of the coindexed element

sub addindex{
	my @frase = @_;
	my @coindexes = ();
	my $provvisoria = '';
	my $posizione = '';
	foreach $elem(@frase){   		#per ogni stringa dell'array frase
		if (($elem =~ m/\*\[/) and ($elem !~ m/\[\]/)){  	 #se e' una traccia coindicizzata
			$index = substr $elem, ((index $elem, '-') + 1), 7;  #ne calcola il coindice
			$index = substr $index, 0, ((index $index, '/') -1);
			push (@coindexes, $index);		#e lo mette nell'array dei coindici della frase
		}
	}
	foreach $elem(@frase){
		foreach $item(@coindexes){        	#per ogni coindice cerca la parola coindicizzata
			if ((substr $elem, 0, (index $elem, '/')) eq $item){ 	 #e attacca alla parola la posizione dell'elemento coindicizzato
				if ($elem !~ m/^$posizione\/\*$posizione/){
					$provvisoria = substr $elem, 0, ((index $elem, '/') + 1);
					$posizione = substr $elem, 0, (index $elem, '/');
					$elem = substr $elem, ((index $elem, '/') +1);
					$elem = $provvisoria.'*'.$posizione.'-'.$elem;		#per renderla riconoscibile come coindice
				}
			}
		}
	}
	return @frase;
}


### traceintraposedsubj #########################################################
#add the traces for each subject which is on the left of the verb and has a modifier between itself and the verb

sub traceintraposedsubj{
	my $subj = shift(@_);
	my @frase = @_;
	my @fratellisubj = ();
	my $padresubj = '';
	if (pospadre($subj) > position_in_tree($subj)){	#se e' soggetto a sinistra del verbo
			$padresubj = padre($subj,@frase);	#trova il padre
	}
	@fratellisubj = ();
	@figlipadresubj = ();
	@figlipadresubj = figli($padresubj,@frase);
	foreach $frat(@figlipadresubj){			
		if (compar($frat,$subj) < 0){ 
			push(@fratellisubj,$frat);
		}		
	}
	if (scalar(@fratellisubj) > 0){
		$newtracesubj = substr $subj,((index $subj,'/') + 1);		#crea una traccia
		$newtracesubj = substr $newtracesubj,(index $newtracesubj,'/');	#condicizzata con il soggetto
		$newtracesubj = pospadre($subj).'.17/*[f-'.(position_in_tree($subj)).']'.$newtracesubj; #usa .17 per non duplicare indici
		push(@frase,$newtracesubj);
		$newsubj = $subj;
		$newsubj =~ s/VERB-SUBJ/VERB-EXTRAPOSEDSUBJ/;
		@frase = sostituisciinarray($subj,$newsubj,@frase);		
	}	
	@frase = sort {compar($a,$b)}@frase;
	return @frase;
}



###### traceintraposedmodifier ##############################################################################
#add the traces for the modifiers positioned between their head and a complement of the same head
#(ex. "la reazione velenosa di Valona" where "velenosa" is modifier and "di Valona" complement)

sub traceintraposedmodifier{
my @frase = @_;
my $indice = '16';
foreach $elem(@frase){
	if ($elem !~ m/^UP/){
		@figli = figli($elem,@frase);      #calcola i figli di elem ordinati in un array figli
		@array_des_modifiers = ();
		@array_des_arguments = ();
		@array_sin_modifiers = ();
		@array_sin_arguments = ();
		if (scalar(@figli) > 0){
			foreach $figlio(@figli){             #per ogni figlio di @figli
				if ((($figlio =~ m/NOUN\-SUBJ/) or ($figlio =~ m/AGENT/) or ($figlio !~ m/\-SUBJ/)) 
					 and ($figlio !~ m/PUNCT/) and (($figlio !~ m/CONJ/) or ($figlio =~ m/CONJTEXT/))){ #and ($figlio !~ m/AUX/)){#se non e' punto o ausiliare
					$relaz = dependency_type($figlio); #calcola il tipo di dipendenza (=arg/mod/coord) che lo lega a $elem
					if (($relaz eq 'MOD') and ($figlio !~ m/AUX/)){	# poi conta quanti mod compaiono come figli di $elem a destra e sinistra
						if (position_in_tree($figlio) > position_in_tree($elem)){ #separa i mod di des e di sin e
							push(@array_des_modifiers,$figlio);	#per metterli negli arraymodifiers adatti
						}
						if (position_in_tree($figlio) < position_in_tree($elem)){
							push(@array_sin_modifiers,$figlio);
						}
					}
					else{if (($relaz eq 'ARG') or ($figlio =~ m/AUX/)){ 	#separa gli argomenti di des e di sin
							if (position_in_tree($figlio) > position_in_tree($elem)){
								push(@array_des_arguments,$figlio);	#per metterli negli arrayarguments adatti
							}
							if (position_in_tree($figlio) < position_in_tree($elem)){
								push(@array_sin_arguments,$figlio);
							}
						}
					}
				}
			}
		}
		if ((scalar(@array_des_modifiers) > 0) and (scalar(@array_des_arguments) >0)){ #se $elem ha almeno un mod ed un arg dx
			foreach $modf(@array_des_modifiers){	#allora si deve verificare se il mod precede l'arg, perche' in tal caso occorretracciare
				$occorretracciare_des = 'no';
				foreach $argm(@array_des_arguments){
					if (position_in_tree($modf) < position_in_tree($argm)){
						$occorretracciare_des = 'si';
					}
				}
				if ($occorretracciare_des eq 'si'){#se occorretracciare si mette l'indice di coreferenza nel mod
					$posmodf = position_in_tree($modf);#e si sostituisce la relazione con INTRAMOD (specie di visitor)
					$newmodf = 'UP'.$posmodf.'/*'.$posmodf.'-'.(substr $modf,(index $modf,'/') + 1);
					$newmodf = (substr $newmodf,0,(rindex $newmodf,'/') + 1).'INTRAMD';
					$newtrace = substr $modf,((index $modf,'/') + 1);	# e poi si crea una traccia
					$newtrace = substr $newtrace,(index $newtrace,'/');	#condicizzata con il mod appena coindicizzato
					foreach $ele(@array_des_arguments){#ora occorre calcolare la posizione in cui inserire la traccia creata
						$trovato = 'no';
						@subtree_des = subtree($ele,$trovato,@frase);#pertanto si prendono gli arg des di elem e tutti i loro
						foreach $rr(@subtree_des){#eventuali sottoalberi
							if (($rr ne $ele) and ($rr !~ m/^yy/)){
								push(@array_des_arguments,'yy'.$rr);
							}
						}
					}
					foreach $tt(@array_des_arguments){
						if ($tt =~ m/^yy/){
							$newtt = substr $tt,2;
							@array_des_arguments = sostituisciinarray($tt,$newtt,@array_des_arguments);
						}
					}
					if (scalar(@array_des_arguments) > 1){#poi si cerca l'elemento maggiore tra gli arg des di elem
						foreach $ele(@array_des_arguments){#e gli elementi appartenenti ai loro sottoalberi
							foreach $elle(@array_des_arguments){
								if (position_in_tree($ele) > position_in_tree($elle)){
									$maxargom = $ele;
								}
							}
						}
					}
					else{
					$maxargom = @array_des_arguments[0];
					}
					$posmaxargom = position_in_tree($maxargom);#trovato l'elemento maggiore se ne calcola la posizione
					if ($posmaxargom =~ m/\*\[/){
						$posmaxargom = substr $posmaxargom,0,((index $posmaxargom,'.') - 1);
					}
					if ($posmaxargom =~ m/\./){
						$posmaxargom = substr $posmaxargom,0,(index $posmaxargom,'.');
					}
					foreach $it(@frase){
							if ((substr $it,0,(index $it,'/')) =~ m/$posmaxargom.$indice/){
									$indice =++$indice;
							}
						}
					$newtrace = 'UP'.$posmaxargom.'.'.$indice.'/*[f-'.$posmodf.']'.$newtrace; 
					push(@frase,$newtrace);							
					@frase = sostituisciinarray($modf,$newmodf,@frase);
				}
			}
		}
		if ((scalar(@array_sin_modifiers) > 0) and (scalar(@array_sin_arguments) >0)){ #se $elem ha almeno un mod ed un arg sx
			foreach $modf(@array_sin_modifiers){
					$occorretracciare_sin = 'no';
					foreach $argm(@array_sin_arguments){#si verifica se qualche mod si trova dopo arg
						if (position_in_tree($modf) > position_in_tree($argm)){#perche' in tal cso occorretracciare
							$occorretracciare_sin = 'si';
						}
					}
					if ($occorretracciare_sin eq 'si'){ #se occorretracciare si mette l'indice di coreferenza nel mod
						$posmodf = position_in_tree($modf);#e si sostituisce la relazione con INTRAMD
						$newmodf = 'UP'.$posmodf.'/*'.$posmodf.'-'.(substr $modf,(index $modf,'/') + 1);
						$newmodf = (substr $newmodf,0,(rindex $newmodf,'/') + 1).'INTRAMD';
						$newtrace = substr $modf,((index $modf,'/') + 1);	#poi crea una traccia
						$newtrace = substr $newtrace,(index $newtrace,'/');#condicizzata con il mod
						foreach $ele(@array_sin_arguments){#ora occorre calcolare la posizione in cui inserire la traccia creata
							$trovato = 'no';
							@subtree_sin = subtree($ele,$trovato,@frase);#pertanto si prendono gli arg sin di elem e tutti i loro
							foreach $rr(@subtree_sin){#eventuali sottoalberi
								if (($rr ne $ele) and ($rr !~ m/^zz/)){
									push(@array_sin_arguments,'zz'.$rr);
								}
							}
						}
						foreach $zz(@array_sin_arguments){
							if ($zz =~ m/^zz/){
								$newzz = substr $zz,2;
								@array_sin_arguments = sostituisciinarray($zz,$newzz,@array_sin_arguments);
							}
						}
						if (scalar(@array_sin_arguments) > 1){#poi si cerca l'elemento minore tra gli arg sin di elem
							foreach $el(@array_sin_arguments){#e gli elementi appartenenti ai loro sottoalberi
								foreach $ell(@array_sin_arguments){
									if (position_in_tree($el) < position_in_tree($ell)){
										$minargom = $el;
									}
								}
							}
						}
						else{$minargom = @array_sin_arguments[0];
						}
						$posminargom = (position_in_tree($minargom) -1);
						if ($posminargom =~ m/\*\[/){
							$posminargom = substr $posminargom,0,((index $posminargom,'.') - 1);
						}
						if ($posminargom =~ m/\./){
							$posminargom = substr $posminargom,0,(index $posminargom,'.');
						}
						foreach $it(@frase){
							if ((substr $it,0,(index $it,'/')) =~ m/$posminargom.$indice/){
									$indice = ++$indice;
							}
						}
						$newtrace = 'UP'.$posminargom.'.'.$indice.'/*[f-'.$posmodf.']'.$newtrace;
						push(@frase,$newtrace);
						@frase = sostituisciinarray($modf,$newmodf,@frase);
					}
				}
			}
		}
	}

foreach $tok(@frase){
	if ($tok =~ m/^UP/){
			$newtok = substr $tok,2;
			@frase = sostituisciinarray($tok,$newtok,@frase);
	}
}



@frase = sort {compar($a,$b)}@frase;
return @frase;
}


###### subtree #######################################################################################
#output the subtree of the argument node

sub subtree{
	my $nodo = $_[0];
	$trovato = $_[1];   #$trovato `e inizializzata all'esterno
	my @albero = @_;
	if ($trovato eq 'no'){  #solo quando si entra in subtree da una chiamata esterna
		@subtree = ();  #l'array del subtree deve essere vuoto, ma deve non deve essere
	}                       #svuotato nelle chiamate ricorsive
	$trovato = 'si';
	if (scalar(figli($nodo,@albero)) > 0){     #se il nodo ha figli
		push(@subtree,$nodo);              #viene messo nell'array subtree e la routine
		foreach $figlio(figli($nodo,@albero)){     #viene richiamata su ogni suo figlio
			subtree($figlio,$trovato,@albero);
		}
	}
	else{push(@subtree,$nodo);                 #se invece il nodo non ha figli, viene messo nell'array
	}
return @subtree;
}


###### dependency_type #####################################################################################
#reading the relation table, output the kind of the relation of the argument (modifier or argument) or a warning message when the relation is missing 

sub dependency_type {
	my $arg = shift(@_);
	chomp $arg;
	my $relaz_arg = rel($arg);
	chomp($relaz_arg);
	my @relaz_table = ();
	my $type = '';
	open(FILEHANDLErel, '<TUTtoPENN1_0-rel-table.txt');  #legge la tabella delle relazioni, dove e' specificato per
	while (<FILEHANDLErel>){            #ogni rel se e' arg o mod, e mette i dati in un array
		$ccc = $_;
		push (@relaz_table,$ccc);}
	close(FILEHANDLErel);
	my $relaz = shift(@relaz_table);
	my $name_relaz = substr $relaz, 0, (rindex $relaz, '>');
	chomp($name_relaz);
	my $type_relaz = substr $relaz, ((rindex $relaz, '>') + 1);
	chomp($type_relaz);
	my $trovato = 0;   
	while (($trovato == 0) and (scalar(@relaz_table) > 0)){
	       	if($relaz_arg eq $name_relaz){
			$trovato = ++$trovato;
			$type = $type_relaz;
		}
		else {$relaz = shift(@relaz_table);
			$name_relaz = substr $relaz, 0, (rindex $relaz, '>');
			chomp($name_relaz);
			$type_relaz = substr $relaz, ((rindex $relaz, '>') + 1);
			chomp($type_relaz);
}
	}
if ($trovato == 0){print "in preprocessor (linea 467): ",$relaz_arg," is a relation missing in TUTtoPENN1_0-rel-table.txt\n";}
return $type;
}


###### buildinfoword ###############################################################################
#extracts from the line of the TUT file te information useful for building the corresponding Penn-like PoS tagging separated by /.
#Ex.; 1/la/ART~D/3/VERB-SUBJ
#For locutions only the head word is taken into account in the new format.

sub buildinfoword {
	my $arg = shift(@_);
	my $frase = @_;
	my $infoword = relazcolpadre($arg);
	if ($arg =~ m/LOCUTION/){		#se e' una locuzione rigida tiene solo il lemma della testa
		$infoword = posizione($arg)."/".lemma($arg)."/".morfocategoria($arg,@frase)."/".posizionepadre($arg)."/".$infoword;
	}						#altrimenti tiene la parola
	else{$infoword = posizione($arg)."/".parola($arg)."/".morfocategoria($arg,@frase)."/".posizionepadre($arg)."/".$infoword;
	}
	return $infoword;
}


###### posizione ####################################################################################
#given a line in tut format, output the position of the word in the sentence

sub posizione {
my $arg = shift(@_);
	my $posizione = 0;
	$primospazio = substr $arg, 0, 2;
	if ($primospazio =~ m/\s/){
		$posizione = substr $arg, 0,1;}
	else {$primospazio = substr $arg, 0,3;
		if ($primospazio =~ m/\s/){
			$posizione = substr $arg, 0,2;}
		else{$primospazio = substr $arg, 0,4;
			if ($primospazio =~ m/\s/){
				$posizione = substr $arg, 0,3;}
			else{$primospazio = substr $arg, 0,5;
				if ($primospazio =~ m/\s/){
					$posizione = substr $arg, 0,4;}
				else{$posizione = substr $arg, 0,5;}}}}
}


###### parola ######################################################################################
#given a line in tut format, output the word

sub parola {
	my $arg = shift(@_);
	my $parola = '';
	my $posopencoindex = '';
	my $posclosecoindex = '';
	my $coindex = '';
	my $kindofcoindex = '';
	my $posopentonda = '';
	my $primospazio = '';
	my $inizio = '';
	if (($arg =~ m/PUNCT/) and ($arg =~ m/\s\(\s\(\#/)){    	#se ? una parentesi tonda aperta
		$parola = '(';
	}
	else{if (($arg =~ m/PUNCT/) and ($arg =~ m/\s\)\s\(\#/)){   #se ? una parentesi tonda chiusa
			$parola = ')';
			}
			else{if ($arg =~ m/\st\s/){       				#se e' una traccia
						$posopencoindex = index $arg, '[';	#trova la coppia di quadre che include l'eventuale
						$posclosecoindex = index $arg, ']';	#coindice e il tipo di traccia (i.e. full, word, partial)
						if (($posclosecoindex - $posopencoindex) > 2){    #se e' coindicizzata trova il coindice
							$coindex = substr $arg, ($posopencoindex + 1), (($posclosecoindex - 2) - $posopencoindex);
							$kindofcoindex = substr $arg, (($posopencoindex +1) + length($coindex)),1;#e il tipo del coindice
							$parola =  '*['.$kindofcoindex.'-'.$coindex.']';    #mette la traccia coindicizzata nel formato prelimiare alla
						}				# traduzione in CONS-TUT (dove il tipo sparisce in favore dell'indice sull'elemento coindicizzato)
						else {$parola = '*[]';	#se non ? coindicizzata mette solo le quadre
					}
			}
			else{$posopentonda = index $arg, '(';		#se ? una parola, la trova facendo riferimento alle tonde e agli spazi
				my $inizio = 0;					#tipici della rappresentazione delle parole in TUT
				$primospazio = substr $arg, 0, 2;
				if ($primospazio =~ m/\s/){
					$inizio = 2;
				}
				else {$primospazio = substr $arg, 0,3;
					if ($primospazio =~ m/\s/){
						$inizio = 3;
					}
					else{$primospazio = substr $arg, 0,4;
						if ($primospazio =~ m/\s/){
							$inizio = 4;
						}
						else{$primospazio = substr $arg, 0,5;
							if ($primospazio =~ m/\s/){
								$inizio = 5;
							}
							else{$inizio = 6;
							}
						}
					}
				}
				$parola = substr $arg, $inizio, ($posopentonda - $inizio -1);
				if ($parola =~ m/^\s/){
					$parola = substr $parola,1;
				}
				if ($parola eq '/'){
					$parola = '#/#';
				}
			}
		}
	}
	return $parola;
}


###### lemma  #######################################################################################3
#given a line in tut format, output the lemma

sub lemma {
	my $arg = shift(@_);
	my $lemma = substr $arg, (index $arg,'('), ((index $arg, ')') + 1); #riduce la stringa a quello che sta tra le 2 parentesi del POS tagging
	my $posprimospazio = (index $lemma,' ') - 1; #trova il primo spazio dentro $lemma
	$lemma = substr $lemma,1, $posprimospazio; #trova il lemma come primo elemento del POS tagging
	return $lemma;
}


###### morfocategoria #######################################################################################
#given a line in tut format, output the morphological category

sub morfocategoria {
	my $arg = shift(@_);
	my @frase = @_;
	my $posclosetonda = '';
	my $posopentonda = '';
	my $nontrovato = 0;
	my $categoria = '';
	my $stringa = '';
	if (($arg =~ m/SPECIAL/) and
		(($arg =~ m/\s\%\s\(\#/) or ($arg =~ m/\s\/\s\(\#/) or ($arg =~ m/\sa\s\(\#/) or
		($arg =~ m/\sb\s\(\#/) or ($arg =~ m/\sa\s\(A/))) {
		$categoria = 'SPECIAL';
	}
	if ($arg =~ m/INTERJ/){
		$categoria = 'ADVB';
	}
	if ($arg =~ m/DATE/){
		$categoria = 'DATE';
	}
	if ($arg =~ m/PHRAS/){
		$categoria = 'PHRAS';
	}
	if (($arg =~ m/PUNCT/) and
		(($arg =~ m/\s\)\s\(\#/) or ($arg =~ m/\s\(\s\(\#/) or ($arg =~ m/\s\]\s\(\#/) or ($arg =~ m/\s\[\s\(\#/) or
		($arg =~ m/\s\.\s\(\#/) or ($arg =~ m/\s\;\s\(\#/) or ($arg =~ m/\s\:\s\(\#/) or ($arg =~ m/\s\,\s\(\#/) or
		($arg =~ m/\s\'\s\(\#/) or ($arg =~ m/\s\"\s\(\#/)  or ($arg =~ m/\s\?\s\(\#/) or ($arg =~ m/\s\!\s\(\#/) or
		($arg =~ m/\s\-\s\(\#/) or ($arg =~ m/\s\?\s\(\#/) or ($arg =~ m/\s\?\s\(\#/) or ($arg =~ m/\s\.\.\.\s\(\#/)
 		or ($arg =~ m/\s\´\s\(\#/) or ($arg =~ m/\s\ª\s\(\#/) or ($arg =~ m/\s\«\s\(\#/) or ($arg =~ m/\s\»\s\(\#/))){
			$categoria = 'PUNCT';
	}
	else{if ($arg =~ m/\sVERB\s/){        	#se la categoria e' VERB allora mette le informazioni sul tempo
		if ($arg =~ m/PARTICIPLE PAST/){
			$categoria = 'PA';
		}
		else{if ($arg =~ m/INFINITE/){
				$categoria = 'IN';
		       }
		       else{if ($arg =~ m/GERUND/){
					$categoria = 'GE';
				     }
				     else{if ($arg =~ m/IND PRES/){
							$categoria = 'RE';
						   }
				   			else{if ($arg =~ m/REMPAST/){
									$categoria = 'RA';
					 				}
					 				else{if ($arg =~ m/PARTICIPLE PRES/){
					 						$categoria = 'PE';
					 						}
					 						else{if ($arg =~ m/IMPERF/){
													$categoria = 'IM';
											 		}
											 		else{if ($arg =~ m/FUT/){
															$categoria = 'FU';
						 	      							}
													      else{if (($arg =~ m/IMPER/) and ($arg !~ m/IMPERF/)){
																	$categoria = 'IP';
								     							}
												 	      	      else{if ($arg =~ m/CONDIZ/){
																			$categoria = 'CO';
																	    }
																	     else{if ($arg =~ m/CONG/){
																			       $categoria = 'CG';
																			   }
									    								}
								     							 }
						 	      						  }
											 		}
					 						}
				   					}
				  			}
			    	  }
		        }
		  }
		  if ($arg =~ m/\sAUX\s/){          	 #e sul tipo di verbo (MAIN, MOD o AUX)
				$categoria = 'VAU~'.$categoria;
			}
			else{if ($arg =~ m/\sMAIN\s/){
						$categoria = 'VMA~'.$categoria;
		      		}
		      		else{$categoria = 'VMO~'.$categoria;
		      		}      	#NB: il verbo causativo non viene marcato in TUT
			}
		}
		else{if ($arg =~ m/\sNOUN\s/){
					$categoria = 'NOU';
					if ($arg =~ m/COMMON/){
						$categoria = $categoria.'~C';
					}
					else{if ($arg =~ m/PROPER/){
								$categoria = $categoria.'~PR';
			      		  }
					}
					if (($arg =~ m/SING/) and ($arg !~ m/PROPER/)){
						$categoria = $categoria.'S';
					}
					else{if (($arg =~ m/PL/) and ($arg !~ m/PROPER/)){
								$categoria = $categoria.'P';
			      		  }
			      		 else {if (($arg =~ m/ALLVAL/) and ($arg !~ m/PROPER/)){
			      					$categoria = $categoria.'A';
				     			}
			      		 }
			        }
			  }
	  		  else{if ($arg =~ m/\sPRON\s/){
							$categoria = 'PRO';
		    				if ($arg =~ m/POSS/){
		    	   				$categoria = $categoria.'~PO';
		    	   			}
		    	   			else{if ($arg =~ m/INTERR/){
		    	   						$categoria = $categoria.'~IN';
		    	   					}
		    	   	       			else{if ($arg =~ m/REFL\-IMPERS/){
												$categoria = $categoria.'~RI';
		    	   	       	       			}
				    	   	       	      	else{if ($arg =~ m/DEMONS/){
														$categoria = $categoria.'~DE';
		    			   	       	      	      }
								     			  else{if ($arg =~ m/INDEF/){
															$categoria = $categoria.'~ID';
		    	   	       	    		               }
							   							else{if ($arg =~ m/LOC/){
																	$categoria = $categoria.'~LO';
		    	   	       	      	 							}
										 						else{if ($arg =~ m/PERS/){
																			$categoria = $categoria.'~PE';
		    			   	       	      								}
																		else{if ($arg =~ m/RELAT/){
																					$categoria = $categoria.'~RE';
		    	   	       	      												}
		    	   	       	      												else{if ($arg =~ m/ORDIN/){
		    	   	       	      															$categoria = $categoria.'~OR';
		    	   	       	      				  										}
		    	   	       	      												}
								        								}
								 								}
							  							}
						     					  }
					      					}
									}
							}
		  			}
				 	else{if ($arg =~ m/\sADJ\s/){
								$categoria = 'ADJ~';
								if ($arg =~ m/DEMONS/){
									$categoria = $categoria.'DE';
								}
								else{if ($arg =~ m/DEITT/){
											if ($arg =~ m/ADJC\+DEITT\-RMOD/){
												$categoria = $categoria.'DI';
											}
											else{$categoria = $categoria.'DI2';}
						      			}
										else{if ($arg =~ m/ORDIN/){
													$categoria = $categoria.'OR';
												}
												else{if ($arg =~ m/QUALIF/){
															$categoria = $categoria.'QU';
			      										}
														else{if ($arg =~ m/POSS/){
																	$categoria = $categoria.'PO';
			      												}
																else{if ($arg =~ m/INDEF/){
																			$categoria = $categoria.'IN';
			      														}
																		else{if ($arg =~ m/INTERR/){
																					$categoria = $categoria.'IR';
										      									}
																			    else{if ($arg =~ m/EXCLAM/){
																							$categoria = $categoria.'EX';
											    										}		
										      									}
			      														}
																}
														}
												}
										}
								}
						}
						else{if ($arg =~ m/\sART\s/){
									$categoria = 'ART~';
									if ($arg =~ m/\sDEF\s/){
										$categoria = $categoria.'DE';
									}
									else{$categoria = $categoria.'IN';
									}
							   }
							   else{if ($arg =~ m/\sADV\s/){
											$categoria = 'ADVB';
									}
									else{if ($arg =~ m/\sPREDET\s/){
												if (scalar(figli($arg,@frase)) > 0){
													$categoria = "PRDT2";
												}
												else{$categoria = 'PRDT';
												}
									      }
									      else{if ($arg =~ m/\sPREP\s/){
						      						$categoria = 'PREP';
											     }
											     else{if ($arg =~ m/\sNUM\s/){
							    					 		if ((($arg =~ m/DET/) and ($arg =~ m/ARG/)) or 
												     		    ($arg =~ m/COORD/) or ($arg =~ m/CARDIN/) 
												     		    or ($arg =~ m/APPOSITION/)){
																	$categoria = 'NUMR';
															}
															else{$categoria = 'NUMR2';
														}
								   				 }
											     else{if ($arg =~ m/\sSPECIAL\s/){
													   		$categoria = 'SPEC';
														 }
														 else{if ($arg =~ m/\sCONJ\s/){
															   		$categoria = 'CONJ';
																}
														 }
											   	 }
							     		  }
						        	}
							}
					}
			    }
			}
		}
	}
}
return $categoria;
}


###### posizionepadre ###############################################################################
#given a line in tut format, output the position of the head word

sub posizionepadre {
	my $arg = shift(@_);
	$pospunvirg = rindex $arg, ';';
	$posclosequad = rindex $arg, '[';
	$lenght = $pospunvirg - $posclosequad - 1;
return (substr $arg, ($posclosequad + 1), $lenght);}


###### relazcolpadre ################################################################################
#given a line in tut format, output the dependency relation that link the word to the head

sub relazcolpadre {
	my $arg = shift(@_);
	my $relaz = '';
	$posclosequad = rindex $arg, ']';
	$pospunvirg = rindex $arg, ';';
	$lenght = $posclosequad - $pospunvirg - 1;
	$relaz = substr $arg, ($pospunvirg + 1), $lenght;
	if ($relaz =~ m/\//){
		$relaz = (substr $relaz,0,(rindex $relaz, '/'))."%".(substr $relaz,((rindex $relaz, '/') +1));
	}
	return $relaz;
}


###### figli ########################################################################################
#given a sentence as an array and a node, find the child of the node in the sentence

sub figli{
	my $arg =  $_[0];
	my @frase = @_[1..$#_];
	my $posiz = '';
	my @arrayfigli = ();
	if ($arg =~ m/^tra/){                  #calcola la posizione di $arg
			$posiz = substr $arg, 3, ((index $arg, '/') - 3);  #tenendo conto del fatto che potrebbe
	}                                                          #essere tracciato
	else {$posiz = substr $arg, 0, (index $arg, '/');
	}
	foreach $elem (@frase){                #per ogni elemento della frase
			$padreelem = substr $elem, 0, (rindex $elem, '/');    #calcola la posizione del padre
			$padreelem = substr $padreelem, ((rindex $padreelem, '/') + 1);
			if ($padreelem eq $posiz){     #e la confronta con la posizione di $arg
					push(@arrayfigli,$elem);  #se corrispondono mette l'elemento nell'array dei figli
			}
	}
	return @arrayfigli;
}


######### padre ##################################################################################
#given a line in tut format and a sentence as an array, output the line of the head word of the given line in the sentence

sub padre{
	my $arg =  $_[0];
	my @frase = @_[1..$#_];
	my $pospadre = substr $arg, 0, (rindex $arg, '/');    #calcola la posizione del padre
	$pospadre = substr $pospadre, ((rindex $pospadre, '/') + 1);
	my $padre = shift(@arrayfrase);                       #considera il primo elemento dell'array
	my $trovato = 0;                                      #inizializza la var di uscita dal while
	while ($trovato == 0){ #finche' l'array non e' vuoto e il padre trovato     
			if((substr $padre, 0, (index $padre, '/')) eq $pospadre){
					$trovato = ++$trovato;
			}
			else {$padre = shift(@frase);
			}
	}
	return $padre;
}


######### pospadre ##################################################################################
#given a line in tut format, output the position of the head word

sub pospadre{
	my $arg =  $_[0];
	$pospadre = substr $arg, 0, (rindex $arg, '/');
	$pospadre = substr $pospadre, ((rindex $pospadre, '/') + 1);
return $pospadre;
}


######### rel #######################################################################################
#given a line in tut format, output the dependency relation that link the word to the head

sub rel{
	my $arg =  $_[0];
	$rel = substr $arg, ((rindex $arg, '/') + 1), (length($arg) - (rindex $arg, '/'));
return $rel;
}


######## inpos #######################################################################################
#given a position in the sentence, output the line which is positioned in such a position within the sentence
sub inpos{
	my $arg = shift(@_);
	my @frase = @_;
	my $eleminpos = '';
	foreach $oggetto(@frase){
		if ((substr $oggetto,0,(index $oggetto,'/')) eq $arg){
			$eleminpos = $oggetto;
		}
	}
return $eleminpos;
}


###### senzacommento ################################################################################
#given a line in tut format, output the same line without the comment (if exists)

sub senzacommento {
	my $arg = shift(@_);
	$interrog = index $arg, '???';      #cerca il commento (introdotto da '???') nella stringa
	if ($interrog != -1){                    #se la stringa contiene commento
			$stringard = substr $arg, $interrog;   	     #trova la parte commentata
			$lenrd = (length $arg) - (length $stringard);    #calcola la lunghezza della parte non commentata
			$arg = substr $arg, 0, $lenrd;	     #calcola la stringa senza parte commentata e la assegna a $stringa
	}
	else {$arg = $arg;
	}
}


####################################################################################################
#given A and B and and array, substitues B with A in the array (if occurs)

sub sostituisciinarray {
	my $vecchio = shift(@_);
	my $nuovo = shift(@_);
	my $posvecchio = '';
	if ($vecchio =~ m/^UP/){
			$posvecchio = posizione(substr $vecchio,2);
	}
	else{$posvecchio = posizione($vecchio);
	}
	my @array = @_;
	my @arraytemp1 = ();
	foreach $elemento(@array){
			$poselemento = '';
			if ($elemento =~ m/^UP/){
					$poselemento = posizione(substr $elemento,2);
			}
			else{$poselemento = posizione($elemento);
			}
			if ($poselemento  ne $posvecchio){
					push (@arraytemp1,$elemento);
			}
			else {push (@arraytemp1,$nuovo);
			}
	}
	@array = @arraytemp1;
	return @array;
}


sub compar{return (position_in_tree(shift(@_)) <=> position_in_tree(shift(@_)));
}


sub position_in_tree{
	my $arg =  shift(@_);
	my $posiz = substr $arg, 0, (index $arg,'/');
	if ($posiz =~ m/^UP/){
			$posiz = substr $posiz,2;
	}
	$posiz = substr $posiz,((index $posiz,' ') + 1);
	return $posiz;
}


return 1;
END{}


